@@ -637,6 +637,7 @@ class Doxy2SWIG:
         self.add_text('* * * * *  \n')
     
     def do_includes(self, node):
+        return # omit #include <> tags
         self.add_text('\nC++ includes: ')
         self.subnode_parse(node)
         self.add_text('\n')
@@ -700,7 +701,8 @@ class Doxy2SWIG:
 
         for line in ''.join(self.pieces).splitlines():
             keep_markdown_newline = line[-2:] == '  '
-            w_line = textwrap.wrap(line, width=width, break_long_words=False)
+            present_indent = len(line) - len(line.lstrip())
+            w_line = textwrap.wrap(line, width=width, subsequent_indent = ' ' * present_indent, break_long_words=False)
             if w_line == []:
                 w_line = ['']
             if keep_markdown_newline:
@@ -828,7 +830,7 @@ class Doxy2SWIG:
             for nn in self.get_specific_subnodes(node, 'xrefdescription'):
                 self.subnode_parse(nn, pieces=[''], indent=4)
         else:
-            self.add_text(['**' + title + '**', '\n'])
+            self.add_text(['.. admonition:: ' + title, '\n\n'])
             for nn in self.get_specific_subnodes(node, 'xrefdescription'):
                 self.subnode_parse(nn, pieces=[''], indent=4)
 
