@@ -7,4 +7,4 @@ seq = "GAGUAGUGGAACCAGGCUAUGUUUGUGACUCGCAGACUAACA"
 (ss, mfe) = RNA.fold(seq)
 
 # print output
-print "%s\n%s [ %6.2f ]" % (seq, ss, mfe)
+print("{}\n{} [ {:6.2f} ]".format(seq, ss, mfe))
