@@ -23,10 +23,6 @@
 /**
  *  @addtogroup kl_neighborhood_pf
  *  @{
- *
- *  @brief Compute the partition function and stochastically sample secondary structures for a partitioning of
- *  the secondary structure space according to the base pair distance to two fixed reference structures
- *
  */
 
 #include <ViennaRNA/fold_compound.h>
@@ -46,7 +42,7 @@
  *
  *  @see  vrna_pf_TwoD()
  */
-typedef struct vrna_sol_TwoD_pf_t {
+typedef struct {
   int         k;  /**<  @brief  Distance to first reference */
   int         l;  /**<  @brief  Distance to second reference */
   FLT_OR_DBL  q;  /**<  @brief  partition function */
@@ -69,13 +65,13 @@ typedef struct vrna_sol_TwoD_pf_t {
  * @see vrna_fold_compound_TwoD(), vrna_fold_compound_free(), #vrna_fold_compound
  *      #vrna_sol_TwoD_pf_t
  *
- * @param vc            The datastructure containing all necessary folding attributes and matrices
+ * @param fc            The datastructure containing all necessary folding attributes and matrices
  * @param maxDistance1  The maximum basepair distance to reference1 (may be -1)
  * @param maxDistance2  The maximum basepair distance to reference2 (may be -1)
  * @returns             A list of partition funtions for the corresponding distance classes
  */
 vrna_sol_TwoD_pf_t *
-vrna_pf_TwoD(vrna_fold_compound_t *vc,
+vrna_pf_TwoD(vrna_fold_compound_t *fc,
              int                  maxDistance1,
              int                  maxDistance2);
 
@@ -84,7 +80,6 @@ vrna_pf_TwoD(vrna_fold_compound_t *vc,
 
 /**
  *  @addtogroup kl_neighborhood_stochbt
- *  @brief Contains functions related to stochastic backtracking from a specified distance class
  *  @{
  */
 
@@ -100,13 +95,13 @@ vrna_pf_TwoD(vrna_fold_compound_t *vc,
  *
  *  @see      vrna_pf_TwoD()
  *
- *  @param[inout]  vc The #vrna_fold_compound_t datastructure containing all necessary folding attributes and matrices
+ *  @param[inout]  fc The #vrna_fold_compound_t datastructure containing all necessary folding attributes and matrices
  *  @param[in]  d1    The distance to reference1 (may be -1)
  *  @param[in]  d2    The distance to reference2
  *  @returns    A sampled secondary structure in dot-bracket notation
  */
 char *
-vrna_pbacktrack_TwoD(vrna_fold_compound_t *vc,
+vrna_pbacktrack_TwoD(vrna_fold_compound_t *fc,
                      int                  d1,
                      int                  d2);
 
@@ -119,19 +114,20 @@ vrna_pbacktrack_TwoD(vrna_fold_compound_t *vc,
  * i.e. structures beginning from the 5' end with a specified length of the sequence, are backtracked
  *
  * @note      This function does not work (since it makes no sense) for circular RNA sequences!
+ *
  * @pre       The argument 'vars' must contain precalculated partition function matrices,
  *            i.e. a call to vrna_pf_TwoD() preceding this function is mandatory!
  *
  * @see       vrna_pbacktrack_TwoD(), vrna_pf_TwoD()
  *
- *  @param[inout] vc    The #vrna_fold_compound_t datastructure containing all necessary folding attributes and matrices
+ *  @param[inout] fc    The #vrna_fold_compound_t datastructure containing all necessary folding attributes and matrices
  *  @param[in]  d1      The distance to reference1 (may be -1)
  *  @param[in]  d2      The distance to reference2
  *  @param[in]  length  The length of the structure beginning from the 5' end
  *  @returns            A sampled secondary structure in dot-bracket notation
  */
 char *
-vrna_pbacktrack5_TwoD(vrna_fold_compound_t  *vc,
+vrna_pbacktrack5_TwoD(vrna_fold_compound_t  *fc,
                       int                   d1,
                       int                   d2,
                       unsigned int          length);
@@ -361,6 +357,7 @@ DEPRECATED(char *
  * i.e. structures beginning from the 5' end with a specified length of the sequence, are backtracked
  *
  * @note      This function does not work (since it makes no sense) for circular RNA sequences!
+ *
  * @pre       The argument 'vars' must contain precalculated partition function matrices,
  *            i.e. a call to TwoDpfold() preceding this function is mandatory!
  *
