@@ -15,8 +15,9 @@
 #include <math.h>
 #include "ViennaRNA/dist_vars.h"
 #include "ViennaRNA/fold_vars.h"
-#include "ViennaRNA/part_func.h"
+#include "ViennaRNA/partfunc/global.h"
 #include "ViennaRNA/utils/basic.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/profiledist.h"
 
 PRIVATE int *alignment[2];
@@ -181,8 +182,11 @@ PrfEditCost(int         i,
   int     k, kmax;
 
   kmax = (int)T1[1];
-  if ((int)T2[1] != kmax)
-    vrna_message_error("inconsistent Profiles in PrfEditCost");
+
+  if ((int)T2[1] != kmax) {
+    vrna_log_error("inconsistent Profiles in PrfEditCost");
+    return (double)INF;
+  }
 
   if (i == 0)
     for (dist = 0., k = 0; k < kmax; k++)
