@@ -41,13 +41,17 @@
 #include <ctype.h>
 #include <string.h>
 #include "ViennaRNA/utils/basic.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/params/constants.h"
 #include "ViennaRNA/params/default.h"
 #include "ViennaRNA/fold_vars.h"
 #include "ViennaRNA/fold.h"
 #include "ViennaRNA/pair_mat.h"
 #include "ViennaRNA/params/basic.h"
-#include "ViennaRNA/loops/all.h"
+#include "ViennaRNA/eval/exterior.h"
+#include "ViennaRNA/eval/hairpin.h"
+#include "ViennaRNA/eval/internal.h"
+#include "ViennaRNA/eval/multibranch.h"
 #include "ViennaRNA/plex.h"
 #include "ViennaRNA/ali_plex.h"
 
@@ -197,7 +201,7 @@ aliduplexfold(const char  *s1[],
 {
   int       i, j, s, n_seq, Emin = INF, i_min = 0, j_min = 0;
   char      *struc;
-  duplexT   mfe;
+  duplexT   mfe = {0};
   vrna_md_t md;
   short     **S1, **S2;
   int       *type;
@@ -207,17 +211,27 @@ aliduplexfold(const char  *s1[],
   for (s = 0; s1[s] != NULL; s++);
   n_seq = s;
   for (s = 0; s2[s] != NULL; s++);
-  if (n_seq != s)
-    vrna_message_error("unequal number of sequences in aliduplexfold()\n");
+
+  if (n_seq != s) {
+    vrna_log_error("unequal number of sequences in aliduplexfold()\n");
+    return mfe;
+  }
 
   set_model_details(&md);
-  if ((!P) || (fabs(P->temperature - temperature) > 1e-6)) {
+  if (!P) {
     update_fold_params();
-    if (P)
-      free(P);
-
     P = vrna_params(&md);
     make_pair_matrix();
+  } else {
+    md.window_size = P->model_details.window_size;
+    md.max_bp_span = P->model_details.max_bp_span;
+    /* check if model_details are the same as before */
+    if (memcmp(&md, &(P->model_details), sizeof(vrna_md_t))) {
+      free(P);
+      update_fold_params();
+      P = vrna_params(&md);
+      make_pair_matrix();
+    }
   }
 
   c = (int **)vrna_alloc(sizeof(int *) * (n3 + 1));
@@ -227,11 +241,19 @@ aliduplexfold(const char  *s1[],
   S1  = (short **)vrna_alloc((n_seq + 1) * sizeof(short *));
   S2  = (short **)vrna_alloc((n_seq + 1) * sizeof(short *));
   for (s = 0; s < n_seq; s++) {
-    if (strlen(s1[s]) != n3)
-      vrna_message_error("uneqal seqence lengths");
+    if (strlen(s1[s]) != n3) {
+      vrna_log_error("uneqal seqence lengths");
+      free(S1);
+      free(S2);
+      return mfe;
+    }
 
-    if (strlen(s2[s]) != n4)
-      vrna_message_error("uneqal seqence lengths");
+    if (strlen(s2[s]) != n4) {
+      vrna_log_error("uneqal seqence lengths");
+      free(S1);
+      free(S2);
+      return mfe;
+    }
 
     S1[s] = encode_seq(s1[s]);
     S2[s] = encode_seq(s2[s]);
@@ -254,7 +276,7 @@ aliduplexfold(const char  *s1[],
 
       for (s = 0; s < n_seq; s++)
         c[i][j] +=
-          vrna_E_ext_stem(type[s], (i > 1) ? S1[s][i - 1] : -1, (j < n4) ? S2[s][j + 1] : -1,
+          vrna_E_exterior_stem(type[s], (i > 1) ? S1[s][i - 1] : -1, (j < n4) ? S2[s][j + 1] : -1,
                           P) + 2 * extension_cost;
       for (k = i - 1; k > 0 && k > i - MAXLOOP - 2; k--) {
         for (l = j + 1; l <= n4; l++) {
@@ -270,7 +292,7 @@ aliduplexfold(const char  *s1[],
             if (type2 == 0)
               type2 = 7;
 
-            E += E_IntLoop(i - k - 1, l - j - 1, type2, rtype[type[s]],
+            E += vrna_E_internal(i - k - 1, l - j - 1, type2, rtype[type[s]],
                            S1[s][k + 1], S2[s][l - 1], S1[s][i - 1], S2[s][j + 1],
                            P) + (i - k + l - j) * extension_cost;
           }
@@ -281,7 +303,7 @@ aliduplexfold(const char  *s1[],
       E       = c[i][j];
       for (s = 0; s < n_seq; s++)
         E +=
-          vrna_E_ext_stem(rtype[type[s]], (j > 1) ? S2[s][j - 1] : -1, (i < n3) ? S1[s][i + 1] : -1,
+          vrna_E_exterior_stem(rtype[type[s]], (j > 1) ? S2[s][j - 1] : -1, (i < n3) ? S1[s][i + 1] : -1,
                           P) + 2 * extension_cost;
       if (E < Emin) {
         Emin  = E;
@@ -346,8 +368,11 @@ alibacktrack(int          n3,
   for (s = 0; S1[s] != NULL; s++);
   n_seq = s;
   for (s = 0; S2[s] != NULL; s++);
-  if (n_seq != s)
-    vrna_message_error("unequal number of sequences in alibacktrack()\n");
+
+  if (n_seq != s) {
+    vrna_log_error("unequal number of sequences in alibacktrack()\n");
+    return NULL;
+  }
 
   st1   = (char *)vrna_alloc(sizeof(char) * (n3 + 1));
   st2   = (char *)vrna_alloc(sizeof(char) * (n4 + 1));
@@ -384,7 +409,7 @@ alibacktrack(int          n3,
           if (type2 == 0)
             type2 = 7;
 
-          LE += E_IntLoop(i - k - 1, l - j - 1, type2, rtype[type[s]],
+          LE += vrna_E_internal(i - k - 1, l - j - 1, type2, rtype[type[s]],
                           S1[s][k + 1], S2[s][l - 1], S1[s][i - 1], S2[s][j + 1],
                           P) + (i - k + l - j) * extension_cost;
         }
@@ -401,12 +426,18 @@ alibacktrack(int          n3,
     if (!traced) {
       for (s = 0; s < n_seq; s++)
         E -=
-          vrna_E_ext_stem(type[s], (i > 1) ? S1[s][i - 1] : -1, (j < n4) ? S2[s][j + 1] : -1,
+          vrna_E_exterior_stem(type[s], (i > 1) ? S1[s][i - 1] : -1, (j < n4) ? S2[s][j + 1] : -1,
                           P) + 2 * extension_cost;
-      if (E != n_seq * P->DuplexInit + n_seq * 2 * extension_cost)
-        vrna_message_error("backtrack failed in aliduplex");
-      else
+      if (E != n_seq * P->DuplexInit + n_seq * 2 * extension_cost) {
+        vrna_log_error("backtrack failed in aliduplex");
+        free(st1);
+        free(st2);
+        free(type);
+        free(struc);
+        return NULL;
+      } else {
         break;
+      }
     }
   }
   if (i > 1)
@@ -458,8 +489,8 @@ aliLduplexfold(const char *s1[],
   int   bopen       = b_b;
   int   bext        = b_a + extension_cost;
   int   iopen       = il_b;
-  int   iext_s      = 2 * (il_a + extension_cost);  /* iext_s 2 nt nucleotide extension of interior loop, on i and j side */
-  int   iext_ass    = 50 + il_a + extension_cost;   /* iext_ass assymetric extension of interior loop, either on i or on j side. */
+  int   iext_s      = 2 * (il_a + extension_cost);  /* iext_s 2 nt nucleotide extension of internal loop, on i and j side */
+  int   iext_ass    = 50 + il_a + extension_cost;   /* iext_ass assymetric extension of internal loop, either on i or on j side. */
   int   min_colonne = INF;                          /* enthaelt das maximum einer kolonne */
   int   i_length;
   int   max_pos;                                    /* get position of the best hit */
@@ -470,21 +501,63 @@ aliLduplexfold(const char *s1[],
   /* FOLLOWING NEXT 4 LINE DEFINES AN ARRAY CONTAINING POSITION OF THE SUBOPT IN S1 */
   int   *position; /* contains the position of the hits with energy > E */
   int   *position_j;
-
+  vrna_md_t md;
 
   n1  = (int)strlen(s1[0]);
   n2  = (int)strlen(s2[0]);
   for (s = 0; s1[s]; s++);
   n_seq = s;
   for (s = 0; s2[s]; s++);
-  if (n_seq != s)
-    vrna_message_error("unequal number of sequences in aliduplexfold()\n");
+
+  if (n_seq != s) {
+    vrna_log_error("unequal number of sequences in aliduplexfold()\n");
+    return NULL;
+  }
 
   position    = (int *)vrna_alloc((delta + (n1) + 4 + delta) * sizeof(int));
   position_j  = (int *)vrna_alloc((delta + (n1) + 4 + delta) * sizeof(int));
 
-  if ((!P) || (fabs(P->temperature - temperature) > 1e-6))
+  set_model_details(&md);
+  if (!P) {
     update_dfold_params();
+    P = vrna_params(&md);
+    make_pair_matrix();
+  } else {
+    md.window_size = P->model_details.window_size;
+    md.max_bp_span = P->model_details.max_bp_span;
+    /* check if model_details are the same as before */
+    if (memcmp(&md, &(P->model_details), sizeof(vrna_md_t))) {
+      free(P);
+      update_dfold_params();
+      P = vrna_params(&md);
+      make_pair_matrix();
+    }
+  }
+
+  S1  = (short **)vrna_alloc((n_seq + 1) * sizeof(short *));
+  S2  = (short **)vrna_alloc((n_seq + 1) * sizeof(short *));
+  for (s = 0; s < n_seq; s++) {
+    if (strlen(s1[s]) != n1) {
+      vrna_log_error("uneqal seqence lengths");
+      free(position);
+      free(position_j);
+      free(S1);
+      free(S2);
+      return NULL;
+    }
+
+    if (strlen(s2[s]) != n2) {
+      vrna_log_error("uneqal seqence lengths");
+      free(position);
+      free(position_j);
+      free(S1);
+      free(S2);
+      return NULL;
+    }
+
+    S1[s] = encode_seq(s1[s]);
+    S2[s] = encode_seq(s2[s]);
+  }
 
   lc    = (int **)vrna_alloc(sizeof(int *) * 5);
   lin   = (int **)vrna_alloc(sizeof(int *) * 5);
@@ -502,19 +575,6 @@ aliLduplexfold(const char *s1[],
     liny[i] = (int *)vrna_alloc(sizeof(int) * (n2 + 5));
   }
 
-
-  S1  = (short **)vrna_alloc((n_seq + 1) * sizeof(short *));
-  S2  = (short **)vrna_alloc((n_seq + 1) * sizeof(short *));
-  for (s = 0; s < n_seq; s++) {
-    if (strlen(s1[s]) != n1)
-      vrna_message_error("uneqal seqence lengths");
-
-    if (strlen(s2[s]) != n2)
-      vrna_message_error("uneqal seqence lengths");
-
-    S1[s] = encode_seq(s1[s]);
-    S2[s] = encode_seq(s2[s]);
-  }
   type = (int *)vrna_alloc(n_seq * sizeof(int));
   /**
   *** array initialization
@@ -551,7 +611,7 @@ aliLduplexfold(const char *s1[],
       *** c_stack -> stacking of c
       *** c_10, c01 -> stack from bulge
       *** c_nm -> arrives in stack from nxm loop
-      *** c_in -> arrives in stack from interior loop
+      *** c_in -> arrives in stack from internal loop
       *** c_bx -> arrives in stack from large bulge on target
       *** c_by -> arrives in stack from large bulge on query
       ***
@@ -618,34 +678,34 @@ aliLduplexfold(const char *s1[],
         continue;
 
       for (s = 0; s < n_seq; s++)
-        lc[idx][j] += vrna_E_ext_stem(type[s], S1[s][i - 1], S2[s][j + 1], P) + 2 * extension_cost;
+        lc[idx][j] += vrna_E_exterior_stem(type[s], S1[s][i - 1], S2[s][j + 1], P) + 2 * extension_cost;
       for (s = 0; s < n_seq; s++) {
         type2 = pair[S1[s][i - 1]][S2[s][j + 1]];
         if (type2 == 0)
           type2 = 7;
 
         c_stack +=
-          E_IntLoop(0, 0, type2, rtype[type[s]], S1[s][i], S2[s][j], S1[s][i - 1], S2[s][j + 1],
+          vrna_E_internal(0, 0, type2, rtype[type[s]], S1[s][i], S2[s][j], S1[s][i - 1], S2[s][j + 1],
                     P) + 2 * extension_cost;
         type2 = pair[S1[s][i - 1]][S2[s][j + 2]];
         if (type2 == 0)
           type2 = 7;
 
         c_01 +=
-          E_IntLoop(0, 1, type2, rtype[type[s]], S1[s][i], S2[s][j + 1], S1[s][i - 1], S2[s][j + 1],
+          vrna_E_internal(0, 1, type2, rtype[type[s]], S1[s][i], S2[s][j + 1], S1[s][i - 1], S2[s][j + 1],
                     P) + 3 * extension_cost;
         type2 = pair[S1[s][i - 2]][S2[s][j + 1]];
         if (type2 == 0)
           type2 = 7;
 
         c_10 +=
-          E_IntLoop(1, 0, type2, rtype[type[s]], S1[s][i - 1], S2[s][j], S1[s][i - 1], S2[s][j + 1],
+          vrna_E_internal(1, 0, type2, rtype[type[s]], S1[s][i - 1], S2[s][j], S1[s][i - 1], S2[s][j + 1],
                     P) + 3 * extension_cost;
         type2 = pair[S1[s][i - 2]][S2[s][j + 2]];
         if (type2 == 0)
           type2 = 7;
 
-        c_11 += E_IntLoop(1,
+        c_11 += vrna_E_internal(1,
                           1,
                           type2,
                           rtype[type[s]],
@@ -658,7 +718,7 @@ aliLduplexfold(const char *s1[],
         if (type2 == 0)
           type2 = 7;
 
-        c_22 += E_IntLoop(2,
+        c_22 += vrna_E_internal(2,
                           2,
                           type2,
                           rtype[type[s]],
@@ -671,7 +731,7 @@ aliLduplexfold(const char *s1[],
         if (type2 == 0)
           type2 = 7;
 
-        c_21 += E_IntLoop(2,
+        c_21 += vrna_E_internal(2,
                           1,
                           type2,
                           rtype[type[s]],
@@ -684,7 +744,7 @@ aliLduplexfold(const char *s1[],
         if (type2 == 0)
           type2 = 7;
 
-        c_12 += E_IntLoop(1,
+        c_12 += vrna_E_internal(1,
                           2,
                           type2,
                           rtype[type[s]],
@@ -697,7 +757,7 @@ aliLduplexfold(const char *s1[],
         if (type2 == 0)
           type2 = 7;
 
-        c_32 += E_IntLoop(3,
+        c_32 += vrna_E_internal(3,
                           2,
                           type2,
                           rtype[type[s]],
@@ -710,7 +770,7 @@ aliLduplexfold(const char *s1[],
         if (type2 == 0)
           type2 = 7;
 
-        c_23 += E_IntLoop(2,
+        c_23 += vrna_E_internal(2,
                           3,
                           type2,
                           rtype[type[s]],
@@ -770,7 +830,7 @@ aliLduplexfold(const char *s1[],
       lc[idx][j]  -= psc;
       temp        = lc[idx][j];
       for (s = 0; s < n_seq; s++)
-        temp += vrna_E_ext_stem(rtype[type[s]], S2[s][j - 1], S1[s][i + 1], P) + 2 * extension_cost;
+        temp += vrna_E_exterior_stem(rtype[type[s]], S2[s][j - 1], S1[s][i + 1], P) + 2 * extension_cost;
       if (min_colonne > temp) {
         min_colonne   = temp;
         min_j_colonne = j;
@@ -999,7 +1059,7 @@ aliduplexfold_XS(const char *s1[],
   int       *type, *type2;
 
   struc = NULL;
-  duplexT   mfe;
+  duplexT   mfe = {0};
   vrna_md_t md;
   int       n_seq;
 
@@ -1011,13 +1071,20 @@ aliduplexfold_XS(const char *s1[],
   /* printf("%d \n",i_pos); */
 
   set_model_details(&md);
-  if ((!P) || (fabs(P->temperature - temperature) > 1e-6)) {
+  if (!P) {
     update_fold_params();
-    if (P)
-      free(P);
-
     P = vrna_params(&md);
     make_pair_matrix();
+  } else {
+    md.window_size = P->model_details.window_size;
+    md.max_bp_span = P->model_details.max_bp_span;
+    /* check if model_details are the same as before */
+    if (memcmp(&md, &(P->model_details), sizeof(vrna_md_t))) {
+      free(P);
+      update_fold_params();
+      P = vrna_params(&md);
+      make_pair_matrix();
+    }
   }
 
   c = (int **)vrna_alloc(sizeof(int *) * (n3 + 1));
@@ -1029,11 +1096,18 @@ aliduplexfold_XS(const char *s1[],
   S1  = (short **)vrna_alloc((n_seq + 1) * sizeof(short *));
   S2  = (short **)vrna_alloc((n_seq + 1) * sizeof(short *));
   for (s = 0; s < n_seq; s++) {
-    if (strlen(s1[s]) != n3)
-      vrna_message_error("uneqal seqence lengths");
-
-    if (strlen(s2[s]) != n4)
-      vrna_message_error("uneqal seqence lengths");
+    if (strlen(s1[s]) != n3) {
+      vrna_log_error("uneqal seqence lengths");
+      free(S1);
+      free(S2);
+      return mfe;
+    }
+    if (strlen(s2[s]) != n4) {
+      vrna_log_error("uneqal seqence lengths");
+      free(S1);
+      free(S2);
+      return mfe;
+    }
 
     S1[s] = encode_seq(s1[s]);
     S2[s] = encode_seq(s2[s]);
@@ -1052,7 +1126,7 @@ aliduplexfold_XS(const char *s1[],
       type[s] = 7;
 
   for (s = 0; s < n_seq; s++)
-    c[i][j] += vrna_E_ext_stem(rtype[type[s]],
+    c[i][j] += vrna_E_exterior_stem(rtype[type[s]],
                                (j_flag ? S2[s][j - 1] : -1),
                                (i_flag ? S1[s][i + 1] : -1),
                                P);
@@ -1088,7 +1162,7 @@ aliduplexfold_XS(const char *s1[],
             if (type3 == 0)
               type3 = 7;
 
-            E += E_IntLoop(p - k - 1, l - q - 1, type2[s], rtype[type3],
+            E += vrna_E_internal(p - k - 1, l - q - 1, type2[s], rtype[type3],
                            S1[s][k + 1], S2[s][l - 1], S1[s][p - 1], S2[s][q + 1], P);
           }
           c[k][l] = MIN2(c[k][l], c[p][q] + E);
@@ -1099,7 +1173,7 @@ aliduplexfold_XS(const char *s1[],
       E       += n_seq * (access_s1[i - k + 1][i_pos] + access_s2[l - 1][j_pos + (l - 1) - 1]);
       for (s = 0; s < n_seq; s++)
         E +=
-          vrna_E_ext_stem(type2[s], (k > 1) ? S1[s][k - 1] : -1, (l < n4) ? S2[s][l + 1] : -1, P);
+          vrna_E_exterior_stem(type2[s], (k > 1) ? S1[s][k - 1] : -1, (l < n4) ? S2[s][l + 1] : -1, P);
       if (E < Emin) {
         Emin  = E;
         k_min = k;
@@ -1190,7 +1264,7 @@ alibacktrack_XS(int         n3,
   n_seq = s;
   for (s = 0; S2[s] != NULL; s++);
   if (n_seq != s)
-    vrna_message_error("unequal number of sequences in alibacktrack()\n");
+    vrna_log_error("unequal number of sequences in alibacktrack()\n");
 
   st1   = (char *)vrna_alloc(sizeof(char) * (n3 + 1));
   st2   = (char *)vrna_alloc(sizeof(char) * (n4 + 1));
@@ -1221,7 +1295,7 @@ alibacktrack_XS(int         n3,
           if (type2 == 0)
             type2 = 7;
 
-          LE += E_IntLoop(k - i - 1, j - l - 1, type[s], rtype[type2],
+          LE += vrna_E_internal(k - i - 1, j - l - 1, type[s], rtype[type2],
                           S1[s][i + 1], S2[s][j - 1], S1[s][k - 1], S2[s][l + 1], P);
         }
         if (E == c[k][l] + LE) {
@@ -1241,7 +1315,7 @@ alibacktrack_XS(int         n3,
 
       break;
       if (E != n_seq * P->DuplexInit)
-        vrna_message_error("backtrack failed in fold duplex bal");
+        vrna_log_error("backtrack failed in fold duplex bal");
       else
         break;
     }
@@ -1287,8 +1361,8 @@ aliLduplexfold_XS(const char  *s1[],
   int   bopen       = b_b;
   int   bext        = b_a;
   int   iopen       = il_b;
-  int   iext_s      = 2 * (il_a); /* iext_s 2 nt nucleotide extension of interior loop, on i and j side */
-  int   iext_ass    = 50 + il_a;  /* iext_ass assymetric extension of interior loop, either on i or on j side. */
+  int   iext_s      = 2 * (il_a); /* iext_s 2 nt nucleotide extension of internal loop, on i and j side */
+  int   iext_ass    = 50 + il_a;  /* iext_ass assymetric extension of internal loop, either on i or on j side. */
   int   min_colonne = INF;        /* enthaelt das maximum einer kolonne */
   int   i_length;
   int   max_pos;                  /* get position of the best hit */
@@ -1300,6 +1374,7 @@ aliLduplexfold_XS(const char  *s1[],
   int   *position; /* contains the position of the hits with energy > E */
   int   *position_j;
   int   maxPenalty[4];
+  vrna_md_t   md;
 
   n1  = (int)strlen(s1[0]);
   n2  = (int)strlen(s2[0]);
@@ -1307,7 +1382,7 @@ aliLduplexfold_XS(const char  *s1[],
   n_seq = s;
   for (s = 0; s2[s]; s++);
   if (n_seq != s)
-    vrna_message_error("unequal number of sequences in aliduplexfold()\n");
+    vrna_log_error("unequal number of sequences in aliduplexfold()\n");
 
   position    = (int *)vrna_alloc((delta + (n1) + 4 + delta) * sizeof(int));
   position_j  = (int *)vrna_alloc((delta + (n1) + 4 + delta) * sizeof(int));
@@ -1315,8 +1390,22 @@ aliLduplexfold_XS(const char  *s1[],
   /**
   *** extension penalty, computed only once, further reduce the computation time
   **/
-  if ((!P) || (fabs(P->temperature - temperature) > 1e-6))
+  set_model_details(&md);
+  if (!P) {
     update_dfold_params();
+    P = vrna_params(&md);
+    make_pair_matrix();
+  } else {
+    md.window_size = P->model_details.window_size;
+    md.max_bp_span = P->model_details.max_bp_span;
+    /* check if model_details are the same as before */
+    if (memcmp(&md, &(P->model_details), sizeof(vrna_md_t))) {
+      free(P);
+      update_dfold_params();
+      P = vrna_params(&md);
+      make_pair_matrix();
+    }
+  }
 
   maxPenalty[0] = (int)-1 * P->stack[2][2] / 2;
   maxPenalty[1] = (int)-1 * P->stack[2][2];
@@ -1345,10 +1434,10 @@ aliLduplexfold_XS(const char  *s1[],
   S2  = (short **)vrna_alloc((n_seq + 1) * sizeof(short *));
   for (s = 0; s < n_seq; s++) {
     if (strlen(s1[s]) != n1)
-      vrna_message_error("uneqal seqence lengths");
+      vrna_log_error("uneqal seqence lengths");
 
     if (strlen(s2[s]) != n2)
-      vrna_message_error("uneqal seqence lengths");
+      vrna_log_error("uneqal seqence lengths");
 
     S1[s] = encode_seq(s1[s]);
     S2[s] = encode_seq(s2[s]);
@@ -1466,34 +1555,34 @@ aliLduplexfold_XS(const char  *s1[],
         continue;
 
       for (s = 0; s < n_seq; s++)
-        lc[idx][j] += vrna_E_ext_stem(type[s], S1[s][i - 1], S2[s][j + 1], P);
+        lc[idx][j] += vrna_E_exterior_stem(type[s], S1[s][i - 1], S2[s][j + 1], P);
       for (s = 0; s < n_seq; s++) {
         type2 = pair[S1[s][i - 1]][S2[s][j + 1]];
         if (type2 == 0)
           type2 = 7;
 
         c_stack +=
-          E_IntLoop(0, 0, type2, rtype[type[s]], S1[s][i], S2[s][j], S1[s][i - 1], S2[s][j + 1],
+          vrna_E_internal(0, 0, type2, rtype[type[s]], S1[s][i], S2[s][j], S1[s][i - 1], S2[s][j + 1],
                     P) + di1 + dj1;
         type2 = pair[S1[s][i - 1]][S2[s][j + 2]];
         if (type2 == 0)
           type2 = 7;
 
         c_01 +=
-          E_IntLoop(0, 1, type2, rtype[type[s]], S1[s][i], S2[s][j + 1], S1[s][i - 1], S2[s][j + 1],
+          vrna_E_internal(0, 1, type2, rtype[type[s]], S1[s][i], S2[s][j + 1], S1[s][i - 1], S2[s][j + 1],
                     P) + di1 + dj2;
         type2 = pair[S1[s][i - 2]][S2[s][j + 1]];
         if (type2 == 0)
           type2 = 7;
 
         c_10 +=
-          E_IntLoop(1, 0, type2, rtype[type[s]], S1[s][i - 1], S2[s][j], S1[s][i - 1], S2[s][j + 1],
+          vrna_E_internal(1, 0, type2, rtype[type[s]], S1[s][i - 1], S2[s][j], S1[s][i - 1], S2[s][j + 1],
                     P) + di2 + dj1;
         type2 = pair[S1[s][i - 2]][S2[s][j + 2]];
         if (type2 == 0)
           type2 = 7;
 
-        c_11 += E_IntLoop(1,
+        c_11 += vrna_E_internal(1,
                           1,
                           type2,
                           rtype[type[s]],
@@ -1506,7 +1595,7 @@ aliLduplexfold_XS(const char  *s1[],
         if (type2 == 0)
           type2 = 7;
 
-        c_22 += E_IntLoop(2,
+        c_22 += vrna_E_internal(2,
                           2,
                           type2,
                           rtype[type[s]],
@@ -1519,7 +1608,7 @@ aliLduplexfold_XS(const char  *s1[],
         if (type2 == 0)
           type2 = 7;
 
-        c_21 += E_IntLoop(2,
+        c_21 += vrna_E_internal(2,
                           1,
                           type2,
                           rtype[type[s]],
@@ -1532,7 +1621,7 @@ aliLduplexfold_XS(const char  *s1[],
         if (type2 == 0)
           type2 = 7;
 
-        c_12 += E_IntLoop(1,
+        c_12 += vrna_E_internal(1,
                           2,
                           type2,
                           rtype[type[s]],
@@ -1545,7 +1634,7 @@ aliLduplexfold_XS(const char  *s1[],
         if (type2 == 0)
           type2 = 7;
 
-        c_32 += E_IntLoop(3,
+        c_32 += vrna_E_internal(3,
                           2,
                           type2,
                           rtype[type[s]],
@@ -1558,7 +1647,7 @@ aliLduplexfold_XS(const char  *s1[],
         if (type2 == 0)
           type2 = 7;
 
-        c_23 += E_IntLoop(2,
+        c_23 += vrna_E_internal(2,
                           3,
                           type2,
                           rtype[type[s]],
@@ -1619,7 +1708,7 @@ aliLduplexfold_XS(const char  *s1[],
       temp        = lc[idx][j];
 
       for (s = 0; s < n_seq; s++)
-        temp += vrna_E_ext_stem(rtype[type[s]], S2[s][j - 1], S1[s][i + 1], P);
+        temp += vrna_E_exterior_stem(rtype[type[s]], S2[s][j - 1], S1[s][i + 1], P);
       if (min_colonne > temp) {
         min_colonne   = temp;
         min_j_colonne = j;
