@@ -4,14 +4,14 @@
 /**
  *  @file ViennaRNA/aln_util.h
  *  @brief      Use ViennaRNA/utils/alignments.h instead
- *  @deprecated Use ViennaRNA/utils/alignments.h instead
+ *  @deprecated Use ViennaRNA/sequences/alignments.h instead
  */
 
 #ifndef VRNA_DISABLE_BACKWARD_COMPATIBILITY
 # ifdef VRNA_WARN_DEPRECATED
-#warning "Including deprecated header file <ViennaRNA/aln_util.h>! Use <ViennaRNA/utils/alignments.h> instead!"
+#warning "Including deprecated header file <ViennaRNA/aln_util.h>! Use <ViennaRNA/sequences/alignments.h> instead!"
 # endif
-#include <ViennaRNA/utils/alignments.h>
+#include <ViennaRNA/sequences/alignments.h>
 #endif
 
 #endif
