@@ -41,6 +41,7 @@
 #include <ctype.h>
 #include <string.h>
 #include "ViennaRNA/utils/basic.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/params/default.h"
 #include "ViennaRNA/fold_vars.h"
 #include "ViennaRNA/fold.h"
@@ -48,7 +49,10 @@
 #include "ViennaRNA/params/basic.h"
 #include "ViennaRNA/plex.h"
 #include "ViennaRNA/ali_plex.h"
-#include "ViennaRNA/loops/all.h"
+#include "ViennaRNA/eval/exterior.h"
+#include "ViennaRNA/eval/hairpin.h"
+#include "ViennaRNA/eval/internal.h"
+#include "ViennaRNA/eval/multibranch.h"
 
 /* int subopt_sorted=0; */
 
@@ -247,14 +251,20 @@ duplexfold_CXS(const char *s1,
   }
 
   set_model_details(&md);
-
-  if ((!P) || (fabs(P->temperature - temperature) > 1e-6)) {
+  if (!P) {
     update_fold_params();
-    if (P)
-      free(P);
-
     P = vrna_params(&md);
     make_pair_matrix();
+  } else {
+    md.window_size = P->model_details.window_size;
+    md.max_bp_span = P->model_details.max_bp_span;
+    /* check if model_details are the same as before */
+    if (memcmp(&md, &(P->model_details), sizeof(vrna_md_t))) {
+      free(P);
+      update_fold_params();
+      P = vrna_params(&md);
+      make_pair_matrix();
+    }
   }
 
   c = (int **)vrna_alloc(sizeof(int *) * (n3 + 1));
@@ -303,7 +313,7 @@ duplexfold_CXS(const char *s1,
           if (!type3)
             continue;
 
-          E = E_IntLoop(p - k - 1,
+          E = vrna_E_internal(p - k - 1,
                         l - q - 1,
                         type2,
                         rtype[type3],
@@ -431,7 +441,7 @@ backtrack_CXS(int         i,
     st2[j - 1]  = ')';
     type        = pair[S1[i]][S2[j]];
     if (!type)
-      vrna_message_error("backtrack failed in fold duplex bli");
+      vrna_log_error("backtrack failed in fold duplex bli");
 
     for (k = i + 1; k <= n3 && k > i - MAXLOOP - 2; k++) {
       for (l = j - 1; l >= previous_const[j] && l >= 1; l--) {
@@ -443,7 +453,7 @@ backtrack_CXS(int         i,
         if (!type2)
           continue;
 
-        LE = E_IntLoop(k - i - 1,
+        LE = vrna_E_internal(k - i - 1,
                        j - l - 1,
                        type,
                        rtype[type2],
@@ -476,7 +486,7 @@ backtrack_CXS(int         i,
 
       /* break; */
       if (E != P->DuplexInit + bonus_2) {
-        vrna_message_error("backtrack failed in fold duplex bal");
+        vrna_log_error("backtrack failed in fold duplex bal");
       } else {
         *Emin -= bonus_2;
         break;
@@ -525,8 +535,8 @@ Lduplexfold_CXS(const char  *s1,
   int       bopen       = b_b;
   int       bext        = b_a;
   int       iopen       = il_b;
-  int       iext_s      = 2 * il_a;   /* iext_s 2 nt nucleotide extension of interior loop, on i and j side */
-  int       iext_ass    = 50 + il_a;  /* iext_ass assymetric extension of interior loop, either on i or on j side. */
+  int       iext_s      = 2 * il_a;   /* iext_s 2 nt nucleotide extension of internal loop, on i and j side */
+  int       iext_ass    = 50 + il_a;  /* iext_ass assymetric extension of internal loop, either on i or on j side. */
   int       min_colonne = INF;        /* enthaelt das maximum einer kolonne */
   int       i_length;
   int       max_pos;                  /* get position of the best hit */
@@ -556,13 +566,21 @@ Lduplexfold_CXS(const char  *s1,
 
   set_model_details(&md);
 
-  if ((!P) || (fabs(P->temperature - temperature) > 1e-6)) {
+  set_model_details(&md);
+  if (!P) {
     update_dfold_params();
-    if (P)
-      free(P);
-
     P = vrna_params(&md);
     make_pair_matrix();
+  } else {
+    md.window_size = P->model_details.window_size;
+    md.max_bp_span = P->model_details.max_bp_span;
+    /* check if model_details are the same as before */
+    if (memcmp(&md, &(P->model_details), sizeof(vrna_md_t))) {
+      free(P);
+      update_dfold_params();
+      P = vrna_params(&md);
+      make_pair_matrix();
+    }
   }
 
   encode_seqs(s1, s2);
@@ -671,7 +689,7 @@ Lduplexfold_CXS(const char  *s1,
       if ((type2 = pair[S1[i - 1]][S2[j + 1]])) {
         lc[idx][j] =
           MIN2(lc[idx_1][j + 1] +
-               E_IntLoop(0, 0, type2, rtype[type], SS1[i], SS2[j], SS1[i - 1], SS2[j + 1],
+               vrna_E_internal(0, 0, type2, rtype[type], SS1[i], SS2[j], SS1[i - 1], SS2[j + 1],
                          P) + di1 + dj1,
                lc[idx][j]);                                                                                                          /* 0x0+1x1 */
       }
@@ -679,7 +697,7 @@ Lduplexfold_CXS(const char  *s1,
       if ((type2 = pair[S1[i - 2]][S2[j + 1]])) {
         lc[idx][j] =
           MIN2(lc[idx_2][j + 1] +
-               E_IntLoop(1, 0, type2, rtype[type], SS1[i - 1], SS2[j], SS1[i - 1], SS2[j + 1],
+               vrna_E_internal(1, 0, type2, rtype[type], SS1[i - 1], SS2[j], SS1[i - 1], SS2[j + 1],
                          P) + di2 + dj1,
                lc[idx][j]);                                                                                                          /* 0x1 +1x1 */
       }
@@ -689,7 +707,7 @@ Lduplexfold_CXS(const char  *s1,
         if ((type2 = pair[S1[i - 1]][S2[j + 2]])) {
           lc[idx][j] =
             MIN2(lc[idx_1][j + 2] +
-                 E_IntLoop(0, 1, type2, rtype[type], SS1[i], SS2[j + 1], SS1[i - 1], SS2[j + 1],
+                 vrna_E_internal(0, 1, type2, rtype[type], SS1[i], SS2[j + 1], SS1[i - 1], SS2[j + 1],
                            P) + di1 + dj2,
                  lc[idx][j]);                                                                                                          /* 1x0 + 1x1 */
         }
@@ -697,7 +715,7 @@ Lduplexfold_CXS(const char  *s1,
         if ((type2 = pair[S1[i - 2]][S2[j + 2]])) {
           lc[idx][j] =
             MIN2(lc[idx_2][j + 2] +
-                 E_IntLoop(1, 1, type2, rtype[type], SS1[i - 1], SS2[j + 1], SS1[i - 1], SS2[j + 1],
+                 vrna_E_internal(1, 1, type2, rtype[type], SS1[i - 1], SS2[j + 1], SS1[i - 1], SS2[j + 1],
                            P) + di2 + dj2,
                  lc[idx][j]);                                                                                                              /*  1x1 +1x1 */
         }
@@ -705,7 +723,7 @@ Lduplexfold_CXS(const char  *s1,
         if ((type2 = pair[S1[i - 3]][S2[j + 2]])) {
           lc[idx][j] =
             MIN2(lc[idx_3][j + 2] +
-                 E_IntLoop(2, 1, type2, rtype[type], SS1[i - 2], SS2[j + 1], SS1[i - 1], SS2[j + 1],
+                 vrna_E_internal(2, 1, type2, rtype[type], SS1[i - 2], SS2[j + 1], SS1[i - 1], SS2[j + 1],
                            P) + di3 + dj2,
                  lc[idx][j]);                                                                                                              /*  2x1 +1x1 */
         }
@@ -714,7 +732,7 @@ Lduplexfold_CXS(const char  *s1,
           if ((type2 = pair[S1[i - 3]][S2[j + 3]])) {
             lc[idx][j] =
               MIN2(lc[idx_3][j + 3] +
-                   E_IntLoop(2, 2, type2, rtype[type], SS1[i - 2], SS2[j + 2], SS1[i - 1],
+                   vrna_E_internal(2, 2, type2, rtype[type], SS1[i - 2], SS2[j + 2], SS1[i - 1],
                              SS2[j + 1],
                              P) + di3 + dj3,
                    lc[idx][j]);                                                                                                            /* 2x2 + 1x1 */
@@ -723,7 +741,7 @@ Lduplexfold_CXS(const char  *s1,
           if ((type2 = pair[S1[i - 2]][S2[j + 3]])) {
             lc[idx][j] =
               MIN2(lc[idx_2][j + 3] +
-                   E_IntLoop(1, 2, type2, rtype[type], SS1[i - 1], SS2[j + 2], SS1[i - 1],
+                   vrna_E_internal(1, 2, type2, rtype[type], SS1[i - 1], SS2[j + 2], SS1[i - 1],
                              SS2[j + 1],
                              P) + di2 + dj3,
                    lc[idx][j]);                                                                                                             /*  1x2 +1x1 */
@@ -732,7 +750,7 @@ Lduplexfold_CXS(const char  *s1,
           if ((type2 = pair[S1[i - 4]][S2[j + 3]])) {
             lc[idx][j] =
               MIN2(lc[idx_4][j + 3] +
-                   E_IntLoop(3, 2, type2, rtype[type], SS1[i - 3], SS2[j + 2], SS1[i - 1],
+                   vrna_E_internal(3, 2, type2, rtype[type], SS1[i - 3], SS2[j + 2], SS1[i - 1],
                              SS2[j + 1],
                              P) + di4 + dj3,
                    lc[idx][j]);
@@ -742,7 +760,7 @@ Lduplexfold_CXS(const char  *s1,
             if ((type2 = pair[S1[i - 3]][S2[j + 4]])) {
               lc[idx][j] =
                 MIN2(lc[idx_3][j + 4] +
-                     E_IntLoop(2, 3, type2, rtype[type], SS1[i - 2], SS2[j + 3], SS1[i - 1],
+                     vrna_E_internal(2, 3, type2, rtype[type], SS1[i - 2], SS2[j + 3], SS1[i - 1],
                                SS2[j + 1],
                                P) + di3 + dj4,
                      lc[idx][j]);
@@ -1022,13 +1040,20 @@ duplexfold_C(const char *s1,
   n4  = (int)strlen(s2);
 
   set_model_details(&md);
-  if ((!P) || (fabs(P->temperature - temperature) > 1e-6)) {
+  if (!P) {
     update_fold_params();
-    if (P)
-      free(P);
-
     P = vrna_params(&md);
     make_pair_matrix();
+  } else {
+    md.window_size = P->model_details.window_size;
+    md.max_bp_span = P->model_details.max_bp_span;
+    /* check if model_details are the same as before */
+    if (memcmp(&md, &(P->model_details), sizeof(vrna_md_t))) {
+      free(P);
+      update_fold_params();
+      P = vrna_params(&md);
+      make_pair_matrix();
+    }
   }
 
   previous_const        = (int *)vrna_alloc(sizeof(int) * (n4 + 1));
@@ -1076,7 +1101,7 @@ duplexfold_C(const char *s1,
           if (!type2)
             continue;
 
-          E = E_IntLoop(i - k - 1, l - j - 1, type2, rtype[type],
+          E = vrna_E_internal(i - k - 1, l - j - 1, type2, rtype[type],
                         SS1[k + 1], SS2[l - 1], SS1[i - 1], SS2[j + 1],
                         P) + (i - k + l - j) * extension_cost + bonus_2;
           c[i][j] = MIN2(c[i][j], c[k][l] + E);
@@ -1171,7 +1196,7 @@ backtrack_C(int         i,
     st2[j - 1]  = ')';
     type        = pair[S1[i]][S2[j]];
     if (!type)
-      vrna_message_error("backtrack failed in fold duplex a");
+      vrna_log_error("backtrack failed in fold duplex a");
 
     for (k = i - 1; k > 0 && k > i - MAXLOOP - 2; k--) {
       for (l = j + 1; l <= previous_const[j]; l++) {
@@ -1183,7 +1208,7 @@ backtrack_C(int         i,
         if (!type2)
           continue;
 
-        LE = E_IntLoop(i - k - 1, l - j - 1, type2, rtype[type],
+        LE = vrna_E_internal(i - k - 1, l - j - 1, type2, rtype[type],
                        SS1[k + 1], SS2[l - 1], SS1[i - 1], SS2[j + 1],
                        P) + (i - k + l - j) * extension_cost + bonus_2;
         if (E == c[k][l] + LE) {
@@ -1210,7 +1235,7 @@ backtrack_C(int         i,
         E -= P->TerminalAU;
 
       if (E != P->DuplexInit + 2 * extension_cost + bonus_2) {
-        vrna_message_error("backtrack failed in fold duplex b");
+        vrna_log_error("backtrack failed in fold duplex b");
       } else {
         *Emin -= bonus_2;
         break;
@@ -1264,8 +1289,8 @@ Lduplexfold_C(const char  *s1,
   int bopen       = b_b;
   int bext        = b_a + extension_cost;
   int iopen       = il_b;
-  int iext_s      = 2 * (il_a + extension_cost);  /* iext_s 2 nt nucleotide extension of interior loop, on i and j side */
-  int iext_ass    = 50 + il_a + extension_cost;   /* iext_ass assymetric extension of interior loop, either on i or on j side. */
+  int iext_s      = 2 * (il_a + extension_cost);  /* iext_s 2 nt nucleotide extension of internal loop, on i and j side */
+  int iext_ass    = 50 + il_a + extension_cost;   /* iext_ass assymetric extension of internal loop, either on i or on j side. */
   int min_colonne = INF;                          /* enthaelt das maximum einer kolonne */
   int i_length;
   int max_pos;                                    /* get position of the best hit */
@@ -1275,6 +1300,7 @@ Lduplexfold_C(const char  *s1,
   int max             = INF;
   int bonus           = -10000;
   int constthreshold  = 0; /* minimal threshold corresponding to a structure complying to all constraints */
+  vrna_md_t md;
 
   i = 0;
   while (structure[i] != '\0') {
@@ -1310,8 +1336,22 @@ Lduplexfold_C(const char  *s1,
    * END OF DEFINITION FOR NEEDED SUBOPT DATA
    */
 
-  if ((!P) || (fabs(P->temperature - temperature) > 1e-6))
+  set_model_details(&md);
+  if (!P) {
     update_dfold_params();
+    P = vrna_params(&md);
+    make_pair_matrix();
+  } else {
+    md.window_size = P->model_details.window_size;
+    md.max_bp_span = P->model_details.max_bp_span;
+    /* check if model_details are the same as before */
+    if (memcmp(&md, &(P->model_details), sizeof(vrna_md_t))) {
+      free(P);
+      update_dfold_params();
+      P = vrna_params(&md);
+      make_pair_matrix();
+    }
+  }
 
   lc    = (int **)vrna_alloc(sizeof(int *) * 5);
   lin   = (int **)vrna_alloc(sizeof(int *) * 5);
@@ -1393,13 +1433,13 @@ Lduplexfold_C(const char  *s1,
       type2       = pair[S1[i - 1]][S2[j + 1]];
       lc[idx][j]  =
         MIN2(lc[idx_1][j + 1] +
-             E_IntLoop(0, 0, type2, rtype[type], SS1[i], SS2[j], SS1[i - 1], SS2[j + 1],
+             vrna_E_internal(0, 0, type2, rtype[type], SS1[i], SS2[j], SS1[i - 1], SS2[j + 1],
                        P) + 2 * extension_cost,
              lc[idx][j]);
       type2       = pair[S1[i - 2]][S2[j + 1]];
       lc[idx][j]  =
         MIN2(lc[idx_2][j + 1] +
-             E_IntLoop(1, 0, type2, rtype[type], SS1[i - 1], SS2[j], SS1[i - 1], SS2[j + 1],
+             vrna_E_internal(1, 0, type2, rtype[type], SS1[i - 1], SS2[j], SS1[i - 1], SS2[j + 1],
                        P) + 3 * extension_cost,
              lc[idx][j]);
       /* kleine loops checks wird in den folgenden if test gemacht. */
@@ -1407,45 +1447,45 @@ Lduplexfold_C(const char  *s1,
         type2       = pair[S1[i - 1]][S2[j + 2]];
         lc[idx][j]  =
           MIN2(lc[idx_1][j + 2] +
-               E_IntLoop(0, 1, type2, rtype[type], SS1[i], SS2[j + 1], SS1[i - 1], SS2[j + 1],
+               vrna_E_internal(0, 1, type2, rtype[type], SS1[i], SS2[j + 1], SS1[i - 1], SS2[j + 1],
                          P) + 3 * extension_cost,
                lc[idx][j]);
         type2       = pair[S1[i - 2]][S2[j + 2]];
         lc[idx][j]  =
           MIN2(lc[idx_2][j + 2] +
-               E_IntLoop(1, 1, type2, rtype[type], SS1[i - 1], SS2[j + 1], SS1[i - 1], SS2[j + 1],
+               vrna_E_internal(1, 1, type2, rtype[type], SS1[i - 1], SS2[j + 1], SS1[i - 1], SS2[j + 1],
                          P) + 4 * extension_cost,
                lc[idx][j]);
         type2       = pair[S1[i - 3]][S2[j + 2]];
         lc[idx][j]  =
           MIN2(lc[idx_3][j + 2] +
-               E_IntLoop(2, 1, type2, rtype[type], SS1[i - 2], SS2[j + 1], SS1[i - 1], SS2[j + 1],
+               vrna_E_internal(2, 1, type2, rtype[type], SS1[i - 2], SS2[j + 1], SS1[i - 1], SS2[j + 1],
                          P) + 5 * extension_cost,
                lc[idx][j]);
         if (!(structure[j + 1] == '|')) {
           type2       = pair[S1[i - 3]][S2[j + 3]];
           lc[idx][j]  =
             MIN2(lc[idx_3][j + 3] +
-                 E_IntLoop(2, 2, type2, rtype[type], SS1[i - 2], SS2[j + 2], SS1[i - 1], SS2[j + 1],
+                 vrna_E_internal(2, 2, type2, rtype[type], SS1[i - 2], SS2[j + 2], SS1[i - 1], SS2[j + 1],
                            P) + 6 * extension_cost,
                  lc[idx][j]);
           type2       = pair[S1[i - 2]][S2[j + 3]];
           lc[idx][j]  =
             MIN2(lc[idx_2][j + 3] +
-                 E_IntLoop(1, 2, type2, rtype[type], SS1[i - 1], SS2[j + 2], SS1[i - 1], SS2[j + 1],
+                 vrna_E_internal(1, 2, type2, rtype[type], SS1[i - 1], SS2[j + 2], SS1[i - 1], SS2[j + 1],
                            P) + 5 * extension_cost,
                  lc[idx][j]);
           type2       = pair[S1[i - 4]][S2[j + 3]];
           lc[idx][j]  =
             MIN2(lc[idx_4][j + 3] +
-                 E_IntLoop(3, 2, type2, rtype[type], SS1[i - 3], SS2[j + 2], SS1[i - 1], SS2[j + 1],
+                 vrna_E_internal(3, 2, type2, rtype[type], SS1[i - 3], SS2[j + 2], SS1[i - 1], SS2[j + 1],
                            P) + 7 * extension_cost,
                  lc[idx][j]);
           if (!(structure[j + 2] == '|')) {
             type2       = pair[S1[i - 3]][S2[j + 4]];
             lc[idx][j]  =
               MIN2(lc[idx_3][j + 4] +
-                   E_IntLoop(2, 3, type2, rtype[type], SS1[i - 2], SS2[j + 3], SS1[i - 1],
+                   vrna_E_internal(2, 3, type2, rtype[type], SS1[i - 2], SS2[j + 3], SS1[i - 1],
                              SS2[j + 1],
                              P) + 7 * extension_cost,
                    lc[idx][j]);
