@@ -16,7 +16,8 @@
 #include "ViennaRNA/params/constants.h" /* defines MINPSCORE */
 #include "ViennaRNA/fold_vars.h"
 #include "ViennaRNA/utils/basic.h"
-#include "ViennaRNA/utils/alignments.h"
+#include "ViennaRNA/sequences/alignments.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/io/file_formats.h"
 #include "ViennaRNA/params/basic.h"
 #include "ViennaRNA/constraints/basic.h"
@@ -55,8 +56,8 @@
  */
 PRIVATE unsigned char
 default_pair_constraint(vrna_fold_compound_t  *fc,
-                        int                   i,
-                        int                   j);
+                        unsigned int          i,
+                        unsigned int          j);
 
 
 PRIVATE INLINE void
@@ -87,7 +88,7 @@ hc_update_up(vrna_fold_compound_t *vc);
 
 PRIVATE void
 hc_update_up_window(vrna_fold_compound_t  *vc,
-                    int                   i,
+                    unsigned int          i,
                     unsigned int          options);
 
 
@@ -165,10 +166,10 @@ vrna_hc_init(vrna_fold_compound_t *vc)
   hc->type    = VRNA_HC_DEFAULT;
   hc->n       = n;
   hc->mx      = (unsigned char *)vrna_alloc(sizeof(unsigned char) * ((n + 1) * (n + 1) + 1));
-  hc->up_ext  = (int *)vrna_alloc(sizeof(int) * (n + 2));
-  hc->up_hp   = (int *)vrna_alloc(sizeof(int) * (n + 2));
-  hc->up_int  = (int *)vrna_alloc(sizeof(int) * (n + 2));
-  hc->up_ml   = (int *)vrna_alloc(sizeof(int) * (n + 2));
+  hc->up_ext  = (unsigned int *)vrna_alloc(sizeof(unsigned int) * (n + 2));
+  hc->up_hp   = (unsigned int *)vrna_alloc(sizeof(unsigned int) * (n + 2));
+  hc->up_int  = (unsigned int *)vrna_alloc(sizeof(unsigned int) * (n + 2));
+  hc->up_ml   = (unsigned int *)vrna_alloc(sizeof(unsigned int) * (n + 2));
   hc->depot   = NULL;
   hc->state   = STATE_UNINITIALIZED;
 
@@ -234,16 +235,16 @@ vrna_hc_update(vrna_fold_compound_t *fc,
     hc  = fc->hc;
 
     if (i > n) {
-      vrna_message_warning("vrna_hc_update(): Position %u out of range!",
+      vrna_log_warning("vrna_hc_update(): Position %u out of range!",
                            " (Sequence length: %u)",
                            i, n);
     } else {
       /* init up_xx arrays if necessary */
       if (!hc->up_ext) {
-        hc->up_ext  = (int *)vrna_alloc(sizeof(int) * (n + 2));
-        hc->up_hp   = (int *)vrna_alloc(sizeof(int) * (n + 2));
-        hc->up_int  = (int *)vrna_alloc(sizeof(int) * (n + 2));
-        hc->up_ml   = (int *)vrna_alloc(sizeof(int) * (n + 2));
+        hc->up_ext  = (unsigned int *)vrna_alloc(sizeof(unsigned int) * (n + 2));
+        hc->up_hp   = (unsigned int *)vrna_alloc(sizeof(unsigned int) * (n + 2));
+        hc->up_int  = (unsigned int *)vrna_alloc(sizeof(unsigned int) * (n + 2));
+        hc->up_ml   = (unsigned int *)vrna_alloc(sizeof(unsigned int) * (n + 2));
 
         hc_update_up(fc);
       }
@@ -293,47 +294,38 @@ vrna_hc_prepare(vrna_fold_compound_t  *fc,
 
 PUBLIC void
 vrna_hc_add_up(vrna_fold_compound_t *fc,
-               int                  i,
+               unsigned int         i,
                unsigned char        option)
 {
-  unsigned int actual_i, strand_i;
-
-  if (fc) {
-    if (fc->hc) {
-      if ((i <= 0) || (i > fc->length)) {
-        vrna_message_warning("vrna_hc_add_up: position out of range, not doing anything");
-        return;
-      }
-
-      strand_i  = fc->strand_number[i];
-      actual_i  = (unsigned int)i - fc->strand_start[strand_i] + 1;
-
-      hc_depot_store_up(fc,
-                        actual_i,
-                        strand_i,
-                        option);
-
-      fc->hc->state |= STATE_DIRTY_UP;
-    }
-  }
+  (void)vrna_hc_add_up_strand(fc, i, -1, option);
 }
 
 
 PUBLIC int
 vrna_hc_add_up_strand(vrna_fold_compound_t  *fc,
                       unsigned int          i,
-                      unsigned int          strand,
+                      int                   strand_indicator,
                       unsigned char         option)
 {
-  int ret = 0;
+  unsigned int  strand, n_i;
+  int           ret = 0;
 
   if ((fc) &&
       (fc->hc) &&
-      (strand < fc->strands) &&
+      (strand_indicator < (int)fc->strands) &&
       (i > 0)) {
-    unsigned int n_i = (fc->type == VRNA_FC_TYPE_SINGLE) ?
-                       fc->nucleotides[strand].length :
-                       fc->alignment[strand].sequences[0].length;
+
+    /* auto-detect strand? */
+    if (strand_indicator < 0) {
+      strand  = fc->strand_number[i];
+      i       = i - fc->strand_start[strand] + 1;
+    } else {
+      strand = (unsigned int)strand_indicator;
+    }
+
+    n_i = (fc->type == VRNA_FC_TYPE_SINGLE) ?
+          fc->nucleotides[strand].length :
+          fc->alignment[strand].sequences[0].length;
 
     if (i > n_i)
       return ret;
@@ -372,7 +364,7 @@ vrna_hc_add_up_batch(vrna_fold_compound_t *fc,
         pos     = constraints[i].position;
         options = constraints[i].options;
 
-        if ((pos <= 0) || (pos > fc->length))
+        if ((pos <= 0) || ((unsigned int)pos > fc->length))
           break;
 
         strand_i  = sn[pos];
@@ -443,7 +435,7 @@ vrna_hc_add_up_strand_batch(vrna_fold_compound_t  *fc,
 
 PUBLIC void
 vrna_hc_add_bp_nonspecific(vrna_fold_compound_t *vc,
-                           int                  i,
+                           unsigned int         i,
                            int                  d,
                            unsigned char        option)
 {
@@ -453,7 +445,7 @@ vrna_hc_add_bp_nonspecific(vrna_fold_compound_t *vc,
   if (vc) {
     if (vc->hc) {
       if ((i <= 0) || (i > vc->length)) {
-        vrna_message_warning("vrna_hc_add_bp_nonspecific: position out of range, not doing anything");
+        vrna_log_warning("vrna_hc_add_bp_nonspecific: position out of range, not doing anything");
         return;
       }
 
@@ -478,24 +470,38 @@ vrna_hc_add_bp_nonspecific(vrna_fold_compound_t *vc,
 PUBLIC int
 vrna_hc_add_bp_strand(vrna_fold_compound_t  *fc,
                       unsigned int          i,
-                      unsigned int          strand_i,
                       unsigned int          j,
-                      unsigned int          strand_j,
+                      int                   strand_i,
+                      int                   strand_j,
                       unsigned char         option)
 {
-  unsigned int  n_i, n_j;
-  int           ret, turn;
+  unsigned int  n_i, n_j, *sn, *ss, turn;
+  int           ret;
   vrna_hc_t     *hc;
 
   ret = 0;
 
   if ((fc) &&
       (fc->hc) &&
-      (strand_i < fc->strands) &&
-      (strand_j < fc->strands) &&
+      (strand_i < (int)fc->strands) &&
+      (strand_j < (int)fc->strands) &&
       (i > 0) &&
       (j > 0)) {
+    sn  = fc->strand_number;
+    ss  = fc->strand_start;
     hc  = fc->hc;
+
+    /* check whether we need to autodetect strand(s) */
+    if (strand_i < 0) {
+      strand_i  = sn[i];
+      i  = i - ss[strand_i] + 1;
+    }
+
+    if (strand_j < 0) {
+      strand_j  = sn[j];
+      j         = j - ss[strand_j] + 1;
+    }
+
     n_i = (fc->type == VRNA_FC_TYPE_SINGLE) ?
           fc->nucleotides[strand_i].length :
           fc->alignment[strand_i].sequences[0].length;
@@ -532,8 +538,8 @@ vrna_hc_add_bp_strand(vrna_fold_compound_t  *fc,
 
 PUBLIC int
 vrna_hc_add_bp(vrna_fold_compound_t *vc,
-               int                  i,
-               int                  j,
+               unsigned int         i,
+               unsigned int         j,
                unsigned char        option)
 {
   unsigned int  *sn, *ss, strand_i, strand_j, actual_i, actual_j;
@@ -546,10 +552,13 @@ vrna_hc_add_bp(vrna_fold_compound_t *vc,
     ss  = vc->strand_start;
 
     if (vc->hc) {
-      if ((i <= 0) || (j <= i) || (j > vc->length)) {
-        vrna_message_warning("vrna_hc_add_bp: position out of range, omitting constraint");
-      } else if ((sn[i] == sn[j]) && ((j - i - 1) < vc->params->model_details.min_loop_size)) {
-        vrna_message_warning(
+      if ((i <= 0) ||
+          (j <= i) ||
+          (j > vc->length)) {
+        vrna_log_warning("vrna_hc_add_bp: position out of range, omitting constraint");
+      } else if ((sn[i] == sn[j]) &&
+                 ((j - i - 1) < (unsigned int)vc->params->model_details.min_loop_size)) {
+        vrna_log_warning(
           "vrna_hc_add_bp: Pairing partners (%d, %d) violate minimum loop size settings of %dnt, omitting constraint",
           i,
           j,
@@ -566,8 +575,8 @@ vrna_hc_add_bp(vrna_fold_compound_t *vc,
 
         ret = vrna_hc_add_bp_strand(vc,
                                     actual_i,
-                                    strand_i,
                                     actual_j,
+                                    strand_i,
                                     strand_j,
                                     option);
       }
@@ -679,13 +688,13 @@ vrna_hc_add_from_db(vrna_fold_compound_t  *vc,
  */
 PRIVATE unsigned char
 default_pair_constraint(vrna_fold_compound_t  *fc,
-                        int                   i,
-                        int                   j)
+                        unsigned int          i,
+                        unsigned int          j)
 {
   unsigned char constraint, can_stack;
   short         *S;
   unsigned int  *sn;
-  int           type;
+  unsigned int  type;
   vrna_md_t     *md;
 
   sn          = fc->strand_number;
@@ -695,8 +704,8 @@ default_pair_constraint(vrna_fold_compound_t  *fc,
   switch (fc->type) {
     case VRNA_FC_TYPE_SINGLE:
       S = fc->sequence_encoding2;
-      if (((j - i) < md->max_bp_span) &&
-          ((sn[i] != sn[j]) || ((j - i) > md->min_loop_size))) {
+      if (((j - i) < (unsigned int)md->max_bp_span) &&
+          ((sn[i] != sn[j]) || ((j - i) > (unsigned int)md->min_loop_size))) {
         type = md->pair[S[i]][S[j]];
         switch (type) {
           case 0:
@@ -725,13 +734,13 @@ default_pair_constraint(vrna_fold_compound_t  *fc,
           /* can it be enclosed by another base pair? */
           if ((i > 1) &&
               (j < fc->length) &&
-              (((j - i + 2) < md->max_bp_span) || (sn[i - 1] != sn[j + 1])) &&
+              (((j - i + 2) < (unsigned int)md->max_bp_span) || (sn[i - 1] != sn[j + 1])) &&
               (md->pair[S[i - 1]][S[j + 1]]))
             can_stack = VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS;
 
           /* can it enclose another base pair? */
           if ((i + 2 < j) &&
-              (((j - i - 2) > md->min_loop_size) || (sn[i + 1] != sn[j - 1])) &&
+              (((j - i - 2) > (unsigned int)md->min_loop_size) || (sn[i + 1] != sn[j - 1])) &&
               (md->pair[S[i + 1]][S[j - 1]]))
             can_stack = VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS;
 
@@ -743,7 +752,7 @@ default_pair_constraint(vrna_fold_compound_t  *fc,
 
     case VRNA_FC_TYPE_COMPARATIVE:
       if ((sn[i] != sn[j]) ||
-          (((j - i + 1) <= md->max_bp_span) && ((j - i - 1) >= md->min_loop_size))) {
+          (((j - i + 1) <= (unsigned int)md->max_bp_span) && ((j - i - 1) >= (unsigned int)md->min_loop_size))) {
         int min_score = md->cv_fact * MINPSCORE;
         int act_score = (fc->hc->type == VRNA_HC_WINDOW) ?
                         fc->pscore_local[i][j - i] :
@@ -758,7 +767,7 @@ default_pair_constraint(vrna_fold_compound_t  *fc,
           /* can it be enclosed by another base pair? */
           if ((i > 1) &&
               (j < fc->length) &&
-              (((j - i + 2) < md->max_bp_span) || (sn[i - 1] != sn[j + 1]))) {
+              (((j - i + 2) < (unsigned int)md->max_bp_span) || (sn[i - 1] != sn[j + 1]))) {
             int outer_pscore = (fc->hc->type == VRNA_HC_WINDOW) ?
                                fc->pscore_local[i - 1][j - i + 2] :
                                fc->pscore[fc->jindx[j + 1] + i - 1];
@@ -768,7 +777,7 @@ default_pair_constraint(vrna_fold_compound_t  *fc,
 
           /* can it enclose another base pair? */
           if ((i + 2 < j) &&
-              (((j - i - 2) > md->min_loop_size) || (sn[i + 1] != sn[j - 1]))) {
+              (((j - i - 2) > (unsigned int)md->min_loop_size) || (sn[i + 1] != sn[j - 1]))) {
             int inner_pscore = (fc->hc->type == VRNA_HC_WINDOW) ?
                                fc->pscore_local[i + 1][j - i - 2] :
                                fc->pscore[fc->jindx[j - 1] + i + 1];
@@ -790,7 +799,7 @@ default_pair_constraint(vrna_fold_compound_t  *fc,
 PRIVATE INLINE void
 populate_hc_up(vrna_fold_compound_t *fc,
                unsigned int         i,
-               unsigned int         options)
+               unsigned int         options VRNA_UNUSED)
 {
   unsigned char context;
   unsigned int  actual_i, strand;
@@ -1073,7 +1082,7 @@ populate_hc_bp(vrna_fold_compound_t *fc,
                unsigned int         options)
 {
   unsigned char constraint, type, t_down, t_up;
-  unsigned int  max_span, maxdist, j, k, p, n, l, turn, strand, sj, sl, actual_i, actual_j,
+  unsigned int  max_span, maxdist, j, k, p, n, l, strand, sj, sl, actual_i, actual_j,
                 actual_l, *sn, *ss;
   vrna_hc_t     *hc;
 
@@ -1084,7 +1093,6 @@ populate_hc_bp(vrna_fold_compound_t *fc,
   ss        = fc->strand_start;
   strand    = sn[i];
   actual_i  = i - ss[strand] + 1;
-  turn      = fc->params->model_details.min_loop_size;
   hc        = fc->hc;
 
   if (options & VRNA_OPTION_F3) {
@@ -1454,45 +1462,43 @@ apply_DB_constraint(vrna_fold_compound_t  *vc,
 {
   char          *sequence;
   short         *S;
-  unsigned int  length, min_loop_size, num_up, num_bp, num_bp_unspecific,
-                size_up, size_bp, size_bp_unspecific;
-  int           n, i, j, hx, *stack, cut;
+  unsigned int  n, i, j, spos;
+  int           hx, *stack;
   vrna_md_t     *md;
-  vrna_hc_up_t  *up;
-  struct hc_bp  *bp, *bp_unspecific;
+
+  vrna_array(vrna_hc_up_t)  up;
+  vrna_array(struct hc_bp)  bp;
+  vrna_array(struct hc_bp)  bp_unspecific;
 
   if (constraint == NULL)
     return;
 
   sequence      = vc->sequence;
-  length        = (int)vc->length;
   S             = vc->sequence_encoding2;
   md            = &(vc->params->model_details);
-  min_loop_size = md->min_loop_size;
-  cut           = vc->cutpoint;
-  n             = (int)strlen(constraint);
+  n             = strlen(constraint);
   stack         = (int *)vrna_alloc(sizeof(int) * (n + 1));
-  size_up       = size_bp = size_bp_unspecific = 10;
-  num_up        = num_bp = num_bp_unspecific = 0;
-  up            = (vrna_hc_up_t *)vrna_alloc(sizeof(vrna_hc_up_t) * size_up);
-  bp            = (struct hc_bp *)vrna_alloc(sizeof(struct hc_bp) * size_bp);
-  bp_unspecific = (struct hc_bp *)vrna_alloc(sizeof(struct hc_bp) * size_bp_unspecific);
 
+  vrna_array_init(up);
+  vrna_array_init(bp);
+  vrna_array_init(bp_unspecific);
+
+  ;
+  for (hx = 0, spos = 0, j = 1; spos < n; j++, spos++) {
+    switch (constraint[spos]) {
+      case '&':
+        /* multistrand delimiter */
+        j--;
+        break;
 
-  for (hx = 0, j = 1; j <= n; j++) {
-    switch (constraint[j - 1]) {
       /* can't pair */
       case 'x':
         if (options & VRNA_CONSTRAINT_DB_X) {
-          up[num_up].position = j;
-          up[num_up].options  = VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS;
-
-          num_up++;
-
-          if (num_up == size_up) {
-            size_up *= 1.4;
-            up      = (vrna_hc_up_t *)vrna_realloc(up, sizeof(vrna_hc_up_t) * size_up);
-          }
+          vrna_array_append(up,
+                            ((vrna_hc_up_t){
+                              .position = j,
+                              .options = VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS
+                            }));
         }
 
         break;
@@ -1500,21 +1506,14 @@ apply_DB_constraint(vrna_fold_compound_t  *vc,
       /* must pair, i.e. may not be unpaired */
       case '|':
         if (options & VRNA_CONSTRAINT_DB_PIPE) {
-          bp_unspecific[num_bp_unspecific].i        = j;  /* position */
-          bp_unspecific[num_bp_unspecific].j        = 0;  /* direction */
-          bp_unspecific[num_bp_unspecific].options  = VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS;
-
-          if (options & VRNA_CONSTRAINT_DB_ENFORCE_BP)
-            bp_unspecific[num_bp_unspecific].options |= VRNA_CONSTRAINT_CONTEXT_ENFORCE;
-
-          num_bp_unspecific++;
-
-          if (num_bp_unspecific == size_bp_unspecific) {
-            size_bp_unspecific  *= 1.4;
-            bp_unspecific       = (struct hc_bp *)vrna_realloc(bp_unspecific,
-                                                               sizeof(struct hc_bp) *
-                                                               size_bp_unspecific);
-          }
+          vrna_array_append(bp_unspecific,
+                            ((struct hc_bp){
+                              .i = j,
+                              .j = 0,
+                              .options = (options & VRNA_CONSTRAINT_DB_ENFORCE_BP) ?
+                                         VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS | VRNA_CONSTRAINT_CONTEXT_ENFORCE:
+                                         VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS
+                            }));
         }
 
         break;
@@ -1530,7 +1529,7 @@ apply_DB_constraint(vrna_fold_compound_t  *vc,
       case ')':
         if (options & VRNA_CONSTRAINT_DB_RND_BRACK) {
           if (hx <= 0) {
-            vrna_message_warning("vrna_hc_add_from_db: "
+            vrna_log_warning("vrna_hc_add_from_db: "
                                  "Unbalanced brackets in constraint string\n%s\n"
                                  "No constraints will be applied!",
                                  constraint);
@@ -1542,15 +1541,15 @@ apply_DB_constraint(vrna_fold_compound_t  *vc,
           if (options & VRNA_CONSTRAINT_DB_CANONICAL_BP) {
             /* check whether this pair forms a non-canoncial base pair */
             if (md->pair[S[i]][S[j]] == 0) {
-              vrna_message_warning("Removing non-canonical base pair %c%c (%d,%d) from constraint",
+              vrna_log_warning("Removing non-canonical base pair %c%c (%d,%d) from constraint",
                                    sequence[i - 1], sequence[j - 1],
                                    i, j);
               break;
             }
           }
 
-          if ((j - i - 1) < md->min_loop_size) {
-            vrna_message_warning("vrna_hc_add_from_db: "
+          if ((j - i - 1) < (unsigned int)md->min_loop_size) {
+            vrna_log_warning("vrna_hc_add_from_db: "
                                  "Pairing partners (%d, %d) violate minimum loop size settings of %dnt, omitting constraint",
                                  i,
                                  j,
@@ -1558,19 +1557,14 @@ apply_DB_constraint(vrna_fold_compound_t  *vc,
             break;
           }
 
-          bp[num_bp].i        = i;
-          bp[num_bp].j        = j;
-          bp[num_bp].options  = VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS;
-
-          if (options & VRNA_CONSTRAINT_DB_ENFORCE_BP)
-            bp[num_bp].options |= VRNA_CONSTRAINT_CONTEXT_ENFORCE;
-
-          num_bp++;
-
-          if (num_bp == size_bp) {
-            size_bp *= 1.4;
-            bp      = (struct hc_bp *)vrna_realloc(bp, sizeof(struct hc_bp) * size_bp);
-          }
+          vrna_array_append(bp,
+                            ((struct hc_bp){
+                              .i = i,
+                              .j = j,
+                              .options = (options & VRNA_CONSTRAINT_DB_ENFORCE_BP) ?
+                                          VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS | VRNA_CONSTRAINT_CONTEXT_ENFORCE:
+                                          VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS
+                            }));
         }
 
         break;
@@ -1578,35 +1572,12 @@ apply_DB_constraint(vrna_fold_compound_t  *vc,
       /* pairs downstream */
       case '<':
         if (options & VRNA_CONSTRAINT_DB_ANG_BRACK) {
-          bp_unspecific[num_bp_unspecific].i        = j;
-          bp_unspecific[num_bp_unspecific].j        = 1;
-          bp_unspecific[num_bp_unspecific].options  = VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS;
-
-          num_bp_unspecific++;
-
-          if (num_bp_unspecific == size_bp_unspecific) {
-            size_bp_unspecific  *= 1.4;
-            bp_unspecific       = (struct hc_bp *)vrna_realloc(bp_unspecific,
-                                                               sizeof(struct hc_bp) *
-                                                               size_bp_unspecific);
-          }
-
-#if 0
-          if (!(options & VRNA_CONSTRAINT_DB_ENFORCE_BP)) {
-            /* (re-)allow this nucleotide to stay unpaired for nostalgic reasons */
-            up[num_up].position = j;
-            up[num_up].options  = VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS |
-                                  VRNA_CONSTRAINT_CONTEXT_NO_REMOVE;
-
-            num_up++;
-
-            if (num_up == size_up) {
-              size_up *= 1.4;
-              up      = (vrna_hc_up_t *)vrna_realloc(up, sizeof(vrna_hc_up_t) * size_up);
-            }
-          }
-
-#endif
+          vrna_array_append(bp_unspecific,
+                            ((struct hc_bp){
+                              .i = j,
+                              .j = 1,
+                              .options = VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS
+                            }));
         }
 
         break;
@@ -1614,35 +1585,12 @@ apply_DB_constraint(vrna_fold_compound_t  *vc,
       /* pairs upstream */
       case '>':
         if (options & VRNA_CONSTRAINT_DB_ANG_BRACK) {
-          bp_unspecific[num_bp_unspecific].i        = j;
-          bp_unspecific[num_bp_unspecific].j        = -1;
-          bp_unspecific[num_bp_unspecific].options  = VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS;
-
-          num_bp_unspecific++;
-
-          if (num_bp_unspecific == size_bp_unspecific) {
-            size_bp_unspecific  *= 1.4;
-            bp_unspecific       = (struct hc_bp *)vrna_realloc(bp_unspecific,
-                                                               sizeof(struct hc_bp) *
-                                                               size_bp_unspecific);
-          }
-
-#if 0
-          if (!(options & VRNA_CONSTRAINT_DB_ENFORCE_BP)) {
-            /* (re-)allow this nucleotide to stay unpaired for nostalgic reasons */
-            up[num_up].position = j;
-            up[num_up].options  = VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS |
-                                  VRNA_CONSTRAINT_CONTEXT_NO_REMOVE;
-
-            num_up++;
-
-            if (num_up == size_up) {
-              size_up *= 1.4;
-              up      = (vrna_hc_up_t *)vrna_realloc(up, sizeof(vrna_hc_up_t) * size_up);
-            }
-          }
-
-#endif
+          vrna_array_append(bp_unspecific,
+                            ((struct hc_bp){
+                              .i        = j,
+                              .j        = -1,
+                              .options  = VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS
+                            }));
         }
 
         break;
@@ -1651,35 +1599,25 @@ apply_DB_constraint(vrna_fold_compound_t  *vc,
       case 'l':
         if (options & VRNA_CONSTRAINT_DB_INTRAMOL) {
           unsigned int l;
-          if (cut > 1) {
-            if (j < cut) {
-              for (l = MAX2(j + min_loop_size, cut); l <= length; l++) {
-                bp[num_bp].i        = j;
-                bp[num_bp].j        = l;
-                bp[num_bp].options  = VRNA_CONSTRAINT_CONTEXT_NONE |
-                                      VRNA_CONSTRAINT_CONTEXT_NO_REMOVE;
+          if (vc->strands > 1) {
+            for (l = 1; l < vc->strand_start[vc->strand_number[j]]; l++) {
+              vrna_array_append(bp,
+                                ((struct hc_bp){
+                                  .i        = j,
+                                  .j        = l,
+                                  .options  = VRNA_CONSTRAINT_CONTEXT_NONE |
+                                              VRNA_CONSTRAINT_CONTEXT_NO_REMOVE
+                                }));
+            }
 
-                num_bp++;
-
-                if (num_bp == size_bp) {
-                  size_bp *= 1.4;
-                  bp      = (struct hc_bp *)vrna_realloc(bp, sizeof(struct hc_bp) * size_bp);
-                }
-              }
-            } else {
-              for (l = 1; l < MIN2(cut, j - min_loop_size); l++) {
-                bp[num_bp].i        = l;
-                bp[num_bp].j        = j;
-                bp[num_bp].options  = VRNA_CONSTRAINT_CONTEXT_NONE |
-                                      VRNA_CONSTRAINT_CONTEXT_NO_REMOVE;
-
-                num_bp++;
-
-                if (num_bp == size_bp) {
-                  size_bp *= 1.4;
-                  bp      = (struct hc_bp *)vrna_realloc(bp, sizeof(struct hc_bp) * size_bp);
-                }
-              }
+            for (l = vc->strand_end[vc->strand_number[j]] + 1; l <= vc->length; l++) {
+              vrna_array_append(bp,
+                                ((struct hc_bp){
+                                  .i        = j,
+                                  .j        = l,
+                                  .options  = VRNA_CONSTRAINT_CONTEXT_NONE |
+                                              VRNA_CONSTRAINT_CONTEXT_NO_REMOVE
+                                }));
             }
           }
         }
@@ -1690,63 +1628,25 @@ apply_DB_constraint(vrna_fold_compound_t  *vc,
       case 'e':
         if (options & VRNA_CONSTRAINT_DB_INTERMOL) {
           unsigned int l;
-          if (cut > 1) {
-            if (j < cut) {
-              for (l = 1; l < j; l++) {
-                bp[num_bp].i        = l;
-                bp[num_bp].j        = j;
-                bp[num_bp].options  = VRNA_CONSTRAINT_CONTEXT_NONE |
-                                      VRNA_CONSTRAINT_CONTEXT_NO_REMOVE;
+          if (vc->strands > 1) {
+            for (l = vc->strand_start[vc->strand_number[j]]; l < j; l++) {
+                vrna_array_append(bp,
+                                  ((struct hc_bp){
+                                    .i        = l,
+                                    .j        = j,
+                                    .options  = VRNA_CONSTRAINT_CONTEXT_NONE |
+                                                VRNA_CONSTRAINT_CONTEXT_NO_REMOVE
+                                  }));
+            }
 
-                num_bp++;
-
-                if (num_bp == size_bp) {
-                  size_bp *= 1.4;
-                  bp      = (struct hc_bp *)vrna_realloc(bp, sizeof(struct hc_bp) * size_bp);
-                }
-              }
-
-              for (l = j + 1; l < cut; l++) {
-                bp[num_bp].i        = j;
-                bp[num_bp].j        = l;
-                bp[num_bp].options  = VRNA_CONSTRAINT_CONTEXT_NONE |
-                                      VRNA_CONSTRAINT_CONTEXT_NO_REMOVE;
-
-                num_bp++;
-
-                if (num_bp == size_bp) {
-                  size_bp *= 1.4;
-                  bp      = (struct hc_bp *)vrna_realloc(bp, sizeof(struct hc_bp) * size_bp);
-                }
-              }
-            } else {
-              for (l = cut; l < j; l++) {
-                bp[num_bp].i        = l;
-                bp[num_bp].j        = j;
-                bp[num_bp].options  = VRNA_CONSTRAINT_CONTEXT_NONE |
-                                      VRNA_CONSTRAINT_CONTEXT_NO_REMOVE;
-
-                num_bp++;
-
-                if (num_bp == size_bp) {
-                  size_bp *= 1.4;
-                  bp      = (struct hc_bp *)vrna_realloc(bp, sizeof(struct hc_bp) * size_bp);
-                }
-              }
-
-              for (l = j + 1; l <= length; l++) {
-                bp[num_bp].i        = j;
-                bp[num_bp].j        = l;
-                bp[num_bp].options  = VRNA_CONSTRAINT_CONTEXT_NONE |
-                                      VRNA_CONSTRAINT_CONTEXT_NO_REMOVE;
-
-                num_bp++;
-
-                if (num_bp == size_bp) {
-                  size_bp *= 1.4;
-                  bp      = (struct hc_bp *)vrna_realloc(bp, sizeof(struct hc_bp) * size_bp);
-                }
-              }
+            for (l = j + 1; l <= vc->strand_end[vc->strand_number[j]]; l++) {
+                vrna_array_append(bp,
+                                  ((struct hc_bp){
+                                    .i        = j,
+                                    .j        = l,
+                                    .options  = VRNA_CONSTRAINT_CONTEXT_NONE |
+                                                VRNA_CONSTRAINT_CONTEXT_NO_REMOVE
+                                  }));
             }
           }
         }
@@ -1757,16 +1657,16 @@ apply_DB_constraint(vrna_fold_compound_t  *vc,
         break;
 
       default:
-        vrna_message_warning(
+        vrna_log_warning(
           "vrna_hc_add_from_db: "
           "Unrecognized character '%c' in constraint string",
-          constraint[j - 1]);
+          constraint[spos]);
         break;
     }
   }
 
   if (hx != 0) {
-    vrna_message_warning("vrna_hc_add_from_db: "
+    vrna_log_warning("vrna_hc_add_from_db: "
                          "Unbalanced brackets in constraint string\n%s\n"
                          "No constraints will be applied!",
                          constraint);
@@ -1776,31 +1676,34 @@ apply_DB_constraint(vrna_fold_compound_t  *vc,
   /* finally, apply constraints */
 
   /* 1st, unspecific pairing states */
-  for (i = 0; i < num_bp_unspecific; i++)
+  for (i = 0; i < vrna_array_size(bp_unspecific); i++)
     vrna_hc_add_bp_nonspecific(vc,
                                bp_unspecific[i].i,  /* nucleotide position */
                                bp_unspecific[i].j,  /* pairing direction */
                                bp_unspecific[i].options);
 
   /* 2nd, specific base pairs */
-  for (i = 0; i < num_bp; i++)
+  for (i = 0; i < vrna_array_size(bp); i++)
     vrna_hc_add_bp(vc,
                    bp[i].i,
                    bp[i].j,
                    bp[i].options);
 
   /* 3rd, unpaired constraints */
-  if (num_up > 0) {
-    up[num_up].position = 0;  /* end of list marker */
+  if (vrna_array_size(up) > 0) {
+    vrna_array_append(up,
+                      ((vrna_hc_up_t){
+                        .position = 0 /* end of list marker */
+                      }));
     vrna_hc_add_up_batch(vc, up);
   }
 
 db_constraints_exit:
 
   /* clean up */
-  free(up);
-  free(bp);
-  free(bp_unspecific);
+  vrna_array_free(up);
+  vrna_array_free(bp);
+  vrna_array_free(bp_unspecific);
   free(stack);
 }
 
@@ -1852,7 +1755,7 @@ hc_update_up(vrna_fold_compound_t *vc)
       hc->up_hp[i] = (hc->mx[n * i + i] & VRNA_CONSTRAINT_CONTEXT_HP_LOOP) ? 1 +
                      hc->up_hp[i + 1] : 0;
 
-    for (hc->up_int[n + 1] = 0, i = n; i > 0; i--) /* unpaired stretch in interior loop */
+    for (hc->up_int[n + 1] = 0, i = n; i > 0; i--) /* unpaired stretch in internal loop */
       hc->up_int[i] = (hc->mx[n * i + i] & VRNA_CONSTRAINT_CONTEXT_INT_LOOP) ? 1 +
                       hc->up_int[i + 1] : 0;
 
@@ -1912,11 +1815,11 @@ hc_update_up(vrna_fold_compound_t *vc)
 
 PRIVATE void
 hc_update_up_window(vrna_fold_compound_t  *vc,
-                    int                   i,
+                    unsigned int          i,
                     unsigned int          options)
 {
-  vrna_hc_t *hc;
-  int       k, winsize, up_ext, up_hp, up_int, up_ml;
+  vrna_hc_t     *hc;
+  unsigned int  k, kmin, winsize, up_ext, up_hp, up_int, up_ml;
 
   hc      = vc->hc;
   winsize = vc->window_size;
@@ -1953,7 +1856,10 @@ hc_update_up_window(vrna_fold_compound_t  *vc,
   if (options & VRNA_OPTION_F5) {
     /* the sliding window proceeds from 5' to 3' so we update constraints 3' to 5' */
     if (up_ext > 0) {
-      for (k = i - 1; k >= MAX2(1, i - winsize); k--) {
+      kmin = 1;
+      if (i > winsize)
+        kmin = i - winsize;
+      for (k = i - 1; k >= kmin; k--) {
         if (hc->up_ext[k] < 1)
           break;
 
@@ -1962,7 +1868,10 @@ hc_update_up_window(vrna_fold_compound_t  *vc,
     }
 
     if (up_hp > 0) {
-      for (k = i - 1; k >= MAX2(1, i - winsize); k--) {
+      kmin = 1;
+      if (i > winsize)
+        kmin = i - winsize;
+      for (k = i - 1; k >= kmin; k--) {
         if (hc->up_hp[k] < 1)
           break;
 
@@ -1971,7 +1880,10 @@ hc_update_up_window(vrna_fold_compound_t  *vc,
     }
 
     if (up_int > 0) {
-      for (k = i - 1; k >= MAX2(1, i - winsize); k--) {
+      kmin = 1;
+      if (i > winsize)
+        kmin = i - winsize;
+      for (k = i - 1; k >= kmin; k--) {
         if (hc->up_int[k] < 1)
           break;
 
@@ -1980,7 +1892,10 @@ hc_update_up_window(vrna_fold_compound_t  *vc,
     }
 
     if (up_ml > 0) {
-      for (k = i - 1; k >= MAX2(1, i - winsize); k--) {
+      kmin = 1;
+      if (i > winsize)
+        kmin = i - winsize;
+      for (k = i - 1; k >= kmin; k--) {
         if (hc->up_ml[k] < 1)
           break;
 
@@ -2058,7 +1973,7 @@ constrain_ptypes(const char   *constraint,
           break;
         case ')':
           if (hx <= 0)
-            vrna_message_error("%s\nunbalanced brackets in constraint", constraint);
+            vrna_log_error("%s\nunbalanced brackets in constraint", constraint);
 
           i     = stack[--hx];
           type  = ptype[index[j] + i];
@@ -2103,7 +2018,7 @@ constrain_ptypes(const char   *constraint,
           break;
         case ')':
           if (hx <= 0)
-            vrna_message_error("%s\nunbalanced brackets in constraints", constraint);
+            vrna_log_error("%s\nunbalanced brackets in constraints", constraint);
 
           i     = stack[--hx];
           type  = ptype[index[i] - j];
@@ -2126,7 +2041,7 @@ constrain_ptypes(const char   *constraint,
   }
 
   if (hx != 0)
-    vrna_message_error("%s\nunbalanced brackets in constraint string", constraint);
+    vrna_log_error("%s\nunbalanced brackets in constraint string", constraint);
 
   free(index);
   free(stack);
