@@ -253,7 +253,7 @@ DEPRECATED(typedef unsigned char (vrna_callback_hc_evaluate)(int i,
 #define VRNA_CONSTRAINT_CONTEXT_HP_LOOP       (unsigned char)0x02
 
 /**
- *  @brief  Hard constraints flag, base pair encloses an interior loop
+ *  @brief  Hard constraints flag, base pair encloses an internal loop
  *
  *  @ingroup  hard_constraints
  *
@@ -269,7 +269,7 @@ DEPRECATED(typedef unsigned char (vrna_callback_hc_evaluate)(int i,
 #define VRNA_CONSTRAINT_CONTEXT_INT_LOOP_ENC  (unsigned char)0x08
 
 /**
- *  @brief  Hard constraints flag, base pair is enclosed in an interior loop
+ *  @brief  Hard constraints flag, base pair is enclosed in an internal loop
  *
  *  @ingroup  hard_constraints
  *
@@ -349,7 +349,7 @@ typedef enum {
  *  following types of base pairs:
  *  - in the exterior loop (#VRNA_CONSTRAINT_CONTEXT_EXT_LOOP)
  *  - enclosing a hairpin (#VRNA_CONSTRAINT_CONTEXT_HP_LOOP)
- *  - enclosing an interior loop (#VRNA_CONSTRAINT_CONTEXT_INT_LOOP)
+ *  - enclosing an internal loop (#VRNA_CONSTRAINT_CONTEXT_INT_LOOP)
  *  - enclosed by an exterior loop (#VRNA_CONSTRAINT_CONTEXT_INT_LOOP_ENC)
  *  - enclosing a multi branch loop (#VRNA_CONSTRAINT_CONTEXT_MB_LOOP)
  *  - enclosed by a multi branch loop (#VRNA_CONSTRAINT_CONTEXT_MB_LOOP_ENC)
@@ -369,32 +369,19 @@ struct vrna_hc_s {
 
   unsigned char   state;
 
-#ifndef VRNA_DISABLE_C11_FEATURES
-  /* C11 support for unnamed unions/structs */
-  union {
-    struct {
-#endif
-  unsigned char *mx;
-#ifndef VRNA_DISABLE_C11_FEATURES
-};
-struct {
-#endif
-  unsigned char **matrix_local;
-#ifndef VRNA_DISABLE_C11_FEATURES
-};
-};
-#endif
+  unsigned char       *mx;
+  unsigned char       **matrix_local;
 
-  int                 *up_ext;    /**<  @brief  A linear array that holds the number of allowed
+  unsigned int        *up_ext;    /**<  @brief  A linear array that holds the number of allowed
                                    *            unpaired nucleotides in an exterior loop
                                    */
-  int                 *up_hp;     /**<  @brief  A linear array that holds the number of allowed
+  unsigned int        *up_hp;     /**<  @brief  A linear array that holds the number of allowed
                                    *            unpaired nucleotides in a hairpin loop
                                    */
-  int                 *up_int;    /**<  @brief  A linear array that holds the number of allowed
-                                   *            unpaired nucleotides in an interior loop
+  unsigned int        *up_int;    /**<  @brief  A linear array that holds the number of allowed
+                                   *            unpaired nucleotides in an internal loop
                                    */
-  int                 *up_ml;     /**<  @brief  A linear array that holds the number of allowed
+  unsigned int        *up_ml;     /**<  @brief  A linear array that holds the number of allowed
                                    *            unpaired nucleotides in a multi branched loop
                                    */
 
@@ -521,14 +508,39 @@ vrna_hc_update(vrna_fold_compound_t *fc,
  */
 void
 vrna_hc_add_up(vrna_fold_compound_t *fc,
-               int                  i,
+               unsigned int         i,
                unsigned char        option);
 
 
+/**
+ *  @brief  Make a certain nucleotide unpaired
+ *
+ *  This function puts a constraint onto a single nucleotide to limit or enforce
+ *  its structural context. The position @p i should be given in local coordinates
+ *  relative to the strand @p strand it corresponds to.
+ *
+ *  @note
+ *  A negative value for the strand number @p strand indicates autodetection
+ *  of the strand number assuming that coordinate i is given as global coordinates
+ *  for the (current) concatenation of all strands. In this case, the function
+ *  behaves exactly as vrna_hc_add_up().
+ *
+ *  @ingroup  hard_constraints
+ *
+ *  @see  vrna_hc_add_bp(), vrna_hc_add_bp_nonspecific(), vrna_hc_init(),
+ *        #VRNA_CONSTRAINT_CONTEXT_EXT_LOOP, #VRNA_CONSTRAINT_CONTEXT_HP_LOOP,
+ *        #VRNA_CONSTRAINT_CONTEXT_INT_LOOP, #VRNA_CONSTRAINT_CONTEXT_MB_LOOP,
+ *        #VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS
+ *
+ *  @param  fc      The #vrna_fold_compound_t the hard constraints are associated with
+ *  @param  i       The position that needs to stay unpaired (1-based)
+ *  @param  strand  The strand number of nucleotide @p i (0-based, negative value for autodetect)
+ *  @param  option  The options flag indicating how/where to store the hard constraints
+ */
 int
 vrna_hc_add_up_strand(vrna_fold_compound_t  *fc,
                       unsigned int          i,
-                      unsigned int          strand,
+                      int                   strand,
                       unsigned char         option);
 
 
@@ -569,17 +581,47 @@ vrna_hc_add_up_strand_batch(vrna_fold_compound_t  *fc,
  */
 int
 vrna_hc_add_bp(vrna_fold_compound_t *fc,
-               int                  i,
-               int                  j,
+               unsigned int         i,
+               unsigned int         j,
                unsigned char        option);
 
 
+/**
+ *  @brief  Favorize/Enforce  a certain base pair (i,j) where i and j may point to different strands
+ *
+ *  This function adds a base pair constraint for pair (i,j), where the positions i and j are
+ *  relative to the RNA strands i and j correspond to. For each strand @p strand_i and @strand_j
+ *  positions are 1-based. For instance, if position 5 of strand 0 must pair with position 10
+ *  of strand 1, the function call would be
+ *  @code
+ *  vrna_hc_add_bp(fc, 5, 10, 0, 1, VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS | VRNA_CONSTRAINT_CONTEXT_ENFORCE);
+ *  @endcode
+ *
+ *  Negative values for the strand numbers @p strand_i and @p strand_j indicate autodetection
+ *  of the strand number assuming that coordinates i and/or j are given as global coordinates
+ *  for the (current) concatenation of all strands
+ *
+ *  @ingroup  hard_constraints
+ *
+ *  @see  vrna_hc_add_bp_nonspecific(), vrna_hc_add_up(), vrna_hc_add_bp(), vrna_hc_init(),
+ *        #VRNA_CONSTRAINT_CONTEXT_EXT_LOOP, #VRNA_CONSTRAINT_CONTEXT_HP_LOOP,
+ *        #VRNA_CONSTRAINT_CONTEXT_INT_LOOP, #VRNA_CONSTRAINT_CONTEXT_INT_LOOP_ENC,
+ *        #VRNA_CONSTRAINT_CONTEXT_MB_LOOP, #VRNA_CONSTRAINT_CONTEXT_MB_LOOP_ENC,
+ *        #VRNA_CONSTRAINT_CONTEXT_ENFORCE, #VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS
+ *
+ *  @param  fc        The #vrna_fold_compound_t the hard constraints are associated with
+ *  @param  i         The 5' located nucleotide position of the base pair (1-based, relative to @p strand_i)
+ *  @param  j         The 3' located nucleotide position of the base pair (1-based, relative to @p strand_j)
+ *  @param  strand_i  The strand number of pairing partner @p i (0-based, negative value for autodetect)
+ *  @param  strand_i  The strand number of pairing partner @p j (0-based, negative value for autodetect)
+ *  @param  option    The option flag(s) indicating loop types and enforcement of the constraint
+ */
 int
 vrna_hc_add_bp_strand(vrna_fold_compound_t  *fc,
                       unsigned int          i,
-                      unsigned int          strand_i,
                       unsigned int          j,
-                      unsigned int          strand_j,
+                      int                   strand_i,
+                      int                   strand_j,
                       unsigned char         option);
 
 
@@ -602,7 +644,7 @@ vrna_hc_add_bp_strand(vrna_fold_compound_t  *fc,
  */
 void
 vrna_hc_add_bp_nonspecific(vrna_fold_compound_t *fc,
-                           int                  i,
+                           unsigned int         i,
                            int                  d,
                            unsigned char        option);
 
