@@ -1,6 +1,6 @@
 /*
  * Reference implementation for including ligand binding to hairpins, or
- * interior loops, with known sequence and/or structure motif, and
+ * internal loops, with known sequence and/or structure motif, and
  * binding free energy utilizing generic soft constraint feature
  *
  * (c) 2015 Ronny Lorenz - ViennaRNA Package
@@ -17,8 +17,9 @@
 
 #include "ViennaRNA/utils/basic.h"
 #include "ViennaRNA/utils/strings.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/model.h"
-#include "ViennaRNA/eval.h"
+#include "ViennaRNA/eval/structures.h"
 #include "ViennaRNA/constraints/soft.h"
 #include "ViennaRNA/constraints/ligand.h"
 
@@ -176,7 +177,7 @@ vrna_sc_ligand_detect_motifs(vrna_fold_compound_t *vc,
 
       /* check motif type */
       if ((pos->k) && (pos->l)) {
-        /* interior loop motif */
+        /* internal loop motif */
         if ((!pt[pos->k]) || (pt[pos->k] != pos->l))
           continue;
 
@@ -305,7 +306,7 @@ vrna_sc_add_hi_motif(vrna_fold_compound_t *vc,
                      const char           *seq,
                      const char           *structure,
                      FLT_OR_DBL           energy,
-                     unsigned int         options)
+                     unsigned int         options VRNA_UNUSED)
 {
   int         cp, cp2;
   char        *sequence, *motif, *motif_alt;
@@ -323,11 +324,11 @@ vrna_sc_add_hi_motif(vrna_fold_compound_t *vc,
 
   /* check for obvious inconsistencies in input sequence/structure motif */
   if (cp != cp2) {
-    vrna_message_warning(
+    vrna_log_warning(
       "vrna_sc_add_ligand_binding@ligand.c: Cutpoint in sequence and structure motif differ!");
     goto hi_motif_error;
   } else if (strlen(seq) != strlen(structure)) {
-    vrna_message_warning(
+    vrna_log_warning(
       "vrna_sc_add_ligand_binding@ligand.c: length of sequence and structure motif differ!");
     goto hi_motif_error;
   }
@@ -350,12 +351,12 @@ vrna_sc_add_hi_motif(vrna_fold_compound_t *vc,
   if (cp > 0) {
     if ((motif[0] != '(') || (motif[strlen(motif) - 1] != ')') || (motif[cp - 2] != '(') ||
         (motif[cp - 1] != ')')) {
-      vrna_message_warning(
-        "vrna_sc_add_ligand_binding@ligand.c: No closing and/or enclosed pair in interior loop motif!");
+      vrna_log_warning(
+        "vrna_sc_add_ligand_binding@ligand.c: No closing and/or enclosed pair in internal loop motif!");
       goto hi_motif_error;
     }
 
-    /* construct corresponding alternative interior loop motif (....(&)...) */
+    /* construct corresponding alternative internal loop motif (....(&)...) */
     motif_alt[0]                  = '(';
     motif_alt[cp - 2]             = '(';
     motif_alt[cp - 1]             = ')';
@@ -367,7 +368,7 @@ vrna_sc_add_hi_motif(vrna_fold_compound_t *vc,
     vrna_sc_add_exp_f(vc, &expAptamerContrib);
   } else {
     if ((motif[0] != '(') || (motif[strlen(motif) - 1] != ')')) {
-      vrna_message_warning("vrna_sc_add_ligand_binding@ligand.c: No closing pair in hairpin motif!");
+      vrna_log_warning("vrna_sc_add_ligand_binding@ligand.c: No closing pair in hairpin motif!");
       goto hi_motif_error;
     }
 
@@ -400,7 +401,7 @@ vrna_sc_add_hi_motif(vrna_fold_compound_t *vc,
 
   if ((pair_count > 0) && (pairs == NULL)) {
     /* error while parsing structure motif */
-    vrna_message_warning(
+    vrna_log_warning(
       "vrna_sc_add_ligand_binding@ligand.c: Error while parsing additional pairs in structure motif");
     goto hi_motif_error;
   }
@@ -519,8 +520,8 @@ AptamerContrib(int            i,
 static int
 AptamerContribHairpin(int           i,
                       int           j,
-                      int           k,
-                      int           l,
+                      int           k VRNA_UNUSED,
+                      int           l VRNA_UNUSED,
                       unsigned char d,
                       void          *data)
 {
@@ -573,8 +574,8 @@ expAptamerContrib(int           i,
 static FLT_OR_DBL
 expAptamerContribHairpin(int            i,
                          int            j,
-                         int            k,
-                         int            l,
+                         int            k VRNA_UNUSED,
+                         int            l VRNA_UNUSED,
                          unsigned char  d,
                          void           *data)
 {
@@ -640,8 +641,8 @@ backtrack_int_motif(int           i,
 static vrna_basepair_t *
 backtrack_hp_motif(int            i,
                    int            j,
-                   int            k,
-                   int            l,
+                   int            k VRNA_UNUSED,
+                   int            l VRNA_UNUSED,
                    unsigned char  d,
                    void           *data)
 {
@@ -781,7 +782,7 @@ scanForPairs(const char *motif5,
       /* printf("5' p[%d, %d]\n", pairs[*pair_count].i, pairs[*pair_count].j); */
       (*pair_count)++;
       if (stack_count < 0) {
-        vrna_message_warning(
+        vrna_log_warning(
           "vrna_sc_add_ligand_binding@ligand.c: 5' structure motif contains unbalanced brackets");
         free(stack);
         free(pairs);
@@ -801,7 +802,7 @@ scanForPairs(const char *motif5,
         /* printf("3' p[%d, %d]\n", pairs[*pair_count].i, pairs[*pair_count].j); */
         (*pair_count)++;
         if (stack_count < 0) {
-          vrna_message_warning(
+          vrna_log_warning(
             "vrna_sc_add_ligand_binding@ligand.c: 3' structure motif contains unbalanced brackets");
           free(stack);
           free(pairs);
@@ -812,7 +813,7 @@ scanForPairs(const char *motif5,
   }
 
   if (stack_count != 0) {
-    vrna_message_warning(
+    vrna_log_warning(
       "vrna_sc_add_ligand_binding@ligand.c: structure motif contains unbalanced brackets");
     (*pair_count)++;
     free(stack);
