@@ -4,6 +4,7 @@
 #include <math.h>
 
 #include "ViennaRNA/utils/basic.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/fold_compound.h"
 #include "ViennaRNA/datastructures/string.h"
 #include "ViennaRNA/datastructures/array.h"
@@ -23,7 +24,6 @@
 # endif
 #endif
 
-
 /*
  #################################
  # GLOBAL VARIABLES              #
@@ -443,7 +443,7 @@ vrna_sc_mod(vrna_fold_compound_t      *fc,
 
       if (msite > fc->length) {
         if (!(options & VRNA_SC_MOD_SILENT))
-          vrna_message_warning("modification site %u after sequence length (%u)",
+          vrna_log_warning("modification site %u after sequence length (%u)",
                                msite,
                                fc->length);
         continue;
@@ -459,7 +459,7 @@ vrna_sc_mod(vrna_fold_compound_t      *fc,
 
       if (!pass) {
         if (!(options & VRNA_SC_MOD_SILENT))
-          vrna_message_warning(
+          vrna_log_warning(
             "modification site %u lists wrong unmodified base %c (should be %c)",
             msite,
             bases[fc->sequence_encoding[msite]],
@@ -476,10 +476,10 @@ vrna_sc_mod(vrna_fold_compound_t      *fc,
       /* allow for all pairing partners specified in the input */
       for (unsigned int j = 1; j < msite; j++) {
         if ((sn[msite] != sn[j]) ||
-            ((msite - j - 1) >= md->min_loop_size)) {
+            ((msite - j - 1) >= (unsigned int)md->min_loop_size)) {
           for (unsigned int cnt = 0; cnt < params->num_ptypes / 2; cnt++) {
             unsigned int pp_enc = params->pairing_partners_encoding[cnt];
-            if (fc->sequence_encoding[j] == pp_enc) {
+            if ((unsigned int)fc->sequence_encoding[j] == pp_enc) {
               vrna_hc_add_bp(fc,
                              j,
                              msite,
@@ -490,10 +490,10 @@ vrna_sc_mod(vrna_fold_compound_t      *fc,
       }
       for (unsigned int j = msite + 1; j <= fc->length; j++) {
         if ((sn[msite] != sn[j]) ||
-            ((j - msite - 1) >= md->min_loop_size)) {
+            ((j - msite - 1) >= (unsigned int)md->min_loop_size)) {
           for (unsigned int cnt = 0; cnt < params->num_ptypes / 2; cnt++) {
             unsigned int pp_enc = params->pairing_partners_encoding[cnt];
-            if (fc->sequence_encoding[j] == pp_enc) {
+            if ((unsigned int)fc->sequence_encoding[j] == pp_enc) {
               vrna_hc_add_bp(fc,
                              msite,
                              j,
@@ -811,8 +811,8 @@ init_stacks(struct vrna_sc_mod_param_s  *params,
               e = (*dG)[i][sj][si];
 
             diffs->stack_diff[i][sj][si] = e - P->stack[pair_MP][tt];
-#ifdef DEBUG
-            printf("d_stack(%c%c, %c%c) = %d = %d - %d\n",
+
+            vrna_log_debug("d_stack(%c%c, %c%c) = %d = %d - %d",
                    nt[5],
                    nt[enc_pp],
                    nt[sj],
@@ -820,7 +820,6 @@ init_stacks(struct vrna_sc_mod_param_s  *params,
                    diffs->stack_diff[i][sj][si],
                    e,
                    P->stack[pair_MP][tt]);
-#endif
           }
 
           if ((*dG)[i + 1][sj][si] != INF) {
@@ -830,8 +829,8 @@ init_stacks(struct vrna_sc_mod_param_s  *params,
               e = (*dG)[i + 1][sj][si];
 
             diffs->stack_diff[i + 1][sj][si] = e - P->stack[pair_PM][tt];
-#ifdef DEBUG
-            printf("d_stack(%c%c, %c%c) = %d = %d - %d\n",
+
+            vrna_log_debug("d_stack(%c%c, %c%c) = %d = %d - %d",
                    nt[enc_pp],
                    nt[5],
                    nt[sj],
@@ -839,7 +838,6 @@ init_stacks(struct vrna_sc_mod_param_s  *params,
                    diffs->stack_diff[i + 1][sj][si],
                    e,
                    P->stack[pair_PM][tt]);
-#endif
           }
         }
       }
@@ -857,14 +855,14 @@ init_mismatches(struct vrna_sc_mod_param_s  *params,
     '\0', 'A', 'C', 'G', 'U', 'M'
   };
 
-#ifdef DEBUG
   char          bp[3] = {
     0
-  }, *bpairs[7] = {
-    "NN", "CG", "GC", "GU", "UG", "AU", "UA"
   };
-#endif
-  unsigned int  i, si, sj, enc_unmod, enc_pp, siu, sju, pair_MP, pair_PM;
+  char          *bpairs[8] = {
+    "NN", "CG", "GC", "GU", "UG", "AU", "UA", "XX"
+  };
+
+  unsigned int  i, si, sj, enc_unmod, enc_pp, siu, sju, pair_enc;
   int           e, (*dG)[MAX_PAIRS][MAX_ALPHABET][MAX_ALPHABET],
   (*dH)[MAX_PAIRS][MAX_ALPHABET][MAX_ALPHABET];
   double        tempf;
@@ -878,35 +876,38 @@ init_mismatches(struct vrna_sc_mod_param_s  *params,
   nt[5]     = params->one_letter_code;
 
   if (params->available & MOD_PARAMS_MISMATCH_dG) {
-    /* process all closing pairs without modified bases */
-    for (i = 1; i <= params->num_ptypes + NBPAIRS; i += 2) {
-      if (i > NBPAIRS) {
-        /* closing pair consists of at least one modified base */
-        enc_pp = params->pairing_partners_encoding[(i - NBPAIRS - 1) / 2];
-        /* pair type of unmodified version as encoded in RNAlib */
-        pair_MP = md->pair[enc_unmod][enc_pp];
-        pair_PM = md->pair[enc_pp][enc_unmod];
+    /*  go through all enclosing base pair types, including those that
+     *  arise from the modification. Here we assume that any pair that
+     *  involves a modified base is present in both directions, hence
+     *  'params->num_ptypes' == 2 * num_pairing partners of the modified
+     *  base.
+     */
+    for (i = 1; i <= NBPAIRS + params->num_ptypes; i++) {
+      if (i <= NBPAIRS) {
+        /* 'regular' enclosing pairs */
+        pair_enc = i;
 
-#ifdef DEBUG
-        bp[0] = nt[5];
-        bp[1] = nt[enc_pp];
-#endif
-      } else {
-        /* closing pair is regular base pair without modified bases */
-        pair_MP = i;
-        pair_PM = i + 1;
-#ifdef DEBUG
         bp[0] = bpairs[i][0];
         bp[1] = bpairs[i][1];
-#endif
+      } else {
+        /* an enclosing pair with a modification */
+        enc_pp = params->pairing_partners_encoding[(i - NBPAIRS - 1) / 2];
+        /* pair type of unmodified version as encoded in RNAlib */
+        if ((i - NBPAIRS - 1) % 2) {
+          pair_enc = md->pair[enc_unmod][enc_pp];
+
+          bp[1] = nt[5];
+          bp[0] = nt[enc_pp];
+        } else {
+          pair_enc = md->pair[enc_pp][enc_unmod];
+
+          bp[0] = nt[5];
+          bp[1] = nt[enc_pp];
+        }
+        if (pair_enc == 0)
+          pair_enc = 7;
       }
 
-      if (pair_MP == 0)
-        pair_MP = 7;
-
-      if (pair_PM == 0)
-        pair_PM = 7;
-
       for (si = 1; si < MAX_ALPHABET; si++) {
         for (sj = 1; sj < MAX_ALPHABET; sj++) {
           if (si == 5)
@@ -932,43 +933,16 @@ init_mismatches(struct vrna_sc_mod_param_s  *params,
              * considering the unpaired bases enclosed. Thus, the pair and 5' and 3'
              * unpaired bases must be rotatated
              */
-            diffs->mismatch_diff[i][si][sj] = e - P->mismatchM[pair_PM][sju][siu];
-#ifdef DEBUG
-            printf("d_mm(%c%c, %c, %c) = %d = %d - %d\n",
+            diffs->mismatch_diff[i][si][sj] = e - P->mismatchM[pair_enc][sju][siu];
+
+            vrna_log_debug("d_mm(%c%c, %c, %c) = %d = %d - %d",
                    bp[0],
                    bp[1],
                    nt[si],
                    nt[sj],
                    diffs->mismatch_diff[i][si][sj],
                    e,
-                   P->mismatchM[pair_PM][sju][siu]);
-#endif
-          }
-
-          if ((*dG)[i + 1][si][sj] != INF) {
-            if (params->available & MOD_PARAMS_MISMATCH_dH)
-              e = (*dH)[i + 1][si][sj] - ((*dH)[i + 1][si][sj] - (*dG)[i + 1][si][sj]) * tempf;
-            else
-              e = (*dG)[i + 1][si][sj];
-
-            /*
-             * take mismatch energies from multiloops as they do not contain anything
-             * but mismatch contributions. Also note, that multiloop mismatches are
-             * stored such that unpaired bases are outside of pair, in contrast to
-             * considering the unpaired bases enclosed. Thus, the pair and 5' and 3'
-             * unpaired bases must be rotatated
-             */
-            diffs->mismatch_diff[i + 1][si][sj] = e - P->mismatchM[pair_MP][sju][siu];
-#ifdef DEBUG
-            printf("d_mm(%c%c, %c, %c) = %d = %d - %d\n",
-                   bp[1],
-                   bp[0],
-                   nt[si],
-                   nt[sj],
-                   diffs->mismatch_diff[i + 1][si][sj],
-                   e,
-                   P->mismatchM[pair_MP][sju][siu]);
-#endif
+                   P->mismatchM[pair_enc][sju][siu]);
           }
         }
       }
@@ -986,14 +960,13 @@ init_dangles(struct vrna_sc_mod_param_s *params,
     '\0', 'A', 'C', 'G', 'U', 'M'
   };
 
-#ifdef DEBUG
   char          bp[3] = {
     0
-  }, *bpairs[7] = {
+  };
+  char          *bpairs[7] = {
     "NN", "CG", "GC", "GU", "UG", "AU", "UA"
   };
-#endif
-  unsigned int  i, si, enc_unmod, enc_pp, siu, pair_MP, pair_PM;
+  unsigned int  i, si, enc_unmod, enc_pp, siu, pair_enc;
   int           e, (*dG5)[MAX_PAIRS][MAX_ALPHABET], (*dH5)[MAX_PAIRS][MAX_ALPHABET],
   (*dG3)[MAX_PAIRS][MAX_ALPHABET], (*dH3)[MAX_PAIRS][MAX_ALPHABET];
   double        tempf;
@@ -1010,34 +983,32 @@ init_dangles(struct vrna_sc_mod_param_s *params,
 
   if (params->available & MOD_PARAMS_DANGLES_dG) {
     /* process all closing pairs without modified bases */
-    for (i = 1; i <= params->num_ptypes + NBPAIRS; i += 2) {
-      if (i > NBPAIRS) {
-        /* closing pair consists of at least one modified base */
-        enc_pp = params->pairing_partners_encoding[(i - NBPAIRS - 1) / 2];
-        /* pair type of unmodified version as encoded in RNAlib */
-        pair_MP = md->pair[enc_unmod][enc_pp];
-        pair_PM = md->pair[enc_pp][enc_unmod];
+    for (i = 1; i <= NBPAIRS + params->num_ptypes; i++) {
+      if (i <= NBPAIRS) {
+        /* 'regular' enclosing pairs */
+        pair_enc = i;
 
-#ifdef DEBUG
-        bp[0] = nt[5];
-        bp[1] = nt[enc_pp];
-#endif
-      } else {
-        /* closing pair is regular base pair without modified bases */
-        pair_MP = i;
-        pair_PM = i + 1;
-#ifdef DEBUG
         bp[0] = bpairs[i][0];
         bp[1] = bpairs[i][1];
-#endif
+      } else {
+        /* an enclosing pair with a modification */
+        enc_pp = params->pairing_partners_encoding[(i - NBPAIRS - 1) / 2];
+        /* pair type of unmodified version as encoded in RNAlib */
+        if ((i - NBPAIRS - 1) % 2) {
+          pair_enc = md->pair[enc_unmod][enc_pp];
+
+          bp[1] = nt[5];
+          bp[0] = nt[enc_pp];
+        } else {
+          pair_enc = md->pair[enc_pp][enc_unmod];
+
+          bp[0] = nt[5];
+          bp[1] = nt[enc_pp];
+        }
+        if (pair_enc == 0)
+          pair_enc = 7;
       }
 
-      if (pair_MP == 0)
-        pair_MP = 7;
-
-      if (pair_PM == 0)
-        pair_PM = 7;
-
       for (si = 1; si < MAX_ALPHABET; si++) {
         if (si == 5)
           siu = enc_unmod;
@@ -1055,16 +1026,15 @@ init_dangles(struct vrna_sc_mod_param_s *params,
            * outside of pair, in contrast to considering the unpaired bases
            * enclosed. Thus, the pair must be rotatated
            */
-          diffs->dangle5_diff[i][si] = e - P->dangle5[pair_PM][siu];
-#ifdef DEBUG
-          printf("d_d5(%c%c, %c) = %d = %d - %d\n",
+          diffs->dangle5_diff[i][si] = e - P->dangle5[pair_enc][siu];
+
+          vrna_log_debug("d_d5(%c%c, %c) = %d = %d - %d",
                  bp[0],
                  bp[1],
                  nt[si],
                  diffs->dangle5_diff[i][si],
                  e,
-                 P->dangle5[pair_PM][siu]);
-#endif
+                 P->dangle5[pair_enc][siu]);
         }
 
         if ((*dG3)[i][si] != INF) {
@@ -1078,62 +1048,15 @@ init_dangles(struct vrna_sc_mod_param_s *params,
            * outside of pair, in contrast to considering the unpaired bases
            * enclosed. Thus, the pair must be rotatated
            */
-          diffs->dangle3_diff[i][si] = e - P->dangle3[pair_PM][siu];
-#ifdef DEBUG
-          printf("d_d3(%c%c, %c) = %d = %d - %d\n",
+          diffs->dangle3_diff[i][si] = e - P->dangle3[pair_enc][siu];
+
+          vrna_log_debug("d_d3(%c%c, %c) = %d = %d - %d",
                  bp[0],
                  bp[1],
                  nt[si],
                  diffs->dangle3_diff[i][si],
                  e,
-                 P->dangle3[pair_PM][siu]);
-#endif
-        }
-
-        if ((*dG5)[i + 1][si] != INF) {
-          if (params->available & MOD_PARAMS_DANGLES_dH)
-            e = (*dH5)[i + 1][si] - ((*dH5)[i + 1][si] - (*dG5)[i + 1][si]) * tempf;
-          else
-            e = (*dG5)[i + 1][si];
-
-          /*
-           * Note, that dangling ends arestored such that unpaired bases are
-           * outside of pair, in contrast to considering the unpaired bases
-           * enclosed. Thus, the pair must be rotatated
-           */
-          diffs->dangle5_diff[i + 1][si] = e - P->dangle5[pair_MP][siu];
-#ifdef DEBUG
-          printf("d_d5(%c%c, %c) = %d = %d - %d\n",
-                 bp[1],
-                 bp[0],
-                 nt[si],
-                 diffs->dangle5_diff[i + 1][si],
-                 e,
-                 P->dangle5[pair_MP][siu]);
-#endif
-        }
-
-        if ((*dG3)[i + 1][si] != INF) {
-          if (params->available & MOD_PARAMS_DANGLES_dH)
-            e = (*dH3)[i + 1][si] - ((*dH3)[i + 1][si] - (*dG3)[i + 1][si]) * tempf;
-          else
-            e = (*dG3)[i + 1][si];
-
-          /*
-           * Note, that dangling ends arestored such that unpaired bases are
-           * outside of pair, in contrast to considering the unpaired bases
-           * enclosed. Thus, the pair must be rotatated
-           */
-          diffs->dangle3_diff[i + 1][si] = e - P->dangle3[pair_MP][siu];
-#ifdef DEBUG
-          printf("d_d3(%c%c, %c) = %d = %d - %d\n",
-                 bp[1],
-                 bp[0],
-                 nt[si],
-                 diffs->dangle3_diff[i + 1][si],
-                 e,
-                 P->dangle3[pair_MP][siu]);
-#endif
+                 P->dangle3[pair_enc][siu]);
         }
       }
     }
@@ -1174,14 +1097,13 @@ init_terminal(struct vrna_sc_mod_param_s  *params,
           e = (*dG)[i];
 
         diffs->terminal_diff[i] = e - Terminal_unmod;
-#ifdef DEBUG
-        printf("d_term(%c%c) = %d = %d - %d\n",
+
+        vrna_log_debug("d_term(%c%c) = %d = %d - %d",
                nt[5],
                nt[enc_pp],
                diffs->terminal_diff[i],
                e,
                Terminal_unmod);
-#endif
       }
 
       if ((*dG)[i + 1] != INF) {
@@ -1191,14 +1113,13 @@ init_terminal(struct vrna_sc_mod_param_s  *params,
           e = (*dG)[i + 1];
 
         diffs->terminal_diff[i + 1] = e - Terminal_unmod;
-#ifdef DEBUG
-        printf("d_term(%c%c) = %d = %d - %d\n",
+
+        vrna_log_debug("d_term(%c%c) = %d = %d - %d",
                nt[enc_pp],
                nt[5],
                diffs->terminal_diff[i + 1],
                e,
                Terminal_unmod);
-#endif
       }
     }
   }
@@ -1248,11 +1169,11 @@ terminal(unsigned int       i,
 
 /* hairpin loop correction including terminalAU terms */
 PRIVATE INLINE int
-sc_PAIR_HP_terminal(vrna_fold_compound_t  *fc,
+sc_PAIR_HP_terminal(vrna_fold_compound_t  *fc VRNA_UNUSED,
                     int                   i,
                     int                   j,
-                    int                   k,
-                    int                   l,
+                    int                   k VRNA_UNUSED,
+                    int                   l VRNA_UNUSED,
                     void                  *d)
 {
   return terminal(i, j,
@@ -1265,8 +1186,8 @@ PRIVATE INLINE int
 sc_PAIR_HP_mismatch(vrna_fold_compound_t  *fc,
                     int                   i,
                     int                   j,
-                    int                   k,
-                    int                   l,
+                    int                   k VRNA_UNUSED,
+                    int                   l VRNA_UNUSED,
                     void                  *d)
 {
   return mismatch(fc,
@@ -1290,7 +1211,7 @@ sc_PAIR_HP(vrna_fold_compound_t *fc,
 
 
 PRIVATE INLINE int
-sc_PAIR_IL_stack(vrna_fold_compound_t *fc,
+sc_PAIR_IL_stack(vrna_fold_compound_t *fc VRNA_UNUSED,
                  int                  i,
                  int                  j,
                  int                  k,
@@ -1319,7 +1240,7 @@ sc_PAIR_IL_stack(vrna_fold_compound_t *fc,
 
 
 PRIVATE INLINE int
-sc_PAIR_IL_terminal(vrna_fold_compound_t  *fc,
+sc_PAIR_IL_terminal(vrna_fold_compound_t  *fc VRNA_UNUSED,
                     int                   i,
                     int                   j,
                     int                   k,
@@ -1343,8 +1264,8 @@ sc_PAIR_IL_mismatch(vrna_fold_compound_t  *fc,
                     int                   l,
                     void                  *d)
 {
-  if (((k - i - 1) > 2) &&
-      ((j - l - 1) > 2))
+  if (((k - i) > 1) &&
+      ((j - l) > 1))
     return mismatch(fc, i, j, (energy_corrections *)d) +
            mismatch(fc, l, k, (energy_corrections *)d);
 
@@ -1406,11 +1327,11 @@ sc_PAIR_IL(vrna_fold_compound_t *fc,
 
 
 PRIVATE INLINE int
-sc_PAIR_ML_terminal(vrna_fold_compound_t  *fc,
+sc_PAIR_ML_terminal(vrna_fold_compound_t  *fc VRNA_UNUSED,
                     int                   i,
                     int                   j,
-                    int                   k,
-                    int                   l,
+                    int                   k VRNA_UNUSED,
+                    int                   l VRNA_UNUSED,
                     void                  *d)
 {
   return terminal(i, j,
@@ -1422,8 +1343,8 @@ PRIVATE INLINE int
 sc_PAIR_ML_mismatch(vrna_fold_compound_t  *fc,
                     int                   i,
                     int                   j,
-                    int                   k,
-                    int                   l,
+                    int                   k VRNA_UNUSED,
+                    int                   l VRNA_UNUSED,
                     void                  *d)
 {
   return mismatch(fc,
@@ -1446,9 +1367,9 @@ sc_PAIR_ML(vrna_fold_compound_t *fc,
 
 
 PRIVATE INLINE int
-sc_STEM_terminal(vrna_fold_compound_t *fc,
-                 int                  i,
-                 int                  j,
+sc_STEM_terminal(vrna_fold_compound_t *fc VRNA_UNUSED,
+                 int                  i VRNA_UNUSED,
+                 int                  j VRNA_UNUSED,
                  int                  k,
                  int                  l,
                  void                 *d)
@@ -1460,8 +1381,8 @@ sc_STEM_terminal(vrna_fold_compound_t *fc,
 
 PRIVATE INLINE int
 sc_STEM_mismatch(vrna_fold_compound_t *fc,
-                 int                  i,
-                 int                  j,
+                 int                  i VRNA_UNUSED,
+                 int                  j VRNA_UNUSED,
                  int                  k,
                  int                  l,
                  void                 *d)
@@ -1486,11 +1407,11 @@ sc_STEM(vrna_fold_compound_t  *fc,
 
 
 PRIVATE INLINE int
-sc_EXT_STEM_EXT_terminal(vrna_fold_compound_t *fc,
+sc_EXT_STEM_EXT_terminal(vrna_fold_compound_t *fc VRNA_UNUSED,
                          int                  i,
-                         int                  j,
+                         int                  j VRNA_UNUSED,
                          int                  k,
-                         int                  l,
+                         int                  l VRNA_UNUSED,
                          void                 *d)
 {
   return terminal(k, i,
@@ -1501,9 +1422,9 @@ sc_EXT_STEM_EXT_terminal(vrna_fold_compound_t *fc,
 PRIVATE INLINE int
 sc_EXT_STEM_EXT_mismatch(vrna_fold_compound_t *fc,
                          int                  i,
-                         int                  j,
+                         int                  j VRNA_UNUSED,
                          int                  k,
-                         int                  l,
+                         int                  l VRNA_UNUSED,
                          void                 *d)
 {
   return mismatch(fc,
@@ -1526,10 +1447,10 @@ sc_EXT_STEM_EXT(vrna_fold_compound_t  *fc,
 
 
 PRIVATE INLINE int
-sc_EXT_EXT_STEM_terminal(vrna_fold_compound_t *fc,
-                         int                  i,
+sc_EXT_EXT_STEM_terminal(vrna_fold_compound_t *fc VRNA_UNUSED,
+                         int                  i VRNA_UNUSED,
                          int                  j,
-                         int                  k,
+                         int                  k VRNA_UNUSED,
                          int                  l,
                          void                 *d)
 {
@@ -1540,9 +1461,9 @@ sc_EXT_EXT_STEM_terminal(vrna_fold_compound_t *fc,
 
 PRIVATE INLINE int
 sc_EXT_EXT_STEM_mismatch(vrna_fold_compound_t *fc,
-                         int                  i,
+                         int                  i VRNA_UNUSED,
                          int                  j,
-                         int                  k,
+                         int                  k VRNA_UNUSED,
                          int                  l,
                          void                 *d)
 {
@@ -1566,9 +1487,9 @@ sc_EXT_EXT_STEM(vrna_fold_compound_t  *fc,
 
 
 PRIVATE INLINE int
-sc_EXT_STEM_OUTSIDE_terminal(vrna_fold_compound_t *fc,
-                             int                  i,
-                             int                  j,
+sc_EXT_STEM_OUTSIDE_terminal(vrna_fold_compound_t *fc VRNA_UNUSED,
+                             int                  i VRNA_UNUSED,
+                             int                  j VRNA_UNUSED,
                              int                  k,
                              int                  l,
                              void                 *d)
@@ -1580,8 +1501,8 @@ sc_EXT_STEM_OUTSIDE_terminal(vrna_fold_compound_t *fc,
 
 PRIVATE INLINE int
 sc_EXT_STEM_OUTSIDE_mismatch(vrna_fold_compound_t *fc,
-                             int                  i,
-                             int                  j,
+                             int                  i VRNA_UNUSED,
+                             int                  j VRNA_UNUSED,
                              int                  k,
                              int                  l,
                              void                 *d)
@@ -1606,10 +1527,10 @@ sc_EXT_STEM_OUTSIDE(vrna_fold_compound_t  *fc,
 
 
 PRIVATE INLINE int
-sc_ML_ML_STEM_terminal(vrna_fold_compound_t *fc,
-                       int                  i,
+sc_ML_ML_STEM_terminal(vrna_fold_compound_t *fc VRNA_UNUSED,
+                       int                  i VRNA_UNUSED,
                        int                  j,
-                       int                  k,
+                       int                  k VRNA_UNUSED,
                        int                  l,
                        void                 *d)
 {
@@ -1620,9 +1541,9 @@ sc_ML_ML_STEM_terminal(vrna_fold_compound_t *fc,
 
 PRIVATE INLINE int
 sc_ML_ML_STEM_mismatch(vrna_fold_compound_t *fc,
-                       int                  i,
+                       int                  i VRNA_UNUSED,
                        int                  j,
-                       int                  k,
+                       int                  k VRNA_UNUSED,
                        int                  l,
                        void                 *d)
 {
