@@ -18,14 +18,17 @@
 #include <ctype.h>
 #include <string.h>
 #include "ViennaRNA/utils/basic.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/params/default.h"
 #include "ViennaRNA/fold_vars.h"
 #include "ViennaRNA/fold.h"
 #include "ViennaRNA/pair_mat.h"
 #include "ViennaRNA/params/basic.h"
 #include "ViennaRNA/alifold.h"
-#include "ViennaRNA/subopt.h"
-#include "ViennaRNA/loops/all.h"
+#include "ViennaRNA/eval/exterior.h"
+#include "ViennaRNA/eval/hairpin.h"
+#include "ViennaRNA/eval/internal.h"
+#include "ViennaRNA/subopt/wuchty.h"
 #include "ViennaRNA/duplex.h"
 
 #ifdef _OPENMP
@@ -130,12 +133,18 @@ duplexfold_cu(const char  *s1,
   n2  = (int)strlen(s2);
 
   set_model_details(&md);
-  if ((!P) || (fabs(P->temperature - temperature) > 1e-6)) {
-    if (P)
-      free(P);
-
+  if (!P) {
     P = vrna_params(&md);
     make_pair_matrix();
+  } else {
+    md.window_size = P->model_details.window_size;
+    md.max_bp_span = P->model_details.max_bp_span;
+    /* check if model_details are the same as before */
+    if (memcmp(&md, &(P->model_details), sizeof(vrna_md_t))) {
+      free(P);
+      P = vrna_params(&md);
+      make_pair_matrix();
+    }
   }
 
   c = (int **)vrna_alloc(sizeof(int *) * (n1 + 1));
@@ -155,7 +164,7 @@ duplexfold_cu(const char  *s1,
       if (!type)
         continue;
 
-      c[i][j] += vrna_E_ext_stem(type, (i > 1) ? SS1[i - 1] : -1, (j < n2) ? SS2[j + 1] : -1, P);
+      c[i][j] += vrna_E_exterior_stem(type, (i > 1) ? SS1[i - 1] : -1, (j < n2) ? SS2[j + 1] : -1, P);
       for (k = i - 1; k > 0 && k > i - MAXLOOP - 2; k--) {
         for (l = j + 1; l <= n2; l++) {
           if (i - k + l - j - 2 > MAXLOOP)
@@ -165,13 +174,13 @@ duplexfold_cu(const char  *s1,
           if (!type2)
             continue;
 
-          E = E_IntLoop(i - k - 1, l - j - 1, type2, rtype[type],
+          E = vrna_E_internal(i - k - 1, l - j - 1, type2, rtype[type],
                         SS1[k + 1], SS2[l - 1], SS1[i - 1], SS2[j + 1], P);
           c[i][j] = MIN2(c[i][j], c[k][l] + E);
         }
       }
       E = c[i][j];
-      E += vrna_E_ext_stem(rtype[type], (j > 1) ? SS2[j - 1] : -1, (i < n1) ? SS1[i + 1] : -1, P);
+      E += vrna_E_exterior_stem(rtype[type], (j > 1) ? SS2[j - 1] : -1, (i < n1) ? SS1[i + 1] : -1, P);
       if (E < Emin) {
         Emin  = E;
         i_min = i;
@@ -232,7 +241,7 @@ duplex_subopt(const char  *s1,
         continue;
 
       E   = Ed = c[i][j];
-      Ed  += vrna_E_ext_stem(type, (j > 1) ? SS2[j - 1] : -1, (i < n1) ? SS1[i + 1] : -1, P);
+      Ed  += vrna_E_exterior_stem(type, (j > 1) ? SS2[j - 1] : -1, (i < n1) ? SS1[i + 1] : -1, P);
       if (Ed > thresh)
         continue;
 
@@ -251,7 +260,7 @@ duplex_subopt(const char  *s1,
         continue;
 
       struc = backtrack(i, j);
-      vrna_message_info(stderr, "%d %d %d", i, j, E);
+      vrna_log_info("%d %d %d", i, j, E);
       if (n_subopt + 1 >= n_max) {
         n_max   *= 2;
         subopt  = (duplexT *)vrna_realloc(subopt, n_max * sizeof(duplexT));
@@ -303,8 +312,12 @@ backtrack(int i,
     st1[i - 1]  = '(';
     st2[j - 1]  = ')';
     type        = pair[S1[i]][S2[j]];
-    if (!type)
-      vrna_message_error("backtrack failed in fold duplex");
+    if (!type) {
+      vrna_log_error("backtrack failed in fold duplex");
+      free(st1);
+      free(st2);
+      return NULL;
+    }
 
     for (k = i - 1; k > 0 && k > i - MAXLOOP - 2; k--) {
       for (l = j + 1; l <= n2; l++) {
@@ -316,7 +329,7 @@ backtrack(int i,
         if (!type2)
           continue;
 
-        LE = E_IntLoop(i - k - 1, l - j - 1, type2, rtype[type],
+        LE = vrna_E_internal(i - k - 1, l - j - 1, type2, rtype[type],
                        SS1[k + 1], SS2[l - 1], SS1[i - 1], SS2[j + 1], P);
         if (E == c[k][l] + LE) {
           traced  = 1;
@@ -329,11 +342,15 @@ backtrack(int i,
         break;
     }
     if (!traced) {
-      E -= vrna_E_ext_stem(type, (i > 1) ? SS1[i - 1] : -1, (j < n2) ? SS2[j + 1] : -1, P);
-      if (E != P->DuplexInit)
-        vrna_message_error("backtrack failed in fold duplex");
-      else
+      E -= vrna_E_exterior_stem(type, (i > 1) ? SS1[i - 1] : -1, (j < n2) ? SS2[j + 1] : -1, P);
+      if (E != P->DuplexInit) {
+        vrna_log_error("backtrack failed in fold duplex");
+        free(st1);
+        free(st2);
+        return NULL;
+      } else {
         break;
+      }
     }
   }
   if (i > 1)
@@ -402,7 +419,7 @@ aliduplexfold_cu(const char *s1[],
 {
   int       i, j, s, n_seq, Emin = INF, i_min = 0, j_min = 0;
   char      *struc;
-  duplexT   mfe;
+  duplexT   mfe = {0};
   short     **S1, **S2;
   int       *type;
   vrna_md_t md;
@@ -413,16 +430,24 @@ aliduplexfold_cu(const char *s1[],
   for (s = 0; s1[s] != NULL; s++);
   n_seq = s;
   for (s = 0; s2[s] != NULL; s++);
-  if (n_seq != s)
-    vrna_message_error("unequal number of sequences in aliduplexfold()\n");
+  if (n_seq != s) {
+    vrna_log_error("unequal number of sequences in aliduplexfold()\n");
+    return mfe;
+  }
 
   set_model_details(&md);
-  if ((!P) || (fabs(P->temperature - temperature) > 1e-6)) {
-    if (P)
-      free(P);
-
+  if (!P) {
     P = vrna_params(&md);
     make_pair_matrix();
+  } else {
+    md.window_size = P->model_details.window_size;
+    md.max_bp_span = P->model_details.max_bp_span;
+    /* check if model_details are the same as before */
+    if (memcmp(&md, &(P->model_details), sizeof(vrna_md_t))) {
+      free(P);
+      P = vrna_params(&md);
+      make_pair_matrix();
+    }
   }
 
   c = (int **)vrna_alloc(sizeof(int *) * (n1 + 1));
@@ -432,11 +457,19 @@ aliduplexfold_cu(const char *s1[],
   S1  = (short **)vrna_alloc((n_seq + 1) * sizeof(short *));
   S2  = (short **)vrna_alloc((n_seq + 1) * sizeof(short *));
   for (s = 0; s < n_seq; s++) {
-    if (strlen(s1[s]) != n1)
-      vrna_message_error("uneqal seqence lengths");
+    if (strlen(s1[s]) != n1) {
+      vrna_log_error("uneqal seqence lengths");
+      free(S1);
+      free(S2);
+      return mfe;
+    }
 
-    if (strlen(s2[s]) != n2)
-      vrna_message_error("uneqal seqence lengths");
+    if (strlen(s2[s]) != n2) {
+      vrna_log_error("uneqal seqence lengths");
+      free(S1);
+      free(S2);
+      return mfe;
+    }
 
     S1[s] = encode_sequence(s1[s], 0);
     S2[s] = encode_sequence(s2[s], 0);
@@ -458,7 +491,7 @@ aliduplexfold_cu(const char *s1[],
         continue;
 
       for (s = 0; s < n_seq; s++)
-        c[i][j] += vrna_E_ext_stem(type[s],
+        c[i][j] += vrna_E_exterior_stem(type[s],
                                    (i > 1) ? S1[s][i - 1] : -1,
                                    (j < n2) ? S2[s][j + 1] : -1,
                                    P);
@@ -477,7 +510,7 @@ aliduplexfold_cu(const char *s1[],
             if (type2 == 0)
               type2 = 7;
 
-            E += E_IntLoop(i - k - 1, l - j - 1, type2, rtype[type[s]],
+            E += vrna_E_internal(i - k - 1, l - j - 1, type2, rtype[type[s]],
                            S1[s][k + 1], S2[s][l - 1], S1[s][i - 1], S2[s][j + 1], P);
           }
           c[i][j] = MIN2(c[i][j], c[k][l] + E);
@@ -487,7 +520,7 @@ aliduplexfold_cu(const char *s1[],
       E       = c[i][j];
       for (s = 0; s < n_seq; s++)
         E +=
-          vrna_E_ext_stem(rtype[type[s]],
+          vrna_E_exterior_stem(rtype[type[s]],
                           (j > 1) ? S2[s][j - 1] : -1,
                           (i < n1) ? S1[s][i + 1] : -1,
                           P);
@@ -553,11 +586,19 @@ aliduplex_subopt(const char *s1[],
   S1      = (short **)vrna_alloc((n_seq + 1) * sizeof(short *));
   S2      = (short **)vrna_alloc((n_seq + 1) * sizeof(short *));
   for (s = 0; s < n_seq; s++) {
-    if (strlen(s1[s]) != n1)
-      vrna_message_error("uneqal seqence lengths");
+    if (strlen(s1[s]) != n1) {
+      vrna_log_error("uneqal seqence lengths");
+      free(S1);
+      free(S2);
+      return NULL;
+    }
 
-    if (strlen(s2[s]) != n2)
-      vrna_message_error("uneqal seqence lengths");
+    if (strlen(s2[s]) != n2) {
+      vrna_log_error("uneqal seqence lengths");
+      free(S1);
+      free(S2);
+      return NULL;
+    }
 
     S1[s] = encode_sequence(s1[s], 0);
     S2[s] = encode_sequence(s2[s], 0);
@@ -581,7 +622,7 @@ aliduplex_subopt(const char *s1[],
       E = Ed = c[i][j];
       for (s = 0; s < n_seq; s++)
         Ed +=
-          vrna_E_ext_stem(type[s], (j > 1) ? S2[s][j - 1] : -1, (i < n1) ? S1[s][i + 1] : -1, P);
+          vrna_E_exterior_stem(type[s], (j > 1) ? S2[s][j - 1] : -1, (i < n1) ? S1[s][i + 1] : -1, P);
       if (Ed > thresh)
         continue;
 
@@ -600,7 +641,7 @@ aliduplex_subopt(const char *s1[],
         continue;
 
       struc = alibacktrack(i, j, (const short **)S1, (const short **)S2);
-      vrna_message_info(stderr, "%d %d %d", i, j, E);
+      vrna_log_info("%d %d %d", i, j, E);
       if (n_subopt + 1 >= n_max) {
         n_max   *= 2;
         subopt  = (duplexT *)vrna_realloc(subopt, n_max * sizeof(duplexT));
@@ -651,8 +692,10 @@ alibacktrack(int          i,
   for (s = 0; S1[s] != NULL; s++);
   n_seq = s;
   for (s = 0; S2[s] != NULL; s++);
-  if (n_seq != s)
-    vrna_message_error("unequal number of sequences in alibacktrack()\n");
+  if (n_seq != s) {
+    vrna_log_error("unequal number of sequences in alibacktrack()\n");
+    return NULL;
+  }
 
   st1   = (char *)vrna_alloc(sizeof(char) * (n1 + 1));
   st2   = (char *)vrna_alloc(sizeof(char) * (n2 + 1));
@@ -689,7 +732,7 @@ alibacktrack(int          i,
           if (type2 == 0)
             type2 = 7;
 
-          LE += E_IntLoop(i - k - 1, l - j - 1, type2, rtype[type[s]],
+          LE += vrna_E_internal(i - k - 1, l - j - 1, type2, rtype[type[s]],
                           S1[s][k + 1], S2[s][l - 1], S1[s][i - 1], S2[s][j + 1], P);
         }
         if (E == c[k][l] + LE) {
@@ -704,11 +747,17 @@ alibacktrack(int          i,
     }
     if (!traced) {
       for (s = 0; s < n_seq; s++)
-        E -= vrna_E_ext_stem(type[s], (i > 1) ? S1[s][i - 1] : -1, (j < n2) ? S2[s][j + 1] : -1, P);
-      if (E != n_seq * P->DuplexInit)
-        vrna_message_error("backtrack failed in aliduplex");
-      else
+        E -= vrna_E_exterior_stem(type[s], (i > 1) ? S1[s][i - 1] : -1, (j < n2) ? S2[s][j + 1] : -1, P);
+      if (E != n_seq * P->DuplexInit) {
+        vrna_log_error("backtrack failed in aliduplex");
+        free(st1);
+        free(st2);
+        free(type);
+        free(struc);
+        return NULL;
+      } else {
         break;
+      }
     }
   }
   if (i > 1)
