@@ -24,23 +24,32 @@
 #include <limits.h>
 
 #include "ViennaRNA/utils/basic.h"
-#include "ViennaRNA/utils/structures.h"
 #include "ViennaRNA/utils/strings.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/params/default.h"
 #include "ViennaRNA/datastructures/basic.h"
 #include "ViennaRNA/fold_vars.h"
 #include "ViennaRNA/params/basic.h"
-#include "ViennaRNA/gquad.h"
-#include "ViennaRNA/utils/alignments.h"
-#include "ViennaRNA/ribo.h"
+#include "ViennaRNA/sequences/alignments.h"
+#include "ViennaRNA/params/ribosum.h"
 #include "ViennaRNA/constraints/hard.h"
 #include "ViennaRNA/constraints/soft.h"
-#include "ViennaRNA/part_func.h"
+#include "ViennaRNA/partfunc/global.h"
 #include "ViennaRNA/cofold.h"
 #include "ViennaRNA/mm.h"
-#include "ViennaRNA/alphabet.h"
+#include "ViennaRNA/sequences/alphabet.h"
+#include "ViennaRNA/structures/utils.h"
 #include "ViennaRNA/fold_compound.h"
 
+
+#ifndef INLINE
+#ifdef __GNUC__
+# define INLINE inline
+#else
+# define INLINE
+#endif
+#endif
+
 /*
  #################################
  # PRIVATE MACROS                #
@@ -108,7 +117,7 @@ nullify(vrna_fold_compound_t *fc);
 PUBLIC void
 vrna_fold_compound_free(vrna_fold_compound_t *fc)
 {
-  int s;
+  unsigned int s;
 
   if (fc) {
     /* first destroy common attributes */
@@ -121,6 +130,7 @@ vrna_fold_compound_free(vrna_fold_compound_t *fc)
 
     vrna_hc_free(fc->hc);
     vrna_ud_remove(fc);
+    vrna_gr_reset(fc);
     vrna_sequence_remove_all(fc);
 
     /* now distinguish the fc type */
@@ -202,13 +212,13 @@ vrna_fold_compound(const char       *sequence,
   /* sanity check */
   length = strlen(sequence);
   if (length == 0) {
-    vrna_message_warning("vrna_fold_compound@data_structures.c: "
+    vrna_log_warning("vrna_fold_compound@data_structures.c: "
                          "sequence length must be greater 0");
     return NULL;
   }
 
   if (length > vrna_sequence_length_max(options)) {
-    vrna_message_warning("vrna_fold_compound@data_structures.c: "
+    vrna_log_warning("vrna_fold_compound@data_structures.c: "
                          "sequence length of %d exceeds addressable range",
                          length);
     return NULL;
@@ -289,7 +299,7 @@ vrna_fold_compound_comparative2(const char                **sequences,
                                 vrna_md_t                 *md_p,
                                 unsigned int              options)
 {
-  int                   s, n_seq, length;
+  unsigned int          s, n_seq, length;
   vrna_fold_compound_t  *fc;
   vrna_md_t             md;
   unsigned int          aux_options;
@@ -306,17 +316,17 @@ vrna_fold_compound_comparative2(const char                **sequences,
   length = strlen(sequences[0]);
   /* sanity check */
   if (length == 0) {
-    vrna_message_warning("vrna_fold_compound_comparative: "
+    vrna_log_warning("vrna_fold_compound_comparative: "
                          "sequence length must be greater 0");
   } else if (length > vrna_sequence_length_max(options)) {
-    vrna_message_warning("vrna_fold_compound_comparative: "
+    vrna_log_warning("vrna_fold_compound_comparative: "
                          "sequence length of %d exceeds addressable range",
                          length);
   }
 
   for (s = 0; s < n_seq; s++)
     if (strlen(sequences[s]) != length) {
-      vrna_message_warning("vrna_fold_compound_comparative: "
+      vrna_log_warning("vrna_fold_compound_comparative: "
                            "uneqal sequence lengths in alignment");
       return NULL;
     }
@@ -400,7 +410,7 @@ vrna_fold_compound_TwoD(const char    *sequence,
                         vrna_md_t     *md_p,
                         unsigned int  options)
 {
-  int                   length, l, turn;
+  unsigned int          length, l, turn;
   vrna_fold_compound_t  *fc;
   vrna_md_t             md;
 
@@ -411,11 +421,11 @@ vrna_fold_compound_TwoD(const char    *sequence,
   /* sanity check */
   length = strlen(sequence);
   if (length == 0) {
-    vrna_message_warning("vrna_fold_compound_TwoD: "
+    vrna_log_warning("vrna_fold_compound_TwoD: "
                          "sequence length must be greater 0");
     return NULL;
   } else if (length > vrna_sequence_length_max(options)) {
-    vrna_message_warning("vrna_fold_compound_TwoD: "
+    vrna_log_warning("vrna_fold_compound_TwoD: "
                          "sequence length of %d exceeds addressable range",
                          length);
     return NULL;
@@ -423,14 +433,14 @@ vrna_fold_compound_TwoD(const char    *sequence,
 
   l = strlen(s1);
   if (l != length) {
-    vrna_message_warning("vrna_fold_compound_TwoD: "
+    vrna_log_warning("vrna_fold_compound_TwoD: "
                          "sequence and s1 differ in length");
     return NULL;
   }
 
   l = strlen(s2);
   if (l != length) {
-    vrna_message_warning("vrna_fold_compound_TwoD: "
+    vrna_log_warning("vrna_fold_compound_TwoD: "
                          "sequence and s2 differ in length");
     return NULL;
   }
@@ -514,7 +524,7 @@ vrna_fold_compound_prepare(vrna_fold_compound_t *fc,
 
   /* check maximum sequence length restrictions */
   if (fc->length > vrna_sequence_length_max(options)) {
-    vrna_message_warning(
+    vrna_log_warning(
       "vrna_fold_compound_prepare@data_structures.c: sequence length of %d exceeds addressable range",
       fc->length);
     return 0;
@@ -554,6 +564,9 @@ vrna_fold_compound_prepare(vrna_fold_compound_t *fc,
   /* Add DP matrices, if not they are not present or do not fit current settings */
   vrna_mx_prepare(fc, options);
 
+  /* prepare auxiliary grammar rules data structure, if required */
+  ret &= vrna_gr_prepare(fc, options);
+
   return ret;
 }
 
