@@ -10,6 +10,7 @@
 #include <limits.h>
 
 #include "ViennaRNA/datastructures/basic.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/model.h"
 #include "ViennaRNA/params/basic.h"
 #include "ViennaRNA/fold.h"
@@ -17,7 +18,7 @@
 #include "ViennaRNA/fold_vars.h"
 #include "ViennaRNA/utils/basic.h"
 #include "ViennaRNA/utils/strings.h"
-#include "ViennaRNA/utils/structures.h"
+#include "ViennaRNA/structures/pairtable.h"
 #include "ViennaRNA/landscape/findpath.h"
 
 
@@ -591,7 +592,7 @@ main(int  argc,
 static void
 usage(void)
 {
-  vrna_message_error("usage: findpath.c  [-m depth] [-d[0|1|2]] [-v]");
+  vrna_log_error("usage: findpath.c  [-m depth] [-d[0|1|2]] [-v]");
 }
 
 
