@@ -24,7 +24,8 @@
 
 #include "ViennaRNA/params/constants.h"
 #include "ViennaRNA/utils/basic.h"
-#include "ViennaRNA/alphabet.h"
+#include "ViennaRNA/utils/log.h"
+#include "ViennaRNA/sequences/alphabet.h"
 #include "ViennaRNA/model.h"
 
 /*
@@ -69,7 +70,7 @@ int             logML           = VRNA_MODEL_DEFAULT_LOG_ML;
 
 /* below are some more deprecated global symbols we need to get rid off */
 
-int             james_rule        = 1;    /* interior loops of size 2 get energy 0.8Kcal and
+int             james_rule        = 1;    /* internal loops of size 2 get energy 0.8Kcal and
                                            * no mismatches (no longer used) */
 char            *RibosumFile      = NULL; /* TODO: compile ribosums into program
                                            * Warning: this variable will vanish */
@@ -127,42 +128,44 @@ dm_default[7][7] = DM_DEFAULT;
 
 
 PRIVATE vrna_md_t defaults = {
-  VRNA_MODEL_DEFAULT_TEMPERATURE,
-  1.,
-  VRNA_MODEL_DEFAULT_PF_SMOOTH,
-  VRNA_MODEL_DEFAULT_DANGLES,
-  VRNA_MODEL_DEFAULT_SPECIAL_HP,
-  VRNA_MODEL_DEFAULT_NO_LP,
-  VRNA_MODEL_DEFAULT_NO_GU,
-  VRNA_MODEL_DEFAULT_NO_GU_CLOSURE,
-  VRNA_MODEL_DEFAULT_LOG_ML,
-  VRNA_MODEL_DEFAULT_CIRC,
-  VRNA_MODEL_DEFAULT_GQUAD,
-  VRNA_MODEL_DEFAULT_UNIQ_ML,
-  VRNA_MODEL_DEFAULT_ENERGY_SET,
-  VRNA_MODEL_DEFAULT_BACKTRACK,
-  VRNA_MODEL_DEFAULT_BACKTRACK_TYPE,
-  VRNA_MODEL_DEFAULT_COMPUTE_BPP,
-  { 0 },
-  VRNA_MODEL_DEFAULT_MAX_BP_SPAN,
-  TURN,
-  VRNA_MODEL_DEFAULT_WINDOW_SIZE,
-  VRNA_MODEL_DEFAULT_ALI_OLD_EN,
-  VRNA_MODEL_DEFAULT_ALI_RIBO,
-  VRNA_MODEL_DEFAULT_ALI_CV_FACT,
-  VRNA_MODEL_DEFAULT_ALI_NC_FACT,
-  1.07,
-  BP_REV_DEFAULT,
-  BP_ALIAS_DEFAULT,
-  BP_ENCODING_DEFAULT,
-  DM_DEFAULT,
-  VRNA_MODEL_DEFAULT_SALT,
-  VRNA_MODEL_DEFAULT_SALT_MLLOWER,
-  VRNA_MODEL_DEFAULT_SALT_MLUPPER,
-  VRNA_MODEL_DEFAULT_SALT_DPXINIT,
-  VRNA_MODEL_DEFAULT_SALT_DPXINIT_FACT,
-  VRNA_MODEL_DEFAULT_HELICAL_RISE,
-  VRNA_MODEL_DEFAULT_BACKBONE_LENGTH
+  .temperature = VRNA_MODEL_DEFAULT_TEMPERATURE,
+  .betaScale = 1.,
+  .pf_smooth = VRNA_MODEL_DEFAULT_PF_SMOOTH,
+  .dangles  = VRNA_MODEL_DEFAULT_DANGLES,
+  .special_hp  = VRNA_MODEL_DEFAULT_SPECIAL_HP,
+  .noLP  = VRNA_MODEL_DEFAULT_NO_LP,
+  .noGU  = VRNA_MODEL_DEFAULT_NO_GU,
+  .noGUclosure  = VRNA_MODEL_DEFAULT_NO_GU_CLOSURE,
+  .logML  = VRNA_MODEL_DEFAULT_LOG_ML,
+  .circ  = VRNA_MODEL_DEFAULT_CIRC,
+  .circ_penalty  = VRNA_MODEL_DEFAULT_CIRC_PENALTY,
+  .gquad  = VRNA_MODEL_DEFAULT_GQUAD,
+  .uniq_ML  = VRNA_MODEL_DEFAULT_UNIQ_ML,
+  .energy_set  = VRNA_MODEL_DEFAULT_ENERGY_SET,
+  .backtrack  = VRNA_MODEL_DEFAULT_BACKTRACK,
+  .backtrack_type  = VRNA_MODEL_DEFAULT_BACKTRACK_TYPE,
+  .compute_bpp  = VRNA_MODEL_DEFAULT_COMPUTE_BPP,
+  .nonstandards  = { 0 },
+  .max_bp_span  = VRNA_MODEL_DEFAULT_MAX_BP_SPAN,
+  .min_loop_size  = TURN,
+  .window_size  = VRNA_MODEL_DEFAULT_WINDOW_SIZE,
+  .oldAliEn  = VRNA_MODEL_DEFAULT_ALI_OLD_EN,
+  .ribo  = VRNA_MODEL_DEFAULT_ALI_RIBO,
+  .cv_fact  = VRNA_MODEL_DEFAULT_ALI_CV_FACT,
+  .nc_fact  = VRNA_MODEL_DEFAULT_ALI_NC_FACT,
+  .sfact  = 1.07,
+  .rtype  = BP_REV_DEFAULT,
+  .alias  = BP_ALIAS_DEFAULT,
+  .pair  = BP_ENCODING_DEFAULT,
+  .pair_dist  = DM_DEFAULT,
+  .salt  = VRNA_MODEL_DEFAULT_SALT,
+  .saltMLLower  = VRNA_MODEL_DEFAULT_SALT_MLLOWER,
+  .saltMLUpper  = VRNA_MODEL_DEFAULT_SALT_MLUPPER,
+  .saltDPXInit  = VRNA_MODEL_DEFAULT_SALT_DPXINIT,
+  .saltDPXInitFact  = VRNA_MODEL_DEFAULT_SALT_DPXINIT_FACT,
+  .helical_rise  = VRNA_MODEL_DEFAULT_HELICAL_RISE,
+  .backbone_length  = VRNA_MODEL_DEFAULT_BACKBONE_LENGTH,
+  .circ_alpha0  = VRNA_MODEL_DEFAULT_CIRC_ALPHA0
 };
 
 /*
@@ -244,25 +247,26 @@ vrna_md_option_string(vrna_md_t *md)
 
   *options = '\0';
 
-  if (md) {
-    if (md->dangles != VRNA_MODEL_DEFAULT_DANGLES)
-      sprintf(options + strlen(options), "-d%d ", md->dangles);
+  if (md == NULL)
+    md = &(defaults);
 
-    if (!md->special_hp)
-      strcat(options, "-4 ");
+  sprintf(options + strlen(options), "-d%d ", md->dangles);
 
-    if (md->noLP)
-      strcat(options, "--noLP ");
+  if (!md->special_hp)
+    strcat(options, "-4 ");
 
-    if (md->noGU)
-      strcat(options, "--noGU ");
+  if (md->noLP)
+    strcat(options, "--noLP ");
 
-    if (md->noGUclosure)
-      strcat(options, "--noClosingGU ");
+  if (md->noGU)
+    strcat(options, "--noGU ");
+
+  if (md->noGUclosure)
+    strcat(options, "--noClosingGU ");
+
+  if (md->temperature != VRNA_MODEL_DEFAULT_TEMPERATURE)
+    sprintf(options + strlen(options), "-T %f ", md->temperature);
 
-    if (md->temperature != VRNA_MODEL_DEFAULT_TEMPERATURE)
-      sprintf(options + strlen(options), "-T %f ", md->temperature);
-  }
 
   return options;
 }
@@ -321,7 +325,7 @@ vrna_md_set_nonstandards(vrna_md_t  *md,
         memcpy(nonstandards, &(md->nonstandards[0]), 33 * sizeof(char));
 #endif
       } else {
-        vrna_message_warning("vrna_md_set_nonstandards: list too long, dropping nonstandards!");
+        vrna_log_warning("vrna_md_set_nonstandards: list too long, dropping nonstandards!");
       }
     } else {
       /* remove nonstandards */
@@ -350,6 +354,7 @@ vrna_md_defaults_reset(vrna_md_t *md_p)
   defaults.logML            = VRNA_MODEL_DEFAULT_LOG_ML;
   defaults.gquad            = VRNA_MODEL_DEFAULT_GQUAD;
   defaults.circ             = VRNA_MODEL_DEFAULT_CIRC;
+  defaults.circ_penalty     = VRNA_MODEL_DEFAULT_CIRC_PENALTY;
   defaults.uniq_ML          = VRNA_MODEL_DEFAULT_UNIQ_ML;
   defaults.compute_bpp      = VRNA_MODEL_DEFAULT_COMPUTE_BPP;
   defaults.backtrack        = VRNA_MODEL_DEFAULT_BACKTRACK;
@@ -374,6 +379,7 @@ vrna_md_defaults_reset(vrna_md_t *md_p)
   defaults.saltDPXInitFact  = VRNA_MODEL_DEFAULT_SALT_DPXINIT_FACT;
   defaults.helical_rise     = VRNA_MODEL_DEFAULT_HELICAL_RISE;
   defaults.backbone_length  = VRNA_MODEL_DEFAULT_BACKBONE_LENGTH;
+  defaults.circ_alpha0      = VRNA_MODEL_DEFAULT_CIRC_ALPHA0;
   if (md_p) {
     /* now try to apply user settings */
     /*
@@ -390,6 +396,7 @@ vrna_md_defaults_reset(vrna_md_t *md_p)
     vrna_md_defaults_logML(md_p->logML);
     vrna_md_defaults_gquad(md_p->gquad);
     vrna_md_defaults_circ(md_p->circ);
+    vrna_md_defaults_circ_penalty(md_p->circ_penalty);
     vrna_md_defaults_uniq_ML(md_p->uniq_ML);
     vrna_md_defaults_compute_bpp(md_p->compute_bpp);
     vrna_md_defaults_backtrack(md_p->backtrack);
@@ -413,6 +420,7 @@ vrna_md_defaults_reset(vrna_md_t *md_p)
     vrna_md_defaults_saltDPXInitFact(md_p->saltDPXInitFact);
     vrna_md_defaults_helical_rise(md_p->helical_rise);
     vrna_md_defaults_backbone_length(md_p->backbone_length);
+    vrna_md_defaults_circ_alpha0(md_p->circ_alpha0);
     copy_nonstandards(&defaults, &(md_p->nonstandards[0]));
   }
 
@@ -459,7 +467,7 @@ vrna_md_defaults_temperature(double T)
     temperature = T;
 #endif
   } else {
-    vrna_message_warning(
+    vrna_log_warning(
       "vrna_md_defaults_temperature@model.c: Temperature out of range, T must be above absolute zero. Not changing anything!");
   }
 }
@@ -509,7 +517,7 @@ vrna_md_defaults_dangles(int d)
     dangles = d;
 #endif
   } else {
-    vrna_message_warning(
+    vrna_log_warning(
       "vrna_md_defaults_dangles@model.c: Dangles out of range, must be (0 <= d <= 3). Not changing anything!");
   }
 }
@@ -626,6 +634,20 @@ vrna_md_defaults_circ_get(void)
 }
 
 
+PUBLIC void
+vrna_md_defaults_circ_penalty(int flag)
+{
+  defaults.circ_penalty = flag ? 1 : 0;
+}
+
+
+PUBLIC int
+vrna_md_defaults_circ_penalty_get(void)
+{
+  return defaults.circ_penalty;
+}
+
+
 PUBLIC void
 vrna_md_defaults_gquad(int flag)
 {
@@ -671,7 +693,7 @@ vrna_md_defaults_energy_set(int e)
     /* update pair/rtype/alias arrays accordingly */
     vrna_md_update(&defaults);
   } else {
-    vrna_message_warning(
+    vrna_log_warning(
       "vrna_md_defaults_energy_set@model.c: Energy Set out of range, must be (0 <= e <= 3). Not changing anything!");
   }
 }
@@ -711,7 +733,7 @@ vrna_md_defaults_backtrack_type(char t)
 #endif
       break;
     default:
-      vrna_message_warning(
+      vrna_log_warning(
         "vrna_md_defaults_backtrack_type@model.c: Backtrack type must be any of 'F', 'C', or 'M'. Not changing anything!");
   }
 }
@@ -971,6 +993,20 @@ vrna_md_defaults_backbone_length_get(void)
   return defaults.backbone_length;
 }
 
+PUBLIC void
+vrna_md_defaults_circ_alpha0(double a0)
+{
+  defaults.circ_alpha0 = a0;
+}
+
+
+PUBLIC double
+vrna_md_defaults_circ_alpha0_get(void)
+{
+  return defaults.circ_alpha0;
+}
+
+
 PUBLIC void
 vrna_md_update(vrna_md_t *md)
 {
@@ -1049,7 +1085,7 @@ fill_pair_matrices(vrna_md_t *md)
       break;
 
     default:
-      vrna_message_warning("vrna_md_update: "
+      vrna_log_warning("vrna_md_update: "
                            "Unknown energy_set = %d. "
                            "Using defaults!",
                            md->energy_set);
@@ -1130,6 +1166,7 @@ set_model_details(vrna_md_t *md)
     md->logML           = logML;
     md->gquad           = gquad;
     md->circ            = circ;
+    md->circ_penalty    = VRNA_MODEL_DEFAULT_CIRC_PENALTY;
     md->uniq_ML         = uniq_ML;
     md->compute_bpp     = do_backtrack;
     md->backtrack       = VRNA_MODEL_DEFAULT_BACKTRACK;
@@ -1153,6 +1190,7 @@ set_model_details(vrna_md_t *md)
     md->saltDPXInitFact = defaults.saltDPXInitFact;
     md->helical_rise    = defaults.helical_rise;
     md->backbone_length = defaults.backbone_length;
+    md->circ_alpha0     = defaults.circ_alpha0;
 
     if (nonstandards)
       copy_nonstandards(md, nonstandards);
