@@ -220,6 +220,11 @@ typedef struct vrna_md_s vrna_md_t;
 #define VRNA_MODEL_DEFAULT_BACKBONE_LENGTH   VRNA_MODEL_BACKBONE_LENGTH_RNA
 
 
+#define VRNA_MODEL_DEFAULT_CIRC_PENALTY   1
+
+#define VRNA_MODEL_DEFAULT_CIRC_ALPHA0    4.385
+
+
 #ifndef VRNA_DISABLE_BACKWARD_COMPATIBILITY
 
 #ifndef MAXALPHA
@@ -274,6 +279,7 @@ struct vrna_md_s {
   int     noGUclosure;                      /**<  @brief  Do not allow loops to be closed by GU pair */
   int     logML;                            /**<  @brief  Use logarithmic scaling for multiloops */
   int     circ;                             /**<  @brief  Assume RNA to be circular instead of linear */
+  int     circ_penalty;                     /**<  @brief  Add an entropic penalty to the unpaired circRNA chain */ 
   int     gquad;                            /**<  @brief  Include G-quadruplexes in structure prediction */
   int     uniq_ML;                          /**<  @brief  Flag to ensure unique multi-branch loop decomposition during folding */
   int     energy_set;                       /**<  @brief  Specifies the energy set that defines set of compatible base pairs */
@@ -308,6 +314,8 @@ struct vrna_md_s {
   float   saltDPXInitFact;                  /**<  @brief  */
   float   helical_rise;                     /**<  @brief  */
   float   backbone_length;                  /**<  @brief  */
+
+  double  circ_alpha0;
 };
 
 
@@ -599,6 +607,28 @@ int
 vrna_md_defaults_circ_get(void);
 
 
+/**
+ *  @brief  Set default behavior for fully unpaired circular RNA penalty
+ *
+ *  @see  vrna_E_exterior_loop(), vrna_md_defaults_reset(), vrna_md_set_default(), #vrna_md_t, #VRNA_MODEL_DEFAULT_CIRC_PENALTY
+ *
+ *  @param  flag  On/Off switch (0 = OFF, else = ON)
+ */
+void
+vrna_md_defaults_circ_penalty(int flag);
+
+
+/**
+ *  @brief  Get default behavior for fully unpaired circular RNA penalty
+ *
+ *  @see vrna_md_defaults_circ_penalty(), vrna_md_defaults_reset(), vrna_md_set_default(), #vrna_md_t, #VRNA_MODEL_DEFAULT_CIRC
+ *
+ *  @return The global default settings for adding an entropic penalty for unpaired circular RNA ring
+ */
+int
+vrna_md_defaults_circ_penalty_get(void);
+
+
 /**
  *  @brief  Set default behavior for treatment of G-Quadruplexes
  *
@@ -1007,6 +1037,14 @@ float
 vrna_md_defaults_backbone_length_get(void);
 
 
+void
+vrna_md_defaults_circ_alpha0(double a0);
+
+
+double
+vrna_md_defaults_circ_alpha0_get(void);
+
+
 #ifndef VRNA_DISABLE_BACKWARD_COMPATIBILITY
 
 #define model_detailsT        vrna_md_t               /* restore compatibility of struct rename */
