@@ -434,7 +434,9 @@ get_scaled_params(vrna_md_t *md)
     for (j = 3 * VRNA_GQUAD_MIN_LINKER_LENGTH; j <= 3 * VRNA_GQUAD_MAX_LINKER_LENGTH; j++) {
       double  GQuadAlpha_T  = RESCALE_dG(GQuadAlpha37, GQuadAlphadH, tempf);
       double  GQuadBeta_T   = RESCALE_dG(GQuadBeta37, GQuadBetadH, tempf);
-      params->gquad[i][j] = (int)GQuadAlpha_T * (i - 1) + (int)(((double)GQuadBeta_T) * log(j - 2));
+      double  GQuadEnergy   = (double)GQuadAlpha_T * (double)(i - 1) +
+                              (double)GQuadBeta_T * log((double)j - 2.);
+      params->gquad[i][j]   = (int)GQuadEnergy;
     }
 
   for (i = 0; i < 31; i++)
@@ -528,7 +530,7 @@ get_scaled_params(vrna_md_t *md)
       params->dangle3[i][j] = (dd > 0) ? 0 : dd;  /* must be <= 0 */
     }
 
-  /* interior 1x1 loops */
+  /* internal 1x1 loops */
   for (i = 0; i <= NBPAIRS; i++)
     for (j = 0; j <= NBPAIRS; j++)
       for (k = 0; k < 5; k++)
@@ -537,7 +539,7 @@ get_scaled_params(vrna_md_t *md)
                                                  int11_dH[i][j][k][l],
                                                  tempf);
 
-  /* interior 2x1 loops */
+  /* internal 2x1 loops */
   for (i = 0; i <= NBPAIRS; i++)
     for (j = 0; j <= NBPAIRS; j++)
       for (k = 0; k < 5; k++)
@@ -549,7 +551,7 @@ get_scaled_params(vrna_md_t *md)
                                                       tempf);
         }
 
-  /* interior 2x2 loops */
+  /* internal 2x2 loops */
   for (i = 0; i <= NBPAIRS; i++)
     for (j = 0; j <= NBPAIRS; j++)
       for (k = 0; k < 5; k++)
@@ -633,12 +635,12 @@ get_scaled_exp_params(vrna_md_t *md,
     for (j = 3 * VRNA_GQUAD_MIN_LINKER_LENGTH; j <= 3 * VRNA_GQUAD_MAX_LINKER_LENGTH; j++) {
       double  GQuadAlpha_T  = RESCALE_dG(GQuadAlpha37, GQuadAlphadH, TT);
       double  GQuadBeta_T   = RESCALE_dG(GQuadBeta37, GQuadBetadH, TT);
-      GT = ((double)GQuadAlpha_T) * ((double)(i - 1)) + ((double)GQuadBeta_T) *
-           log(((double)j) - 2.);
-      pf->expgquad[i][j] = exp(-TRUNC_MAYBE(GT) * 10. / kT);
+      double  GQuadEnergy   = (double)GQuadAlpha_T * (double)(i - 1) +
+                              (double)GQuadBeta_T * log((double)j - 2.);
+      pf->expgquad[i][j] = exp(-TRUNC_MAYBE((double)((int)GQuadEnergy)) * 10. / kT);
     }
 
-  /* loop energies: hairpins, bulges, interior, mulit-loops */
+  /* loop energies: hairpins, bulges, internal, mulit-loops */
   for (i = 0; i < 31; i++)
     pf->exphairpin[i] = RESCALE_BF(hairpin37[i], hairpindH[i], TT, kT);
 
@@ -671,7 +673,7 @@ get_scaled_exp_params(vrna_md_t *md,
     }
   }
 
-  /* special case of size 2 interior loops (single mismatch) */
+  /* special case of size 2 internal loops (single mismatch) */
   if (james_rule)
     pf->expinternal[2] = exp(-80 * 10. / kT);
 
@@ -756,7 +758,7 @@ get_scaled_exp_params(vrna_md_t *md,
         }
       }
 
-  /* interior lops of length 2 */
+  /* internal lops of length 2 */
   for (i = 0; i <= NBPAIRS; i++)
     for (j = 0; j <= NBPAIRS; j++)
       for (k = 0; k < 5; k++)
@@ -767,7 +769,7 @@ get_scaled_exp_params(vrna_md_t *md,
                                                 kT);
         }
 
-  /* interior 2x1 loops */
+  /* internal 2x1 loops */
   for (i = 0; i <= NBPAIRS; i++)
     for (j = 0; j <= NBPAIRS; j++)
       for (k = 0; k < 5; k++)
@@ -781,7 +783,7 @@ get_scaled_exp_params(vrna_md_t *md,
           }
         }
 
-  /* interior 2x2 loops */
+  /* internal 2x2 loops */
   for (i = 0; i <= NBPAIRS; i++)
     for (j = 0; j <= NBPAIRS; j++)
       for (k = 0; k < 5; k++)
@@ -868,7 +870,7 @@ get_exp_params_ali(vrna_md_t    *md,
       pf->expgquad[i][j] = exp(-TRUNC_MAYBE(GT) * 10. / kTn);
     }
 
-  /* loop energies: hairpins, bulges, interior, mulit-loops */
+  /* loop energies: hairpins, bulges, internal, mulit-loops */
   for (i = 0; i < 31; i++)
     pf->exphairpin[i] = RESCALE_BF(hairpin37[i], hairpindH[i], TT, kTn);
   /*add penalty for too short hairpins*/
@@ -909,7 +911,7 @@ get_exp_params_ali(vrna_md_t    *md,
     }
   }
 
-  /* special case of size 2 interior loops (single mismatch) */
+  /* special case of size 2 internal loops (single mismatch) */
   if (james_rule)
     pf->expinternal[2] = exp(-80 * 10. / kTn);
 
@@ -1001,7 +1003,7 @@ get_exp_params_ali(vrna_md_t    *md,
         }
       }
 
-  /* interior lops of length 2 */
+  /* internal lops of length 2 */
   for (i = 0; i <= NBPAIRS; i++)
     for (j = 0; j <= NBPAIRS; j++)
       for (k = 0; k < 5; k++)
@@ -1012,7 +1014,7 @@ get_exp_params_ali(vrna_md_t    *md,
                                                 kTn);
         }
 
-  /* interior 2x1 loops */
+  /* internal 2x1 loops */
   for (i = 0; i <= NBPAIRS; i++)
     for (j = 0; j <= NBPAIRS; j++)
       for (k = 0; k < 5; k++)
@@ -1026,7 +1028,7 @@ get_exp_params_ali(vrna_md_t    *md,
           }
         }
 
-  /* interior 2x2 loops */
+  /* internal 2x2 loops */
   for (i = 0; i <= NBPAIRS; i++)
     for (j = 0; j <= NBPAIRS; j++)
       for (k = 0; k < 5; k++)
@@ -1076,7 +1078,7 @@ get_exp_params_ali(vrna_md_t    *md,
 PRIVATE void
 rescale_params(vrna_fold_compound_t *vc)
 {
-  int               i;
+  unsigned int      i;
   vrna_exp_param_t  *pf = vc->exp_params;
   vrna_mx_pf_t      *m  = vc->exp_matrices;
 
