@@ -15,7 +15,8 @@
 #include <string.h>
 
 #include "ViennaRNA/utils/basic.h"
-#include "ViennaRNA/gquad.h"
+#include "ViennaRNA/utils/log.h"
+#include "ViennaRNA/structures/dotbracket.h"
 
 #include "ViennaRNA/plotting/layouts.h"
 
@@ -192,7 +193,7 @@ vrna_plot_coords_pt(const short *pt,
         return vrna_plot_coords_naview_pt(pt, x, y);
 #else
       case VRNA_PLOT_TYPE_NAVIEW:
-        vrna_message_warning("Naview layout algorithm not available in this version of ViennaRNA Package");
+        vrna_log_warning("Naview layout algorithm not available in this version of ViennaRNA Package");
         break;
 
       default:
@@ -260,12 +261,13 @@ rna_layout(const char   *structure,
 {
   struct vrna_plot_layout_s *layout;
   short                     *pt, *pt_g;
-  unsigned int              i, n;
-  int                       xmin, xmax, ymin, ymax, ee, gb, ge, Lg, l[3], r;
+  unsigned int              i, n, ee, Lg, l[3];
+  int                       xmin, xmax, ymin, ymax, gb, ge, r;
 
 
   n               = strlen(structure);
   layout          = (struct vrna_plot_layout_s *)vrna_alloc(sizeof(struct vrna_plot_layout_s));
+  layout->type    = plot_type;
   layout->length  = n;
   layout->x       = NULL;
   layout->y       = NULL;
@@ -277,13 +279,21 @@ rna_layout(const char   *structure,
 
   /* account for possible G-Quadruplexes in dot-bracket string */
   ge = 0;
-  while ((ee = parse_gquad(structure + ge, &Lg, l)) > 0) {
+  while ((ee = vrna_gq_parse(structure + ge, &Lg, l)) > 0) {
     ge  += ee;
-    gb  = ge - Lg * 4 - l[0] - l[1] - l[2] + 1;
+    if (4 * Lg + l[0] + l[1] + l[2] > ee) {
+      gb = n + ge - 4 * Lg - l[0] - l[1] - l[2] + 1;
+    } else {
+      gb  = ge - Lg * 4 - l[0] - l[1] - l[2] + 1;
+    }
+
     /* add pseudo-base pair encloding gquad */
-    for (i = 0; i < (unsigned int)Lg; i++) {
-      pt_g[ge - i]  = gb + i;
-      pt_g[gb + i]  = ge - i;
+    for (i = 0; i < Lg; i++) {
+      unsigned int ii, jj;
+      ii = (gb + i - 1) % n + 1;
+      jj = (n + ge - i - 1) % n + 1;
+      pt_g[ii]  = jj;
+      pt_g[jj]  = ii;
     }
   }
 
@@ -332,7 +342,7 @@ rna_layout(const char   *structure,
 #else
     case VRNA_PLOT_TYPE_NAVIEW:
       i = 0;
-      vrna_message_warning("Naview layout algorithm not available in this version of ViennaRNA Package");
+      vrna_log_warning("Naview layout algorithm not available in this version of ViennaRNA Package");
       break;
 
     default:
@@ -348,7 +358,7 @@ rna_layout(const char   *structure,
   }
 
   if (i != n) {
-    vrna_message_warning("strange things happening in vrna_plot_layout*()...");
+    vrna_log_warning("strange things happening in vrna_plot_layout*()...");
     layout->bbox[0] = layout->bbox[1] = layout->bbox[2] = layout->bbox[3] = 0;
   } else {
     /* adjust bounding box coordinates */
