@@ -43,6 +43,7 @@
 #include <math.h>
 
 #include "ViennaRNA/utils/basic.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/plotting/naview/naview.h"
 
 typedef int LOGICAL;
@@ -359,8 +360,10 @@ static struct loop *construct_loop(int ibase)
 	  lp = construct_loop(rp->end2 < nbase ? rp->end2+1 : 0);
 	}
 	else {
-	  vrna_message_error("naview: Error detected in construct_loop. i = %d not found in region table.",i);
-	  exit(FATAL_ERROR);
+	  vrna_log_error("Error detected in construct_loop. i = %d not found in region table.",i);
+    free(retloop->connections);
+    retloop->connections = NULL;
+    return NULL;
 	}
 	retloop->connections = (struct connection **)
 	  realloc(retloop->connections,
@@ -846,7 +849,7 @@ static void traverse_loop(struct loop *lp, struct connection *anchor_connection)
     if (dcp <= 0.0) dcp += 2*pi;
     if (fabs(dan-dcp) > pi) {
       if (cp->extruded) {
-        vrna_message_warning("...from traverse_loop. Loop %d has crossed regions",
+        vrna_log_warning("...from traverse_loop. Loop %d has crossed regions",
                                     lp->number);
       }
       else if ((cpnext->start - cp->end) != 1) {
@@ -1006,7 +1009,7 @@ static int    find_ic_middle(int icstart, int icend, struct connection *anchor_c
   while (!done) {
     if (count++ > lp->nconnection * 2) {
       printf("Infinite loop detected in find_ic_middle\n");
-      exit(FATAL_ERROR);
+      return ret;
     }
     if (anchor_connection != NULL && lp->connections[ic] == acp) {
       ret = ic;
@@ -1049,7 +1052,7 @@ static void generate_region(struct connection *cp)
   if (bases[cp->start].x > anum - 100.0 ||
       bases[cp->end].x > anum - 100.0) {
     printf("Bad region passed to generate_region. Coordinates not defined.\n");
-    exit(FATAL_ERROR);
+    return;
   }
   for (i=start+1; i<=end; i++) {
     l++;
@@ -1224,8 +1227,8 @@ static void find_center_for_arc(int n,double b,double *hp,double *thetap)
     /*  if (r<0.5) {r = 0.5; h = 0.5*sqrt(1-b*b);} */
     disc = 1.0 - 0.5/(r*r);
     if (fabs(disc) > 1.0) {
-      vrna_message_error("Unexpected large magnitude discriminant = %g %g", disc,r);
-      exit(FATAL_ERROR);
+      vrna_log_error("Unexpected large magnitude discriminant = %g %g", disc,r);
+      return;
     }
     theta = acos(disc);
     /*    theta = 2*acos(sqrt(1-1/(4*r*r))); */
@@ -1239,7 +1242,7 @@ static void find_center_for_arc(int n,double b,double *hp,double *thetap)
     }
   } while    (fabs(e) > 0.0001 && ++iter < maxiter);
   if (iter >= maxiter) {
-    vrna_message_warning("Iteration failed in find_center_for_arc");
+    vrna_log_warning("Iteration failed in find_center_for_arc");
     h = 0.0;
     theta = 0.0;
   }
