@@ -19,14 +19,23 @@
 #include "ViennaRNA/fold_vars.h"
 #include "ViennaRNA/utils/basic.h"
 #include "ViennaRNA/utils/strings.h"
-#include "ViennaRNA/utils/alignments.h"
-#include "ViennaRNA/gquad.h"
+#include "ViennaRNA/sequences/alignments.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/plotting/probabilities.h"
 
 #include "ViennaRNA/static/templates_postscript.h"
 
 #include "ViennaRNA/plotting/ps_helpers.inc"
 
+
+#ifndef INLINE
+#ifdef __GNUC__
+# define INLINE inline
+#else
+# define INLINE
+#endif
+#endif
+
 /*
 #################################
 # PRIVATE MACROS                #
@@ -171,11 +180,19 @@ PS_color_dot_plot(char *seq,
     } else if ((pi[i].type == VRNA_PLIST_TYPE_BASEPAIR) ||
                (pi[i].type == VRNA_PLIST_TYPE_TRIPLE)) {
       fprintf(wastl,"%1.2f %1.2f hsb %d %d %1.6f ubox\n",
-             pi[i].hue, pi[i].sat, pi[i].i, pi[i].j, sqrt(pi[i].p));
+             pi[i].hue,
+             pi[i].sat,
+             (pi[i].i < pi[i].j) ? pi[i].i : pi[i].j,
+             (pi[i].i < pi[i].j) ? pi[i].j : pi[i].i,
+             sqrt(pi[i].p));
 
       if (pi[i].mfe)
         fprintf(wastl,"%1.2f %1.2f hsb %d %d %1.4f lbox\n",
-               pi[i].hue, pi[i].sat, pi[i].i, pi[i].j, pi[i].p);
+               pi[i].hue,
+               pi[i].sat,
+               (pi[i].i < pi[i].j) ? pi[i].i : pi[i].j,
+               (pi[i].i < pi[i].j) ? pi[i].j : pi[i].i,
+               pi[i].p);
     }
     i++;
   }
@@ -208,13 +225,13 @@ vrna_plot_dp_PS_list( char *seq,
 
   FILE          *wastl;
   size_t        cnt;
-  char          *seq_plain, *tmp, **seqs;
+  char          *seq_plain, **seqs;
   unsigned int  *nicks, curr_length;
   int           pl_size, gq_num;
   plist         *pl1;
 
   nicks     = NULL;
-  seq_plain = tmp = NULL;
+  seq_plain = NULL;
   seqs      = vrna_strsplit(seq, "&");
 
   if (seqs) {
@@ -231,7 +248,6 @@ vrna_plot_dp_PS_list( char *seq,
       nicks[0] = curr_length + 1;
       vrna_strcat_printf(&seq_plain, "%s", seqs[1]);
       curr_length += strlen(seqs[1]);
-      free(tmp);
       free(seqs[1]);
 
       /* add all remaining sequences (so far, we do not store the individual nicks for the remaining sequences */
@@ -240,7 +256,6 @@ vrna_plot_dp_PS_list( char *seq,
         nicks[cnt - 1] = curr_length + 1;
         vrna_strcat_printf(&seq_plain, "%s", seqs[cnt]);
         curr_length += strlen(seqs[cnt]);
-        free(tmp);
         free(seqs[cnt++]);
       }
     }
@@ -301,7 +316,7 @@ vrna_plot_dp_EPS( const char              *filename,
   
   fh = fopen(filename, "w");
   if(!fh){
-    vrna_message_warning("can't open %s for dot plot", filename);
+    vrna_log_warning("can't open %s for dot plot", filename);
     return 0; /* return 0 for failure */
   }
 
@@ -482,11 +497,19 @@ PS_color_dot_plot_turn( char *seq,
    i=0;
    while (pi[i].j>0) {
      fprintf(wastl,"%1.2f %1.2f hsb %d %d %1.6f ubox\n",
-             pi[i].hue, pi[i].sat, pi[i].i, pi[i].j, sqrt(pi[i].p));/*sqrt??*/
+             pi[i].hue,
+             pi[i].sat,
+             (pi[i].i < pi[i].j) ? pi[i].i : pi[i].j,
+             (pi[i].i < pi[i].j) ? pi[i].j : pi[i].i,
+             sqrt(pi[i].p));
 
      if (pi[i].mfe)
        fprintf(wastl,"%1.2f %1.2f hsb %d %d %1.4f lbox\n",
-               pi[i].hue, pi[i].sat, pi[i].i, pi[i].j, pi[i].p);
+               pi[i].hue,
+               pi[i].sat,
+               (pi[i].i < pi[i].j) ? pi[i].i : pi[i].j,
+               (pi[i].i < pi[i].j) ? pi[i].j : pi[i].i,
+               pi[i].p);
      i++;
    }
 
@@ -534,7 +557,9 @@ PS_dot_plot_turn( char *seq,
   if (pl) {
     while (pl[i].j>0) {
       fprintf(wastl,"%d %d %1.4f ubox\n",
-              pl[i].i, pl[i].j, sqrt(pl[i].p));
+             (pl[i].i < pl[i].j) ? pl[i].i : pl[i].j,
+             (pl[i].i < pl[i].j) ? pl[i].j : pl[i].i,
+              sqrt(pl[i].p));
       i++;
     }
   }
@@ -650,7 +675,7 @@ EPS_print_sc_motif_data(FILE          *eps,
       }
     }
 
-  /* print triangles for interior loop motifs in upper half */
+  /* print triangles for internal loop motifs in upper half */
   fprintf(eps,"\n%%start of Imotif data\n");
   int   a,b;
   float ppp;
@@ -702,7 +727,11 @@ EPS_print_bpp_data( FILE          *eps,
       tmp = sqrt(pl1->p);
       if ((pl1->type == VRNA_PLIST_TYPE_BASEPAIR) ||
           (pl1->type == VRNA_PLIST_TYPE_TRIPLE))
-          fprintf(eps,"%d %d %1.9f ubox\n", pl1->i, pl1->j, tmp);
+          fprintf(eps,
+                  "%d %d %1.9f ubox\n",
+                  (pl1->i < pl1->j) ? pl1->i : pl1->j,
+                  (pl1->i < pl1->j) ? pl1->j : pl1->i,
+                  tmp);
     }
 
 
@@ -712,7 +741,11 @@ EPS_print_bpp_data( FILE          *eps,
       tmp = sqrt(pl1->p);
       if ((pl1->type == VRNA_PLIST_TYPE_BASEPAIR) ||
           (pl1->type == VRNA_PLIST_TYPE_TRIPLE))
-        fprintf(eps,"%d %d %1.7f lbox\n", pl1->i, pl1->j, tmp);
+        fprintf(eps,
+                "%d %d %1.7f lbox\n",
+                (pl1->i < pl1->j) ? pl1->i : pl1->j,
+                (pl1->i < pl1->j) ? pl1->j : pl1->i,
+                tmp);
     }
 }
 
@@ -1013,7 +1046,7 @@ PS_dot_common(const char    *seq,
 
   wastl = fopen(wastlfile,"w");
   if (wastl==NULL) {
-    vrna_message_warning("can't open %s for dot plot", wastlfile);
+    vrna_log_warning("can't open %s for dot plot", wastlfile);
     return NULL; /* return 0 for failure */
   }
   name = strdup(wastlfile);
@@ -1134,7 +1167,7 @@ int PS_dot_plot(char *string, char *wastlfile) {
     } else {
       mf = NULL;
     }
-    i = PS_dot_plot_list(string, wastlfile, pl, mf, "");
+    i = vrna_plot_dp_PS_list(string, cut_point, wastlfile, pl, mf, "");
     free(mf);
     free(pl);
   }
