@@ -8,8 +8,9 @@
  */
 
 #include <ViennaRNA/datastructures/basic.h>
+#include <ViennaRNA/datastructures/string.h>
 #include <ViennaRNA/model.h>
-#include <ViennaRNA/utils/structures.h>
+#include <ViennaRNA/structures/problist.h>
 
 /**
  *  @addtogroup  annotation_utils
@@ -32,6 +33,12 @@ vrna_annotate_covar_db_extended(const char   **alignment,
                                 vrna_md_t    *md_p,
                                 unsigned int options);
 
+vrna_string_t *
+vrna_annotate_covar_pt(const char       **alignment,
+                       const short int  *pt,
+                       vrna_md_t        *md_p,
+                       double           threshold,
+                       double           min_sat);
 
 /**
  *  @brief  Produce covariance annotation for an alignment given a set of base pairs
