@@ -19,13 +19,18 @@
 #include <ctype.h>
 #include <string.h>
 #include "ViennaRNA/utils/basic.h"
-#include "ViennaRNA/utils/structures.h"
+#include "ViennaRNA/structures/pairtable.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/params/default.h"
+#include "ViennaRNA/datastructures/dp_matrices.h"
 #include "ViennaRNA/fold_vars.h"
 #include "ViennaRNA/pair_mat.h"
 #include "ViennaRNA/params/basic.h"
 #include "ViennaRNA/snofold.h"
-#include "ViennaRNA/loops/all.h"
+#include "ViennaRNA/eval/exterior.h"
+#include "ViennaRNA/eval/hairpin.h"
+#include "ViennaRNA/eval/internal.h"
+#include "ViennaRNA/eval/multibranch.h"
 
 #ifdef __GNUC__
 #define INLINE inline
@@ -158,11 +163,6 @@ PRIVATE char  alpha[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
  * PRIVATE int min_hairpin = TURN;
  */
 
-/*
- * some definitions to take circfold into account...
- * PRIVATE int   *fM2 = NULL;*/        /* fM2 = multiloop region with exactly two stems, extending to 3' end
- */
-PUBLIC int Fc, FcH, FcI, FcM;          /* parts of the exterior loop energies                        */
 /*--------------------------------------------------------------------------*/
 
 void
@@ -171,7 +171,7 @@ snoinitialize_fold(const int length)
   unsigned int n;
 
   if (length < 1)
-    vrna_message_error("snoinitialize_fold: argument must be greater 0");
+    vrna_log_error("snoinitialize_fold: argument must be greater 0");
 
   if (init_length > 0)
     snofree_arrays(length);
@@ -191,7 +191,7 @@ alisnoinitialize_fold(const int length)
   unsigned int n;
 
   if (length < 1)
-    vrna_message_error("snoinitialize_fold: argument must be greater 0");
+    vrna_log_error("snoinitialize_fold: argument must be greater 0");
 
   if (init_length > 0)
     snofree_arrays(length);
@@ -367,6 +367,7 @@ snofold(const char  *string,
         const int   max_half_stem)
 {
   int length, energy, bonus, bonus_cnt, s;
+  vrna_md_t   md;
 
   /* Variable initialization */
   bonus     = 0;
@@ -382,9 +383,20 @@ snofold(const char  *string,
 
   if (length > init_length)
     snoinitialize_fold(length);
-  else if (fabs(P->temperature - temperature) > 1e-6)
-    snoupdate_fold_params();
-
+  else {
+    if (!P) {
+      snoupdate_fold_params();
+    } else {
+      set_model_details(&md);
+      md.window_size = P->model_details.window_size;
+      md.max_bp_span = P->model_details.max_bp_span;
+      /* check if model_details are the same as before */
+      if (memcmp(&md, &(P->model_details), sizeof(vrna_md_t))) {
+        snoupdate_fold_params();
+      }
+    }
+  }
+  
   /* encode_seq(string); */
   BP = (int *)vrna_alloc(sizeof(int) * (length + 2));
   make_ptypes(S, structure);
@@ -507,14 +519,14 @@ make_pscores(int                n,
           break;
         case ']':
           if (hx2 <= 0)
-            vrna_message_error("unbalanced brackets in constraints\n%s", structure);
+            vrna_log_error("unbalanced brackets in constraints\n%s", structure);
 
           i                   = stack2[--hx2];
           pscore[indx[j] + i] = NONE;
           break;
         case ')':
           if (hx <= 0)
-            vrna_message_error("unbalanced brackets in constraints\n%s", structure);
+            vrna_log_error("unbalanced brackets in constraints\n%s", structure);
 
           i     = stack[--hx];
           psij  = pscore[indx[j] + i]; /* store for later */
@@ -535,7 +547,7 @@ make_pscores(int                n,
       }
     }
     if (hx != 0)
-      vrna_message_error("unbalanced brackets in constraint string\n%s", structure);
+      vrna_log_error("unbalanced brackets in constraint string\n%s", structure);
 
     free(stack);
     free(stack2);
@@ -554,6 +566,7 @@ alisnofold(const char **strings,
 {
   int   s, n_seq, length, energy;
   char  *structure;
+  vrna_md_t md;
 
   length = (int)strlen(strings[0]);
   /* structure = (char *) vrna_alloc((unsigned) length+1); */
@@ -561,15 +574,24 @@ alisnofold(const char **strings,
   if (length > init_length)
     alisnoinitialize_fold(length);
 
-  if (fabs(P->temperature - temperature) > 1e-6)
+  if (!P) {
     snoupdate_fold_params();
+  } else {
+    set_model_details(&md);
+    md.window_size = P->model_details.window_size;
+    md.max_bp_span = P->model_details.max_bp_span;
+    /* check if model_details are the same as before */
+    if (memcmp(&md, &(P->model_details), sizeof(vrna_md_t))) {
+      snoupdate_fold_params();
+    }
+  }
 
   for (s = 0; strings[s] != NULL; s++);
   n_seq = s;
   Sali  = (short **)vrna_alloc(n_seq * sizeof(short *));
   for (s = 0; s < n_seq; s++) {
     if (strlen(strings[s]) != length)
-      vrna_message_error("uneqal seqence lengths");
+      vrna_log_error("uneqal seqence lengths");
 
     Sali[s] = aliencode_seq(strings[s]);
   }
@@ -630,7 +652,7 @@ alifill_arrays(const char **strings,
         /* hairpin ----------------------------------------------*/
 
         for (new_c = s = 0; s < n_seq; s++)
-          new_c += E_Hairpin(j - i - 1,
+          new_c += vrna_E_hairpin(j - i - 1,
                              type[s],
                              Sali[s][i + 1],
                              Sali[s][j - 1],
@@ -638,7 +660,7 @@ alifill_arrays(const char **strings,
                              P);
         /*--------------------------------------------------------
          *  check for elementary structures involving more than one
-         *  closing pair (interior loop).
+         *  closing pair (internal loop).
          *  --------------------------------------------------------*/
 
         for (p = i + 1; p <= MIN2(j - 2 - TURN, i + MAXLOOP + 1); p++) {
@@ -658,7 +680,7 @@ alifill_arrays(const char **strings,
               if (type_2 == 0)
                 type_2 = 7;
 
-              energy += E_IntLoop(p - i - 1, j - q - 1, type[s], type_2,
+              energy += vrna_E_internal(p - i - 1, j - q - 1, type[s], type_2,
                                   Sali[s][i + 1], Sali[s][j - 1],
                                   Sali[s][p - 1], Sali[s][q + 1], P);
             }
@@ -760,7 +782,7 @@ alibacktrack(const char **strings,
   if (s == 0) {
     sector[++s].i = 1;
     sector[s].j   = length;
-    sector[s].ml  = 2;
+    sector[s].ml  = VRNA_MX_FLAG_C;
   }
 
   while (s > 0) {
@@ -770,7 +792,7 @@ alibacktrack(const char **strings,
     j   = sector[s].j;
     ml  = sector[s--].ml;  /* ml is a flag indicating if backtracking is to
                            * occur in the fML- (1) or in the f-array (0) */
-    if (ml == 2) {
+    if (ml == VRNA_MX_FLAG_C) {
       base_pair[++b].i  = i;
       base_pair[b].j    = j;
       goto repeat1;
@@ -819,7 +841,7 @@ repeat1:
     {
       int cc = 0;
       for (ss = 0; ss < n_seq; ss++)
-        cc += E_Hairpin(j - i - 1,
+        cc += vrna_E_hairpin(j - i - 1,
                         type[ss],
                         Sali[ss][i + 1],
                         Sali[ss][j - 1],
@@ -840,7 +862,7 @@ repeat1:
           if (type_2 == 0)
             type_2 = 7;
 
-          energy += E_IntLoop(p - i - 1, j - q - 1, type[ss], type_2,
+          energy += vrna_E_internal(p - i - 1, j - q - 1, type[ss], type_2,
                               Sali[ss][i + 1], Sali[ss][j - 1],
                               Sali[ss][p - 1], Sali[ss][q + 1], P);
         }
@@ -866,7 +888,7 @@ repeat1:
      */
     i1                = i + 1;
     j1                = j - 1;
-    sector[s + 1].ml  = sector[s + 2].ml = 1;
+    sector[s + 1].ml  = sector[s + 2].ml = VRNA_MX_FLAG_M;
 
     /*
      *      if (k<=j-3-TURN) { /\\* found the decomposition *\\/ *\/
@@ -940,11 +962,11 @@ fill_arrays(const char  *string,
         if (no_close)
           new_c = FORBIDDEN;
         else
-          new_c = E_Hairpin(j - i - 1, type, S1[i + 1], S1[j - 1], string + i - 1, P); /* computes hair pin structure for subsequence i...j */
+          new_c = vrna_E_hairpin(j - i - 1, type, S1[i + 1], S1[j - 1], string + i - 1, P); /* computes hair pin structure for subsequence i...j */
 
         /*--------------------------------------------------------
          *  check for elementary structures involving more than one
-         *  closing pair (interior loop).
+         *  closing pair (internal loop).
          *  --------------------------------------------------------*/
 
         for (p = i + 1; p <= MIN2(j - 2 - TURN, i + MAXLOOP + 1); p++) {
@@ -970,7 +992,7 @@ fill_arrays(const char  *string,
 
             /* continue unless stack */
 
-            energy = E_IntLoop(p - i - 1, j - q - 1, type, type_2,
+            energy = vrna_E_internal(p - i - 1, j - q - 1, type, type_2,
                                S1[i + 1], S1[j - 1], S1[p - 1], S1[q + 1], P);
             new_c = MIN2(energy + c[indx[q] + p], new_c);
             if ((p == i + 1) && (j == q + 1))
@@ -1125,17 +1147,18 @@ backtrack(const char  *string,
   if (s == 0) {
     sector[++s].i = 1;
     sector[s].j   = length;
-    sector[s].ml  = 2;
+    sector[s].ml  = VRNA_MX_FLAG_C;
   }
 
   while (s > 0) {
-    int ml, cij, traced, i1, j1, /*d3, d5, mm,*/ p, q;
+    unsigned int ml;
+    int cij, traced, i1, j1, /*d3, d5, mm,*/ p, q;
     int canonical = 1;     /* (i,j) closes a canonical structure */
     i   = sector[s].i;
     j   = sector[s].j;
     ml  = sector[s--].ml;  /* ml is a flag indicating if backtracking is to
                            * occur in the fML- (1) or in the f-array (0) */
-    if (ml == 2) {
+    if (ml == VRNA_MX_FLAG_C) {
       base_pair[++b].i  = i;
       base_pair[b].j    = j;
       goto repeat1;
@@ -1182,7 +1205,7 @@ repeat1:
       if (cij == FORBIDDEN)
         continue;
     } else
-    if (cij == E_Hairpin(j - i - 1, type, S1[i + 1], S1[j - 1], string + i - 1, P) + bonus) {
+    if (cij == vrna_E_hairpin(j - i - 1, type, S1[i + 1], S1[j - 1], string + i - 1, P) + bonus) {
       continue;
     }
 
@@ -1205,7 +1228,7 @@ repeat1:
 
         /* continue unless stack */
 
-        energy = E_IntLoop(p - i - 1, j - q - 1, type, type_2,
+        energy = vrna_E_internal(p - i - 1, j - q - 1, type, type_2,
                            S1[i + 1], S1[j - 1], S1[p - 1], S1[q + 1], P);
         new     = energy + c[indx[q] + p] + bonus;
         traced  = (cij == new);
@@ -1229,7 +1252,7 @@ repeat1:
      */
     i1                = i + 1;
     j1                = j - 1;
-    sector[s + 1].ml  = sector[s + 2].ml = 1;
+    sector[s + 1].ml  = sector[s + 2].ml = VRNA_MX_FLAG_M;
 
     /*
      *      if (k<=j-3-TURN) { */ /* found the decomposition
@@ -1257,7 +1280,7 @@ snobacktrack_fold_from_pair(const char  *sequence,
 
   sector[1].i     = i;
   sector[1].j     = j;
-  sector[1].ml    = 2;
+  sector[1].ml    = VRNA_MX_FLAG_C;
   base_pair[0].i  = 0;
   encode_seq(sequence);
   backtrack(sequence, 1);
@@ -1282,13 +1305,13 @@ alisnobacktrack_fold_from_pair(const char **strings,
   n_seq           = s;
   sector[1].i     = i;
   sector[1].j     = j;
-  sector[1].ml    = 2;
+  sector[1].ml    = VRNA_MX_FLAG_C;
   base_pair[0].i  = 0;
   /* encode_seq(sequence); */
   Sali = (short **)vrna_alloc(n_seq * sizeof(short *));
   for (s = 0; s < n_seq; s++) {
     if (strlen(strings[s]) != length)
-      vrna_message_error("uneqal seqence lengths");
+      vrna_log_error("uneqal seqence lengths");
 
     Sali[s] = aliencode_seq(strings[s]);
   }
