@@ -28,7 +28,10 @@
 #include <unistd.h>
 #endif
 
+#include "ViennaRNA/intern/color_output.h"
+
 #include "ViennaRNA/io/utils.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/utils/basic.h"
 
 #ifdef WITH_DMALLOC
@@ -38,10 +41,6 @@
 #define PRIVATE  static
 #define PUBLIC
 
-#define EXIT_ON_ERROR
-
-#include "ViennaRNA/color_output.inc"
-
 /*
  #################################
  # GLOBAL VARIABLES              #
@@ -80,20 +79,20 @@ rj_mix(uint32_t a,
 /* include the following two functions only if not including <dmalloc.h> */
 
 PUBLIC void *
-vrna_alloc(unsigned size)
+vrna_alloc(size_t size)
 {
   void *pointer;
 
-  if ((pointer = (void *)calloc(1, (size_t)size)) == NULL) {
+  if ((pointer = (void *)calloc(1, size)) == NULL) {
 #ifdef EINVAL
     if (errno == EINVAL) {
       fprintf(stderr, "vrna_alloc: requested size: %d\n", size);
-      vrna_message_error("Memory allocation failure -> EINVAL");
+      ("Memory allocation failure -> EINVAL");
     }
 
     if (errno == ENOMEM)
 #endif
-    vrna_message_error("Memory allocation failure -> no memory");
+    vrna_log_error("Memory allocation failure -> no memory");
   }
 
   return pointer;
@@ -102,7 +101,7 @@ vrna_alloc(unsigned size)
 
 PUBLIC void *
 vrna_realloc(void     *p,
-             unsigned size)
+             size_t   size)
 {
   if (p == NULL)
     return vrna_alloc(size);
@@ -112,12 +111,12 @@ vrna_realloc(void     *p,
 #ifdef EINVAL
     if (errno == EINVAL) {
       fprintf(stderr, "vrna_realloc: requested size: %d\n", size);
-      vrna_message_error("vrna_realloc allocation failure -> EINVAL");
+      vrna_log_error("vrna_realloc allocation failure -> EINVAL");
     }
 
     if (errno == ENOMEM)
 #endif
-    vrna_message_error("vrna_realloc allocation failure -> no memory");
+    vrna_log_error("vrna_realloc allocation failure -> no memory");
   }
 
   return p;
@@ -126,118 +125,6 @@ vrna_realloc(void     *p,
 
 #endif
 
-/*------------------------------------------------------------------------*/
-
-PUBLIC void
-vrna_message_error(const char *format,
-                   ...)
-{
-  va_list args;
-
-  va_start(args, format);
-  vrna_message_verror(format, args);
-  va_end(args);
-}
-
-
-PUBLIC void
-vrna_message_verror(const char  *format,
-                    va_list     args)
-{
-#ifndef VRNA_WITHOUT_TTY_COLORS
-  if (isatty(fileno(stderr))) {
-    fprintf(stderr, ANSI_COLOR_RED_B "ERROR: " ANSI_COLOR_RESET ANSI_COLOR_BRIGHT);
-    vfprintf(stderr, format, args);
-    fprintf(stderr, ANSI_COLOR_RESET "\n");
-  } else {
-#endif
-  fprintf(stderr, "ERROR: ");
-  vfprintf(stderr, format, args);
-  fprintf(stderr, "\n");
-#ifndef VRNA_WITHOUT_TTY_COLORS
-}
-
-
-#endif
-
-#ifdef EXIT_ON_ERROR
-  exit(EXIT_FAILURE);
-#endif
-}
-
-
-PUBLIC void
-vrna_message_warning(const char *format,
-                     ...)
-{
-  va_list args;
-
-  va_start(args, format);
-  vrna_message_vwarning(format, args);
-  va_end(args);
-}
-
-
-PUBLIC void
-vrna_message_vwarning(const char  *format,
-                      va_list     args)
-{
-#ifndef VRNA_WITHOUT_TTY_COLORS
-  if (isatty(fileno(stderr))) {
-    fprintf(stderr, ANSI_COLOR_MAGENTA_B "WARNING: " ANSI_COLOR_RESET ANSI_COLOR_BRIGHT);
-    vfprintf(stderr, format, args);
-    fprintf(stderr, ANSI_COLOR_RESET "\n");
-  } else {
-#endif
-  fprintf(stderr, "WARNING: ");
-  vfprintf(stderr, format, args);
-  fprintf(stderr, "\n");
-#ifndef VRNA_WITHOUT_TTY_COLORS
-}
-
-
-#endif
-}
-
-
-PUBLIC void
-vrna_message_info(FILE        *fp,
-                  const char  *format,
-                  ...)
-{
-  va_list args;
-
-  va_start(args, format);
-  vrna_message_vinfo(fp, format, args);
-  va_end(args);
-}
-
-
-PUBLIC void
-vrna_message_vinfo(FILE       *fp,
-                   const char *format,
-                   va_list    args)
-{
-  if (!fp)
-    fp = stdout;
-
-#ifndef VRNA_WITHOUT_TTY_COLORS
-  if (isatty(fileno(fp))) {
-    fprintf(fp, ANSI_COLOR_BLUE_B);
-    vfprintf(fp, format, args);
-    fprintf(fp, ANSI_COLOR_RESET "\n");
-  } else {
-#endif
-  vfprintf(fp, format, args);
-  fprintf(fp, "\n");
-#ifndef VRNA_WITHOUT_TTY_COLORS
-}
-
-
-#endif
-}
-
-
 /*------------------------------------------------------------------------*/
 PUBLIC void
 vrna_init_rand(void)
@@ -571,20 +458,6 @@ print_tty_input_seq_str(const char *s)
 }
 
 
-PUBLIC void
-warn_user(const char message[])
-{
-  vrna_message_warning(message);
-}
-
-
-PUBLIC void
-nrerror(const char message[])
-{
-  vrna_message_error(message);
-}
-
-
 PUBLIC void *
 space(unsigned size)
 {
