@@ -24,10 +24,12 @@
 #include "ViennaRNA/LPfold.h"
 #include "ViennaRNA/plotting/probabilities.h"
 #include "ViennaRNA/plotting/structures.h"
-#include "ViennaRNA/fold.h"
+#include "ViennaRNA/mfe/global.h"
 #include "ViennaRNA/params/io.h"
 #include "ViennaRNA/io/file_formats.h"
-#include "ViennaRNA/PKplex.h"
+#include "ViennaRNA/pk_plex.h"
+
+#include "gengetopt_helpers.h"
 #include "RNAPKplex_cmdl.h"
 
 int
@@ -55,7 +57,7 @@ main(int  argc,
   char                    *id_s1, *s1, *orig_s1, *ParamFile, *ns_bases, *plexstring,
                           *constraint, fname[FILENAME_MAX_LENGTH], *annotation, **rest;
   unsigned int            options;
-  int                     istty, i, noconv, length, pairdist, unpaired, winsize, verbose;
+  int                     istty, i, noconv, length, unpaired, verbose;
   float                   cutoff;
   double                  subopts, pk_penalty;
   vrna_md_t               md;
@@ -63,9 +65,7 @@ main(int  argc,
   options     = 0;
   subopts     = 0.0;
   dangles     = 2;
-  winsize     = 70;
-  cutoff      = 0.01;
-  pairdist    = 0;
+  cutoff      = 1e-6;
   unpaired    = 0;
   noconv      = 0;
   pk_penalty  = 8.10;
@@ -85,6 +85,9 @@ main(int  argc,
   if (PKplex_cmdline_parser(argc, argv, &args_info) != 0)
     exit(1);
 
+  /* prepare logging system and verbose mode */
+  ggo_log_settings(args_info, verbose);
+
   /* temperature */
   if (args_info.temp_given)
     md.temperature = temperature = args_info.temp_arg;
@@ -105,13 +108,15 @@ main(int  argc,
   if (args_info.noClosingGU_given)
     md.noGUclosure = no_closingGU = 1;
 
+  if (args_info.salt_given)
+    md.salt = args_info.salt_arg;
+
   /* do not convert DNA nucleotide "T" to appropriate RNA "U" */
   if (args_info.noconv_given)
     noconv = 1;
 
   /* take another energy parameter set */
-  if (args_info.paramFile_given)
-    ParamFile = strdup(args_info.paramFile_arg);
+  ggo_get_read_paramFile(args_info, &md);
 
   /* Allow other pairs in addition to the usual AU,GC,and GU pairs */
   if (args_info.nsp_given)
@@ -121,10 +126,6 @@ main(int  argc,
   if (args_info.cutoff_given)
     cutoff = args_info.cutoff_arg;
 
-  /* turn on verbose output (mainly for debugging) */
-  if (args_info.verbose_given)
-    verbose = 1;
-
   /* set energy cutoff */
   if (args_info.energyCutoff_given)
     pk_penalty = args_info.energyCutoff_arg;
@@ -133,6 +134,11 @@ main(int  argc,
   if (args_info.subopts_given)
     subopts = args_info.subopts_arg;
 
+  ggo_geometry_settings(args_info, &md);
+
+  ggo_get_md_part(args_info, md);
+
+
   /* free allocated memory of command line data structure */
   PKplex_cmdline_parser_free(&args_info);
 
@@ -141,13 +147,6 @@ main(int  argc,
    # begin initializing
    #############################################
    */
-  if (ParamFile != NULL) {
-    if (!strcmp(ParamFile, "DNA"))
-      vrna_params_load_DNA_Mathews2004();
-    else
-      vrna_params_load(ParamFile, VRNA_PARAMETER_FORMAT_DEFAULT);
-  }
-
   if (ns_bases != NULL)
     vrna_md_set_nonstandards(&md, ns_bases);
 
@@ -183,7 +182,6 @@ main(int  argc,
     strcat(fname, ".ps");
 
     length    = strlen(s1);
-    winsize   = pairdist = length;
     unpaired  = MIN2(30, length - 3);
 
     /* convert DNA alphabet to RNA if not explicitely switched off */
@@ -207,24 +205,26 @@ main(int  argc,
     if (length >= 5) {
       vrna_pk_plex_t *hits, *hit_ptr;
 
-      if (verbose)
-        printf("Winsize = %d\nPairdist = %d\nUnpaired = %d\n", winsize, pairdist, unpaired);
+      vrna_fold_compound_t  *fc;
+      fc = vrna_fold_compound(s1, &md, VRNA_OPTION_DEFAULT);
+      double mfe = vrna_mfe(fc, NULL);
 
       /*
        ########################################################
        # do Plex computations
        ########################################################
        */
-      int                   **access = vrna_pk_plex_accessibility(s1, unpaired, cutoff);
+      vrna_fold_compound_t  *fca = vrna_fold_compound(s1, &md, VRNA_OPTION_DEFAULT | VRNA_OPTION_WINDOW);
+      vrna_exp_params_rescale(fca, &mfe);
+      int                   **access = vrna_pk_plex_accessibility(fca, unpaired, cutoff);
+
+      vrna_fold_compound_free(fca);
 
       if (verbose)
         printf("EnergyCutoff = %f\n", pk_penalty);
 
-      vrna_fold_compound_t  *fc;
       vrna_pk_plex_opt_t    pk_plex_options;
 
-      fc = vrna_fold_compound(s1, &md, VRNA_OPTION_DEFAULT);
-
 
       pk_plex_options = vrna_pk_plex_opt((unsigned int)vrna_convert_kcal_to_dcal(subopts),
                                          MIN2(12, length - 3),
@@ -315,7 +315,11 @@ main(int  argc,
     if (istty)
       vrna_message_input_seq_simple();
   }
-  return 0;
+
+  if (vrna_log_fp() != stderr)
+    fclose(vrna_log_fp());
+
+  return EXIT_SUCCESS;
 }
 
 
