@@ -17,26 +17,31 @@
 #include <string.h>
 #include <limits.h>
 
-#include "ViennaRNA/fold.h"
-#include "ViennaRNA/part_func.h"
+#include "ViennaRNA/mfe/global.h"
+#include "ViennaRNA/partfunc/global.h"
+#include "ViennaRNA/eval/structures.h"
+#include "ViennaRNA/probabilities/structures.h"
+#include "ViennaRNA/probabilities/basepairs.h"
 #include "ViennaRNA/fold_vars.h"
 #include "ViennaRNA/plotting/probabilities.h"
 #include "ViennaRNA/plotting/structures.h"
 #include "ViennaRNA/plotting/alignments.h"
 #include "ViennaRNA/utils/basic.h"
 #include "ViennaRNA/utils/strings.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/alifold.h"
-#include "ViennaRNA/utils/alignments.h"
+#include "ViennaRNA/sequences/alignments.h"
 #include "ViennaRNA/io/file_formats.h"
 #include "ViennaRNA/io/file_formats_msa.h"
 #include "ViennaRNA/io/utils.h"
 #include "ViennaRNA/params/io.h"
-#include "ViennaRNA/MEA.h"
+#include "ViennaRNA/structures/centroid.h"
+#include "ViennaRNA/structures/mea.h"
 #include "ViennaRNA/params/basic.h"
 #include "ViennaRNA/constraints/basic.h"
 #include "ViennaRNA/constraints/hard.h"
 #include "ViennaRNA/constraints/soft.h"
-#include "ViennaRNA/constraints/SHAPE.h"
+#include "ViennaRNA/probing/SHAPE.h"
 #include "ViennaRNA/plotting/utils.h"
 #include "ViennaRNA/datastructures/char_stream.h"
 #include "ViennaRNA/datastructures/stream_output.h"
@@ -46,7 +51,7 @@
 #include "input_id_helpers.h"
 #include "parallel_helpers.h"
 
-#include "ViennaRNA/color_output.inc"
+#include "ViennaRNA/intern/color_output.h"
 
 #define DBL_ROUND(a, digits) (round((a) * pow(10., (double)(digits))) / pow(10., (double)(digits)))
 
@@ -73,6 +78,8 @@ struct options {
   int             non_red;
 
   int             color;
+  double          color_threshold;
+  double          color_min_sat;
   int             aln_PS;
   int             aln_PS_cols;
   int             mis;
@@ -247,12 +254,14 @@ init_default_options(struct options *opt)
   opt->eval_en  = 0;
   opt->non_red  = 0;
 
-  opt->color        = 0;
-  opt->aln_PS       = 0;
-  opt->aln_PS_cols  = 60;
-  opt->mis          = 0;
-  opt->sci          = 0;
-  opt->endgaps      = 0;
+  opt->color            = 0;
+  opt->color_threshold  = 2.;
+  opt->color_min_sat    = 0.2;
+  opt->aln_PS           = 0;
+  opt->aln_PS_cols      = 60;
+  opt->mis              = 0;
+  opt->sci              = 0;
+  opt->endgaps          = 0;
 
   opt->aln_out        = 0;
   opt->aln_out_prefix = NULL;
@@ -339,6 +348,9 @@ main(int  argc,
   if (RNAalifold_cmdline_parser(argc, argv, &args_info) != 0)
     exit(1);
 
+  /* prepare logging system and verbose mode */
+  ggo_log_settings(args_info, opt.verbose);
+
   /* get basic set of model details */
   ggo_get_md_eval(args_info, opt.md);
   ggo_get_md_fold(args_info, opt.md);
@@ -350,7 +362,7 @@ main(int  argc,
 
   /* check dangle model */
   if (!((opt.md.dangles == 0) || (opt.md.dangles == 2))) {
-    vrna_message_warning("required dangle model not implemented, falling back to default dangles=2");
+    vrna_log_warning("required dangle model not implemented, falling back to default dangles=2");
     opt.md.dangles = dangles = 2;
   }
 
@@ -414,6 +426,12 @@ main(int  argc,
   if (args_info.color_given)
     opt.color = 1;
 
+  if (args_info.color_threshold_given)
+    opt.color_threshold = args_info.color_threshold_arg;
+
+  if (args_info.color_min_sat_given)
+    opt.color_min_sat = args_info.color_min_sat_arg;
+
   if (args_info.aln_given)
     opt.aln_PS = 1;
 
@@ -434,7 +452,10 @@ main(int  argc,
     opt.md.uniq_ML      = 1;
     opt.md.compute_bpp  = 0;
     opt.pf              = 1;
-    vrna_init_rand();
+    if (args_info.random_seed_given)
+      vrna_init_rand_seed((unsigned int)args_info.random_seed_arg);
+    else
+      vrna_init_rand();
   }
 
   if (args_info.stochBT_en_given) {
@@ -443,7 +464,10 @@ main(int  argc,
     opt.md.compute_bpp  = 0;
     opt.pf              = 1;
     opt.eval_en         = 1;
-    vrna_init_rand();
+    if (args_info.random_seed_given)
+      vrna_init_rand_seed((unsigned int)args_info.random_seed_arg);
+    else
+      vrna_init_rand();
   }
 
   /* non-redundant backtracing */
@@ -463,12 +487,9 @@ main(int  argc,
   if (args_info.layout_type_given)
     rna_plot_type = args_info.layout_type_arg;
 
-  if (args_info.verbose_given)
-    opt.verbose = 1;
-
   if (args_info.quiet_given) {
     if (opt.verbose)
-      vrna_message_warning(
+      vrna_log_warning(
         "Can not be verbose and quiet at the same time! I keep on being chatty...");
     else
       opt.quiet = 1;
@@ -477,7 +498,7 @@ main(int  argc,
   /* SHAPE reactivity data */
   if (args_info.shape_given) {
     if (opt.verbose)
-      vrna_message_info(stderr, "SHAPE reactivity data correction activated");
+      vrna_log_info("SHAPE reactivity data correction activated");
 
     opt.shape                   = 1;
     opt.shape_files             = (char **)vrna_alloc(sizeof(char *) * (args_info.shape_given + 1));
@@ -503,8 +524,7 @@ main(int  argc,
       }
 
       if (opt.verbose) {
-        vrna_message_info(stderr,
-                          "Using SHAPE reactivity data provided in file %s for sequence %d",
+        vrna_log_info("Using SHAPE reactivity data provided in file %s for sequence %d",
                           opt.shape_files[s],
                           opt.shape_file_association[s] + 1);
       }
@@ -552,7 +572,7 @@ main(int  argc,
         break;
 
       default:
-        vrna_message_warning("Unknown input format specified");
+        vrna_log_warning("Unknown input format specified");
         break;
     }
   }
@@ -581,7 +601,7 @@ main(int  argc,
       if (num_proc_cores(&proc_cores, &proc_cores_conf)) {
         opt.jobs = MIN2(thread_max, proc_cores_conf);
       } else {
-        vrna_message_warning("Could not determine number of available processor cores!\n"
+        vrna_log_warning("Could not determine number of available processor cores!\n"
                              "Defaulting to serial computation");
         opt.jobs = 1;
       }
@@ -591,7 +611,7 @@ main(int  argc,
 
     opt.jobs = MAX2(1, opt.jobs);
 #else
-    vrna_message_warning(
+    vrna_log_warning(
       "This version of RNAfold has been built without parallel input processing capabilities");
 #endif
 
@@ -609,11 +629,8 @@ main(int  argc,
    # begin initializing
    #############################################
    */
-  if (opt.md.circ && opt.md.gquad)
-    vrna_message_error("G-Quadruplex support is currently not available for circular RNA structures");
-
   if (opt.md.circ && opt.md.noLP)
-    vrna_message_warning("Depending on the origin of the circular sequence, "
+    vrna_log_warning("Depending on the origin of the circular sequence, "
                          "some structures may be missed when using --noLP\n"
                          "Try rotating your sequence a few times\n");
 
@@ -657,14 +674,15 @@ main(int  argc,
       if (!skip) {
         FILE *input_stream = fopen((const char *)input_files[i], "r");
 
-        if (!input_stream)
-          vrna_message_error("Unable to open %d. input file \"%s\" for reading",
+        if (!input_stream) {
+          vrna_log_error("Unable to open %d. input file \"%s\" for reading",
                              i + 1,
                              input_files[i]);
+          exit(EXIT_FAILURE);
+        }
 
         if (opt.verbose) {
-          vrna_message_info(stderr,
-                            "Processing %d. input file \"%s\"",
+          vrna_log_info( "Processing %d. input file \"%s\"",
                             i + 1,
                             input_files[i]);
         }
@@ -697,25 +715,26 @@ main(int  argc,
 
     switch (opt.input_format) {
       case VRNA_FILE_FORMAT_MSA_CLUSTAL:
-        vrna_message_error(msg, "Clustal");
+        vrna_log_error(msg, "Clustal");
         break;
 
       case VRNA_FILE_FORMAT_MSA_STOCKHOLM:
-        vrna_message_error(msg, "Stockholm");
+        vrna_log_error(msg, "Stockholm");
         break;
 
       case VRNA_FILE_FORMAT_MSA_FASTA:
-        vrna_message_error(msg, "FASTA");
+        vrna_log_error(msg, "FASTA");
         break;
 
       case VRNA_FILE_FORMAT_MSA_MAF:
-        vrna_message_error(msg, "MAF");
+        vrna_log_error(msg, "MAF");
         break;
 
       default:
-        vrna_message_error(msg, "Unknown");
+        vrna_log_error(msg, "Unknown");
         break;
     }
+    exit(EXIT_FAILURE);
   }
 
   free(opt.shape_files);
@@ -723,6 +742,9 @@ main(int  argc,
   free(opt.filename_delim);
   free_id_data(opt.id_control);
 
+  if (vrna_log_fp() != stderr)
+    fclose(vrna_log_fp());
+
   return EXIT_SUCCESS;
 }
 
@@ -746,25 +768,27 @@ process_input(FILE            *input_stream,
 
       switch (opt->input_format) {
         case VRNA_FILE_FORMAT_MSA_CLUSTAL:
-          vrna_message_error(msg, "Clustal");
+          vrna_log_error(msg, "Clustal");
           break;
 
         case VRNA_FILE_FORMAT_MSA_STOCKHOLM:
-          vrna_message_error(msg, "Stockholm");
+          vrna_log_error(msg, "Stockholm");
           break;
 
         case VRNA_FILE_FORMAT_MSA_FASTA:
-          vrna_message_error(msg, "FASTA");
+          vrna_log_error(msg, "FASTA");
           break;
 
         case VRNA_FILE_FORMAT_MSA_MAF:
-          vrna_message_error(msg, "MAF");
+          vrna_log_error(msg, "MAF");
           break;
 
         default:
-          vrna_message_error(msg, "Unknown");
+          vrna_log_error(msg, "Unknown");
           break;
       }
+
+      exit(EXIT_FAILURE);
     }
 
     input_format = format_guess;
@@ -803,8 +827,8 @@ process_input(FILE            *input_stream,
           break;
 
         default:
-          vrna_message_error("Which input format are you using?");
-          break;
+          vrna_log_error("Which input format are you using?");
+          exit(EXIT_FAILURE);
       }
     }
 
@@ -905,7 +929,7 @@ process_record(struct record_data *record)
                                       VRNA_OPTION_DEFAULT);
 
   if (!vc) {
-    vrna_message_warning("Skipping computations for \"%s\"",
+    vrna_log_warning("Skipping computations for \"%s\"",
                          (record->MSA_ID) ? record->MSA_ID : "identifier unavailable");
     return;
   }
@@ -928,6 +952,9 @@ process_record(struct record_data *record)
                            "%u sequences; length of alignment %u.",
                            n_seq,
                            n);
+    vrna_log_info("%u sequences; length of alignment %u.",
+                           n_seq,
+                           n);
   }
 
   /*
@@ -951,7 +978,7 @@ process_record(struct record_data *record)
   /* check whether the constraint allows for any solution */
   if (fold_constrained) {
     if (min_en == (double)(INF / 100.)) {
-      vrna_message_error(
+      vrna_log_error(
         "Supplied structure constraints create empty solution set for alignment Nr. %d",
         record->number);
       exit(EXIT_FAILURE);
@@ -1023,11 +1050,19 @@ process_record(struct record_data *record)
 
   if (opt->aln_PS) {
     THREADSAFE_FILE_OUTPUT(
-      vrna_file_PS_aln(filename_aln,
-                       (const char **)record->alignment,
-                       (const char **)record->names,
-                       mfe_structure,
-                       opt->aln_PS_cols));
+      vrna_file_PS_aln_opt(filename_aln,
+                           (const char **)record->alignment,
+                           (const char **)record->names,
+                            mfe_structure,
+                            (vrna_aln_opt_t){
+                              .start = 0,
+                              .end = 0,
+                              .offset = 0,
+                              .columns = opt->aln_PS_cols,
+                              .color_threshold = opt->color_threshold,
+                              .color_min_sat = opt->color_min_sat
+                            })
+      );
   }
 
   /* free mfe arrays */
@@ -1074,7 +1109,7 @@ process_record(struct record_data *record)
       THREADSAFE_FILE_OUTPUT({
         aliout = fopen(filename_out, "w");
         if (!aliout)
-          vrna_message_warning("can't open %s ... skipping output", filename_out);
+          vrna_log_warning("can't open %s ... skipping output", filename_out);
         else
           print_aliout(vc, pl, opt->bppmThreshold, mfe_structure, aliout);
 
@@ -1186,12 +1221,14 @@ apply_constraints(vrna_fold_compound_t  *fc,
     unsigned int  length  = fc->length;
     unsigned int  cl      = strlen(cstruc);
 
-    if (cl == 0)
-      vrna_message_warning("structure constraint is missing");
-    else if (cl < length)
-      vrna_message_warning("structure constraint is shorter than sequence");
-    else if (cl > length)
-      vrna_message_error("structure constraint is too long");
+    if (cl == 0) {
+      vrna_log_warning("structure constraint is missing");
+    } else if (cl < length) {
+      vrna_log_warning("structure constraint is shorter than sequence");
+    } else if (cl > length) {
+      vrna_log_error("structure constraint is too long");
+      exit(EXIT_FAILURE);
+    }
 
     /** [Adding hard constraints from pseudo dot-bracket] */
     constraint_options = VRNA_CONSTRAINT_DB_DEFAULT;
@@ -1207,7 +1244,7 @@ apply_constraints(vrna_fold_compound_t  *fc,
 
     vrna_constraints_add(fc, SS_cons, constraint_options);
   } else {
-    vrna_message_warning("Constraint missing");
+    vrna_log_warning("Constraint missing");
   }
 }
 
@@ -1250,9 +1287,15 @@ postscript_layout(const char      *filename,
                   const char      *consensus_structure,
                   struct options  *opt)
 {
-  char **A;
+  vrna_string_t *A;
 
-  A = vrna_annotate_covar_db(alignment, consensus_structure, &(opt->md));
+  short int *pt = vrna_ptable(consensus_structure);
+
+  A = vrna_annotate_covar_pt(alignment,
+                             pt,
+                             &(opt->md),
+                             opt->color_threshold,
+                             opt->color_min_sat);
 
   THREADSAFE_FILE_OUTPUT(
     (void)vrna_file_PS_rnaplot_a(consensus_sequence,
@@ -1262,8 +1305,9 @@ postscript_layout(const char      *filename,
                                  A[1],
                                  &(opt->md)));
 
-  free(A[0]);
-  free(A[1]);
+  free(pt);
+  vrna_string_free(A[0]);
+  vrna_string_free(A[1]);
   free(A);
 }
 
@@ -1407,7 +1451,7 @@ apply_SHAPE_data(vrna_fold_compound_t *fc,
   for (s = 0; opt->shape_file_association[s] != -1; s++);
 
   if ((s != fc->n_seq) && (!opt->quiet)) {
-    vrna_message_warning("Number of sequences in alignment (%d) does not match "
+    vrna_log_warning("Number of sequences in alignment (%d) does not match "
                          "number of provided SHAPE reactivity data files (%d)!",
                          fc->n_seq,
                          s);
