@@ -18,23 +18,25 @@
 #include "ViennaRNA/plotting/structures.h"
 #include "ViennaRNA/fold_vars.h"
 #include "ViennaRNA/params/basic.h"
-#include "ViennaRNA/commands.h"
+#include "ViennaRNA/io/commands.h"
+#include "ViennaRNA/combinatorics/basic.h"
 #include "ViennaRNA/constraints/basic.h"
 #include "ViennaRNA/constraints/hard.h"
 #include "ViennaRNA/constraints/soft.h"
-#include "ViennaRNA/constraints/SHAPE.h"
+#include "ViennaRNA/probing/SHAPE.h"
 #include "ViennaRNA/constraints/soft_special.h"
 #include "ViennaRNA/io/file_formats.h"
 #include "ViennaRNA/io/utils.h"
 #include "ViennaRNA/cofold.h"
 #include "ViennaRNA/fold.h"
 #include "ViennaRNA/part_func_co.h"
-#include "ViennaRNA/part_func.h"
-#include "ViennaRNA/centroid.h"
-#include "ViennaRNA/MEA.h"
+#include "ViennaRNA/partfunc/global.h"
+#include "ViennaRNA/structures/centroid.h"
+#include "ViennaRNA/structures/mea.h"
 #include "ViennaRNA/utils/basic.h"
 #include "ViennaRNA/utils/strings.h"
-#include "ViennaRNA/utils/structures.h"
+#include "ViennaRNA/structures/pairtable.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/params/io.h"
 #include "ViennaRNA/datastructures/char_stream.h"
 #include "ViennaRNA/datastructures/stream_output.h"
@@ -43,7 +45,7 @@
 #include "gengetopt_helpers.h"
 #include "input_id_helpers.h"
 #include "modified_bases_helpers.h"
-#include "ViennaRNA/color_output.inc"
+#include "ViennaRNA/intern/color_output.h"
 #include "parallel_helpers.h"
 
 
@@ -274,6 +276,9 @@ main(int  argc,
   if (RNAcofold_cmdline_parser(argc, argv, &args_info) != 0)
     exit(1);
 
+  /* prepare logging system and verbose mode */
+  ggo_log_settings(args_info, opt.verbose);
+
   /* get basic set of model details */
   ggo_get_md_eval(args_info, opt.md);
   ggo_get_md_fold(args_info, opt.md);
@@ -284,7 +289,7 @@ main(int  argc,
 
   /* check dangle model */
   if ((opt.md.dangles < 0) || (opt.md.dangles > 3)) {
-    vrna_message_warning("required dangle model not implemented, falling back to default dangles=2");
+    vrna_log_warning("required dangle model not implemented, falling back to default dangles=2");
     opt.md.dangles = dangles = 2;
   }
 
@@ -368,9 +373,6 @@ main(int  argc,
       opt.md.compute_bpp = 1;
   }
 
-  if (args_info.verbose_given)
-    opt.verbose = 1;
-
   if (args_info.commands_given)
     opt.commands = vrna_file_commands_read(args_info.commands_arg,
                                            VRNA_CMD_PARSE_HC | VRNA_CMD_PARSE_SC);
@@ -408,7 +410,7 @@ main(int  argc,
         opt.csv_output = 0;
         break;
       default:
-        vrna_message_warning("unknown output format \"%c\", using defaults!",
+        vrna_log_warning("unknown output format \"%c\", using defaults!",
                              *(args_info.output_format_arg));
         break;
     }
@@ -418,7 +420,7 @@ main(int  argc,
   if (args_info.csv_delim_given) {
     opt.csv_output_delim = *(args_info.csv_delim_arg);
     if (!opt.csv_output_delim) {
-      vrna_message_warning("Delimiting character for One-Line output is missing, using defaults!");
+      vrna_log_warning("Delimiting character for One-Line output is missing, using defaults!");
       opt.csv_output_delim = ',';
     }
   }
@@ -436,7 +438,7 @@ main(int  argc,
       if (num_proc_cores(&proc_cores, &proc_cores_conf)) {
         opt.jobs = MIN2(thread_max, proc_cores_conf);
       } else {
-        vrna_message_warning("Could not determine number of available processor cores!\n"
+        vrna_log_warning("Could not determine number of available processor cores!\n"
                              "Defaulting to serial computation");
         opt.jobs = 1;
       }
@@ -446,7 +448,7 @@ main(int  argc,
 
     opt.jobs = MAX2(1, opt.jobs);
 #else
-    vrna_message_warning(
+    vrna_log_warning(
       "This version of RNAfold has been built without parallel input processing capabilities");
 #endif
 
@@ -466,15 +468,17 @@ main(int  argc,
    # begin initializing
    #############################################
    */
-  if (opt.pf && opt.md.gquad)
-    vrna_message_error(
+  if (opt.pf && opt.md.gquad) {
+    vrna_log_error(
       "G-Quadruplex support is currently not available for partition function computations");
+    exit(EXIT_FAILURE);
+  }
 
   if ((opt.csv_output) && (opt.csv_header))
     write_csv_header(stdout, &opt);
 
   if ((opt.verbose) && (opt.jobs > 1))
-    vrna_message_info(stderr, "Preparing %d parallel computation slots", opt.jobs);
+    vrna_log_info("Preparing %d parallel computation slots", opt.jobs);
 
   if (opt.keep_order)
     opt.output_queue = vrna_ostream_init(&flush_cstr_callback, NULL);
@@ -492,13 +496,14 @@ main(int  argc,
       if (!skip) {
         FILE *input_stream = fopen((const char *)input_files[i], "r");
 
-        if (!input_stream)
-          vrna_message_error("Unable to open %d. input file \"%s\" for reading", i + 1,
+        if (!input_stream) {
+          vrna_log_error("Unable to open %d. input file \"%s\" for reading", i + 1,
                              input_files[i]);
+          exit(EXIT_FAILURE);
+        }
 
         if (opt.verbose) {
-          vrna_message_info(stderr,
-                            "Processing %d. input file \"%s\"",
+          vrna_log_info("Processing %d. input file \"%s\"",
                             i + 1,
                             input_files[i]);
         }
@@ -542,6 +547,9 @@ main(int  argc,
 
   free_id_data(opt.id_control);
 
+  if (vrna_log_fp() != stderr)
+    fclose(vrna_log_fp());
+
   return EXIT_SUCCESS;
 }
 
@@ -698,15 +706,17 @@ process_record(struct record_data *record)
                                                 VRNA_OPTION_DEFAULT | VRNA_OPTION_HYBRID);
 
   if (!vc) {
-    vrna_message_warning("Skipping computations for \"%s\"",
+    vrna_log_warning("Skipping computations for \"%s\"",
                          (record->id) ? record->id : "identifier unavailable");
     return;
   }
 
   n = vc->length;
 
-  if (vc->strands > 2)
-    vrna_message_error("More than one strand delimiter in input!");
+  if (vc->strands > 2) {
+    vrna_log_error("More than one strand delimiter in input!");
+    exit(EXIT_FAILURE);
+  }
 
   /* retrieve string stream bound to stdout, 6*length should be enough memory to start with */
   o_stream->data = vrna_cstr(6 * n, stdout);
@@ -714,21 +724,22 @@ process_record(struct record_data *record)
   o_stream->err = vrna_cstr(n, stderr);
 
   if (record->tty) {
-    if (vc->cutpoint == -1) {
-      vrna_message_info(stdout, "length = %u", n);
+    if (vc->strands == 1) {
+      printf("length = %u", n);
     } else {
-      vrna_message_info(stdout,
-                        "length1 = %d\nlength2 = %d",
-                        vc->cutpoint - 1,
-                        (int)n - vc->cutpoint + 1);
+      printf("length1 = %d\nlength2 = %d",
+             vc->nucleotides[0].length,
+             vc->nucleotides[1].length);
     }
   }
 
-  if (vc->cutpoint == vc->length / 2 + 1) {
-    if (!strncmp(vc->sequence, vc->sequence + vc->cutpoint - 1, vc->cutpoint - 1)) {
+  if (vc->strands > 1) {
+    unsigned int r = vrna_rotational_symmetry(vc->sequence);
+    if (r > 1) {
       vrna_cstr_message_warning(o_stream->err,
-                                "Both input strands are identical, thus inducing rotationally symmetry! "
-                                "Symmetry correction might be required to compute actual MFE!");
+                                "The input sequences show a rotational symmetry of %u! "
+                                "Symmetry correction might be required to compute actual MFE!",
+                                r);
     }
   }
 
@@ -745,21 +756,57 @@ process_record(struct record_data *record)
       int           cp        = -1;
 
       cstruc  = vrna_extract_record_rest_structure((const char **)rec_rest, 0, coptions);
-      cstruc  = vrna_cut_point_remove(cstruc, &cp);
-      if (vc->cutpoint != cp) {
-        vrna_message_error("Sequence and Structure have different cut points.\n"
-                           "sequence: %d, structure: %d",
-                           vc->cutpoint, cp);
+
+      char **structures = vrna_strsplit(cstruc, "&");
+      for (unsigned int a = 0; a < vc->strands; a++) {
+        if (!structures[a]) {
+          vrna_log_error("Sequence and Structure have different number of strand delimiters");
+          exit(EXIT_FAILURE);
+        }
+
+        unsigned int l = strlen(structures[a]);
+
+        cl += l;
+
+        switch (vc->type) {
+          case VRNA_FC_TYPE_SINGLE:
+            if (vc->nucleotides[a].length != l) {
+              vrna_log_error(
+                "Structure and sequence part of strand %u differ in length (%u vs. %u)",
+                a,
+                l,
+                vc->nucleotides[a].length);
+              exit(EXIT_FAILURE);
+            }
+
+            break;
+
+          case VRNA_FC_TYPE_COMPARATIVE:
+            if (vc->alignment[a].sequences[0].length != l) {
+              vrna_log_error(
+                "Structure and sequence part of strand %u differ in length (%u vs. %u)",
+                a,
+                l,
+                vc->alignment[a].sequences[0].length);
+              exit(EXIT_FAILURE);
+            }
+            break;
+        }
       }
 
-      cl = (cstruc) ? (int)strlen(cstruc) : 0;
+      for (unsigned int a = 0; a < vc->strands; a++)
+        free(structures[a]);
 
-      if (cl == 0)
-        vrna_message_warning("Structure constraint is missing");
-      else if (cl < n)
-        vrna_message_warning("Structure constraint is shorter than sequence");
-      else if (cl > n)
-        vrna_message_error("Structure constraint is too long");
+      free(structures);
+
+      if (cl == 0) {
+        vrna_log_warning("Structure constraint is missing");
+      } else if (cl < n) {
+        vrna_log_warning("Structure constraint is shorter than sequence");
+      } else if (cl > n) {
+        vrna_log_error("Structure constraint is too long");
+        exit(EXIT_FAILURE);
+      }
 
       if (cstruc) {
         unsigned int constraint_options = VRNA_CONSTRAINT_DB_DEFAULT;
@@ -799,8 +846,10 @@ process_record(struct record_data *record)
     if (opt->concentration_file) {
       /* read from file */
       FILE *fp = fopen(opt->concentration_file, "r");
-      if (fp == NULL)
-        vrna_message_error("could not open concentration file %s", opt->concentration_file);
+      if (fp == NULL) {
+        vrna_log_error("could not open concentration file %s", opt->concentration_file);
+        exit(EXIT_FAILURE);
+      }
 
       concentrations = read_concentrations(fp);
       fclose(fp);
@@ -823,7 +872,7 @@ process_record(struct record_data *record)
   /* check whether the constraint allows for any solution */
   if ((fold_constrained) || (opt->commands)) {
     if (min_en == (double)(INF / 100.)) {
-      vrna_message_error(
+      vrna_log_error(
         "Supplied structure constraints create empty solution set for sequence:\n%s",
         record->sequence);
       exit(EXIT_FAILURE);
@@ -831,7 +880,18 @@ process_record(struct record_data *record)
   }
 
   {
-    char *pstruct = vrna_cut_point_insert(mfe_structure, vc->cutpoint);
+    char  *pstruct    = NULL;
+    char  *tmp_struct = strdup(mfe_structure);
+
+    if (vc->strands == 1) {
+      pstruct = tmp_struct;
+    } else {
+      for (unsigned int i = 1; i < vc->strands; i++) {
+        pstruct = vrna_cut_point_insert(tmp_struct, (int)vc->strand_start[i] + (i - 1));
+        free(tmp_struct);
+        tmp_struct = pstruct;
+      }
+    }
 
     if (opt->csv_output) {
       vrna_cstr_printf(o_stream->data,
@@ -900,10 +960,21 @@ process_record(struct record_data *record)
     AB = AA = BB = vrna_pf_dimer(vc, pairing_propensity);
 
     if (opt->md.compute_bpp) {
-      char *costruc;
+      char *costruc = NULL;
       prAB = vrna_plist_from_probs(vc, opt->bppmThreshold);
 
-      costruc = vrna_cut_point_insert(pairing_propensity, vc->cutpoint);
+      char  *tmp_struct = strdup(pairing_propensity);
+
+      if (vc->strands == 1) {
+        costruc = tmp_struct;
+      } else {
+        for (unsigned int i = 1; i < vc->strands; i++) {
+          costruc = vrna_cut_point_insert(tmp_struct, (int)vc->strand_start[i] + (i - 1));
+          free(tmp_struct);
+          tmp_struct = costruc;
+        }
+      }
+
       if (opt->csv_output) {
         vrna_cstr_printf(o_stream->data,
                          "%c"
@@ -965,7 +1036,7 @@ process_record(struct record_data *record)
 
     if (opt->doT) {
       if (vc->strands < 2) {
-        vrna_message_warning(
+        vrna_log_warning(
           "Sorry, i cannot do that with only one molecule, please give me two or leave it");
         free(mfAB);
         free(prAB);
@@ -1191,7 +1262,7 @@ process_record(struct record_data *record)
         if (filename_dot) {
           THREADSAFE_FILE_OUTPUT(
             (void)vrna_plot_dp_PS_list(record->sequence,
-                                       vc->cutpoint,
+                                       (vc->strands > 1) ? vc->strand_start[1] : -1,
                                        filename_dot,
                                        prAB,
                                        mfAB,
@@ -1318,7 +1389,7 @@ compute_MEA(vrna_fold_compound_t  *fc,
             double                MEAgamma,
             vrna_cstr_t           rec_output)
 {
-  char  *structure, *mea_structure;
+  char  *structure, *mea_structure = NULL;
   float mea, mea_en;
 
   structure = vrna_MEA(fc, MEAgamma, &mea);
@@ -1326,7 +1397,17 @@ compute_MEA(vrna_fold_compound_t  *fc,
   mea_en = vrna_eval_structure(fc, (const char *)structure);
 
   /* insert cut point */
-  mea_structure = vrna_cut_point_insert(structure, fc->cutpoint);
+  char  *tmp_struct = strdup(structure);
+
+  if (fc->strands == 1) {
+    mea_structure = tmp_struct;
+  } else {
+    for (unsigned int i = 1; i < fc->strands; i++) {
+      mea_structure = vrna_cut_point_insert(tmp_struct, (int)fc->strand_start[i] + (i - 1));
+      free(tmp_struct);
+      tmp_struct = mea_structure;
+    }
+  }
 
   vrna_cstr_printf_structure(rec_output, mea_structure, " {%6.2f MEA=%.2f}", mea_en, mea);
 
@@ -1339,14 +1420,24 @@ static void
 compute_centroid(vrna_fold_compound_t *fc,
                  vrna_cstr_t          rec_output)
 {
-  char    *cent, *centroid_structure;
+  char    *cent, *centroid_structure = NULL;
   double  cent_en, dist;
 
   cent    = vrna_centroid(fc, &dist);
   cent_en = vrna_eval_structure(fc, (const char *)cent);
 
   /* insert cut point */
-  centroid_structure = vrna_cut_point_insert(cent, fc->cutpoint);
+  char  *tmp_struct = strdup(cent);
+
+  if (fc->strands == 1) {
+    centroid_structure = tmp_struct;
+  } else {
+    for (unsigned int i = 1; i < fc->strands; i++) {
+      centroid_structure = vrna_cut_point_insert(tmp_struct, (int)fc->strand_start[i] + (i - 1));
+      free(tmp_struct);
+      tmp_struct = centroid_structure;
+    }
+  }
 
   vrna_cstr_printf_structure(rec_output, centroid_structure, " {%6.2f d=%.2f}", cent_en, dist);
 
@@ -1473,11 +1564,11 @@ postscript_layout(vrna_fold_compound_t  *fc,
     filename_plot = strdup("rna.ps");
   }
 
-  if (fc->cutpoint >= 0) {
+  if (fc->strands > 1) {
     annot = vrna_strdup_printf("1 %d 9  0 0.9 0.2 omark\n"
                                "%d %d 9  1 0.1 0.2 omark\n",
-                               fc->cutpoint - 1,
-                               fc->cutpoint + 1,
+                               fc->strand_end[0],
+                               fc->strand_start[1] + 1,
                                fc->length + 1);
   }
 
