@@ -20,11 +20,16 @@
 #include "ViennaRNA/stringdist.h"
 #include "ViennaRNA/utils/basic.h"
 #include "ViennaRNA/utils/strings.h"
-#include "ViennaRNA/utils/structures.h"
+#include "ViennaRNA/structures/pairtable.h"
+#include "ViennaRNA/structures/dotbracket.h"
+#include "ViennaRNA/structures/tree.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/io/utils.h"
 #include "ViennaRNA/datastructures/basic.h"
 #include "RNAdistance_cmdl.h"
 
+#include "gengetopt_helpers.h"
+
 #define MAXNUM      1000    /* max number of structs for distance matrix */
 
 #define PUBLIC
@@ -192,14 +197,21 @@ main(int  argc,
       type  = 1;
     }
 
+    if (n >= MAXNUM) {
+      vrna_log_error("Number of input structures exceeds maximum of %d", MAXNUM);
+      exit(EXIT_FAILURE);
+    }
+
     tree_types    = 0;
     string_types  = 0;
     for (tt = 0; tt < types; tt++) {
       switch (ttype[tt]) {
         case 'f':
         case 'F':
-          if (type != 1)
-            vrna_message_error("Can't convert back to full structure");
+          if (type != 1) {
+            vrna_log_error("Can't convert back to full structure");
+            exit(EXIT_FAILURE);
+          }
 
           xstruc = expand_Full(line);
           if (islower(ttype[tt])) /* tree_edit */
@@ -211,8 +223,10 @@ main(int  argc,
           free(xstruc);
           break;
         case 'P':
-          if (type != 1)
-            vrna_message_error("Can't convert back to full structure");
+          if (type != 1) {
+            vrna_log_error("Can't convert back to full structure");
+            exit(EXIT_FAILURE);
+          }
 
           P[n] = strdup(line);
           break;
@@ -230,7 +244,8 @@ main(int  argc,
               free(xstruc);
               break;
             default:
-              vrna_message_error("Can't convert to HIT structure");
+              vrna_log_error("Can't convert to HIT structure");
+              exit(EXIT_FAILURE);
           }
           break;
         case 'c':
@@ -250,8 +265,8 @@ main(int  argc,
               xstruc = unweight(line);
               break;
             default:
-              vrna_message_error("Unknown structure representation");
-              exit(0);
+              vrna_log_error("Unknown structure representation");
+              exit(EXIT_FAILURE);
           }
           if (islower(ttype[tt]))
             T[tree_types++][n] = make_tree(xstruc);
@@ -282,7 +297,8 @@ main(int  argc,
 
           break;
         default:
-          vrna_message_error("Unknown distance type");
+          vrna_log_error("Unknown distance type");
+          exit(EXIT_FAILURE);
       }
     }
     n++;
@@ -387,7 +403,11 @@ main(int  argc,
     }
     fflush(stdout);
   } while (type != 999);
-  return 0;
+
+  if (vrna_log_fp() != stderr)
+    fclose(vrna_log_fp());
+
+  return EXIT_SUCCESS;
 }
 
 
@@ -608,6 +628,7 @@ PRIVATE void
 command_line(int  argc,
              char *argv[])
 {
+  int           verbose;
   struct        RNAdistance_args_info args_info;
 
   /* default values */
@@ -624,6 +645,9 @@ command_line(int  argc,
   if (RNAdistance_cmdline_parser(argc, argv, &args_info) != 0)
     exit(1);
 
+  /* prepare logging system and verbose mode */
+  ggo_log_settings(args_info, verbose);
+
   /* use specified distance representations */
   if (args_info.distance_given) {
     strncpy(ttype, args_info.distance_arg, 9);
