@@ -8,15 +8,19 @@
 #include <stdlib.h>
 
 #include "ViennaRNA/utils/basic.h"
-#include "ViennaRNA/utils/structures.h"
+#include "ViennaRNA/structures/pairtable.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/params/io.h"
 #include "ViennaRNA/datastructures/basic.h"
 #include "ViennaRNA/fold_vars.h"
 #include "ViennaRNA/params/basic.h"
-#include "ViennaRNA/loops/all.h"
-#include "ViennaRNA/alphabet.h"
-#include "ViennaRNA/mfe.h"
-#include "ViennaRNA/file_utils.h"
+#include "ViennaRNA/eval/exterior.h"
+#include "ViennaRNA/eval/hairpin.h"
+#include "ViennaRNA/eval/internal.h"
+#include "ViennaRNA/eval/multibranch.h"
+#include "ViennaRNA/sequences/alphabet.h"
+#include "ViennaRNA/mfe/global.h"
+#include "ViennaRNA/io/utils.h"
 #include "ViennaRNA/io/file_formats.h"
 #include "ViennaRNA/datastructures/hash_tables.h"
 #ifdef _OPENMP
@@ -266,7 +270,7 @@ decompose_pair(vrna_fold_compound_t         *fc,
       hashtable_list_add_count(result_table, energy_hp, 1);
     }
 
-    /* check for interior loops */
+    /* check for internal loops */
     int maxp = MIN2(j - 2 - turn, i + MAXLOOP + 1);
     int p, q, type_2, pq;
     int energy;
@@ -771,6 +775,9 @@ main(int  argc,
   if (RNAdos_cmdline_parser(argc, argv, &args_info) != 0)
     exit(1);
 
+  /* prepare logging system and verbose mode */
+  ggo_log_settings(args_info, verbose);
+
   char *rnaSequence = NULL;
 
   if (!args_info.sequence_given) {
@@ -794,7 +801,7 @@ main(int  argc,
   /* dangle options */
   if (args_info.dangles_given) {
     if ((args_info.dangles_arg != 0) && (args_info.dangles_arg != 2))
-      vrna_message_warning(
+      vrna_log_warning(
         "required dangle model not implemented, falling back to default dangles=2");
     else
       md.dangles = dangles = args_info.dangles_arg;
@@ -836,7 +843,7 @@ main(int  argc,
                                                 VRNA_OPTION_DEFAULT);
 
   if (!vrna_fold_compound_prepare(fc, VRNA_OPTION_MFE))
-    vrna_message_warning("vrna_mfe@mfe.c: Failed to prepare vrna_fold_compound");
+    vrna_log_warning("vrna_mfe@mfe.c: Failed to prepare vrna_fold_compound");
 
   int max_energy_dcal = max_energy * 100;
   compute_density_of_states(fc, max_energy_dcal, hash_bits, verbose);
@@ -844,5 +851,8 @@ main(int  argc,
   free(rnaSequence);
   vrna_fold_compound_free(fc);
 
+  if (vrna_log_fp() != stderr)
+    fclose(vrna_log_fp());
+
   return EXIT_SUCCESS;
 }
