@@ -24,27 +24,27 @@
 #include "ViennaRNA/params/basic.h"
 #include "ViennaRNA/utils/basic.h"
 #include "ViennaRNA/utils/strings.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/params/io.h"
 #include "ViennaRNA/constraints/basic.h"
-#include "ViennaRNA/constraints/SHAPE.h"
+#include "ViennaRNA/probing/SHAPE.h"
 #include "ViennaRNA/io/file_formats.h"
 #include "ViennaRNA/io/file_formats_msa.h"
-#include "ViennaRNA/eval.h"
+#include "ViennaRNA/eval/structures.h"
 #include "ViennaRNA/cofold.h"
-#include "ViennaRNA/utils/alignments.h"
+#include "ViennaRNA/sequences/alignments.h"
 #include "ViennaRNA/datastructures/char_stream.h"
 #include "ViennaRNA/datastructures/stream_output.h"
-#include "ViennaRNA/color_output.inc"
+#include "ViennaRNA/datastructures/string.h"
+#include "ViennaRNA/intern/color_output.h"
 
 #include "RNAeval_cmdl.h"
-#include "gengetopt_helper.h"
+#include "gengetopt_helpers.h"
 #include "input_id_helpers.h"
 #include "parallel_helpers.h"
 
 #define DBL_ROUND(a, digits) (round((a) * pow(10., (double)(digits))) / pow(10., (double)(digits)))
 
-#define MULTISTRAND_EVAL
-
 struct options {
   unsigned int    msa_format;
   int             filename_full;
@@ -216,6 +216,9 @@ main(int  argc,
   if (RNAeval_cmdline_parser(argc, argv, &args_info) != 0)
     exit(1);
 
+  /* prepare logging system and verbose mode */
+  ggo_log_settings(args_info, opt.verbose);
+
   /* get basic set of model details */
   ggo_get_md_eval(args_info, opt.md);
   ggo_get_circ(args_info, opt.md.circ);
@@ -225,7 +228,7 @@ main(int  argc,
 
   /* check dangle model */
   if ((opt.md.dangles < 0) || (opt.md.dangles > 3)) {
-    vrna_message_warning("required dangle model not implemented, falling back to default dangles=2");
+    vrna_log_warning("required dangle model not implemented, falling back to default dangles=2");
     opt.md.dangles = dangles = 2;
   }
 
@@ -240,10 +243,6 @@ main(int  argc,
   if (args_info.logML_given)
     opt.md.logML = logML = 1;
 
-  /* be verbose */
-  if (args_info.verbose_given)
-    opt.verbose = 1;
-
   if (args_info.msa_given) {
     opt.aln = 1;
 
@@ -272,6 +271,8 @@ main(int  argc,
     }
   }
 
+  ggo_geometry_settings(args_info, &(opt.md));
+
   if (args_info.jobs_given) {
 #if VRNA_WITH_PTHREADS
     int thread_max = max_user_threads();
@@ -281,7 +282,7 @@ main(int  argc,
       if (num_proc_cores(&proc_cores, &proc_cores_conf)) {
         opt.jobs = MIN2(thread_max, proc_cores_conf);
       } else {
-        vrna_message_warning("Could not determine number of available processor cores!\n"
+        vrna_log_warning("Could not determine number of available processor cores!\n"
                              "Defaulting to serial computation");
         opt.jobs = 1;
       }
@@ -291,7 +292,7 @@ main(int  argc,
 
     opt.jobs = MAX2(1, opt.jobs);
 #else
-    vrna_message_warning(
+    vrna_log_warning(
       "This version of RNAeval has been built without parallel input processing capabilities");
 #endif
 
@@ -302,10 +303,11 @@ main(int  argc,
   input_files = collect_unnamed_options(&args_info, &num_input);
   input_files = append_input_files(&args_info, input_files, &num_input);
 
-  if (opt.aln)
+  if (opt.aln) {
     ggo_get_id_control(args_info, opt.id_control, "Alignment", "alignment", "_", 4, 1);
-  else
+  } else {
     ggo_get_id_control(args_info, opt.id_control, "Sequence", "sequence", "_", 4, 1);
+  }
 
   /* free allocated memory of command line data structure */
   RNAeval_cmdline_parser_free(&args_info);
@@ -315,8 +317,12 @@ main(int  argc,
    # begin initializing
    #############################################
    */
-  if (opt.md.circ && opt.md.gquad)
-    vrna_message_error("G-Quadruplex support is currently not available for circular RNA structures");
+#if 0
+  if (opt.md.circ && opt.md.gquad) {
+    vrna_log_error("G-Quadruplex support is currently not available for circular RNA structures");
+    exit(EXIT_FAILURE);
+  }
+#endif
 
   if (opt.keep_order)
     opt.output_queue = vrna_ostream_init(&flush_cstr_callback, NULL);
@@ -343,9 +349,11 @@ main(int  argc,
       if (!skip) {
         FILE *input_stream = fopen((const char *)input_files[i], "r");
 
-        if (!input_stream)
-          vrna_message_error("Unable to open %d. input file \"%s\" for reading", i + 1,
+        if (!input_stream) {
+          vrna_log_error("Unable to open %d. input file \"%s\" for reading", i + 1,
                              input_files[i]);
+          exit(EXIT_FAILURE);
+        }
 
         if (processing_func(input_stream, (const char *)input_files[i], &opt) == 0)
           skip = 1;
@@ -375,6 +383,9 @@ main(int  argc,
 
   free_id_data(opt.id_control);
 
+  if (vrna_log_fp() != stderr)
+    fclose(vrna_log_fp());
+
   return EXIT_SUCCESS;
 }
 
@@ -485,25 +496,26 @@ process_alignment_input(FILE            *input_stream,
 
       switch (opt->msa_format) {
         case VRNA_FILE_FORMAT_MSA_CLUSTAL:
-          vrna_message_error(msg, "Clustal");
+          vrna_log_error(msg, "Clustal");
           break;
 
         case VRNA_FILE_FORMAT_MSA_STOCKHOLM:
-          vrna_message_error(msg, "Stockholm");
+          vrna_log_error(msg, "Stockholm");
           break;
 
         case VRNA_FILE_FORMAT_MSA_FASTA:
-          vrna_message_error(msg, "FASTA");
+          vrna_log_error(msg, "FASTA");
           break;
 
         case VRNA_FILE_FORMAT_MSA_MAF:
-          vrna_message_error(msg, "MAF");
+          vrna_log_error(msg, "MAF");
           break;
 
         default:
-          vrna_message_error(msg, "Unknown");
+          vrna_log_error(msg, "Unknown");
           break;
       }
+      exit(EXIT_FAILURE);
     }
 
     input_format = format_guess;
@@ -542,8 +554,8 @@ process_alignment_input(FILE            *input_stream,
           break;
 
         default:
-          vrna_message_error("Which input format are you using?");
-          break;
+          vrna_log_error("Which input format are you using?");
+          exit(EXIT_FAILURE);
       }
     }
 
@@ -633,7 +645,7 @@ process_record(struct record_data *record)
                           VRNA_OPTION_MFE | VRNA_OPTION_EVAL_ONLY);
 
   if (!vc) {
-    vrna_message_warning("Skipping computations for \"%s\"",
+    vrna_log_warning("Skipping computations for \"%s\"",
                          (record->id) ? record->id : "identifier unavailable");
     return;
   }
@@ -658,25 +670,28 @@ process_record(struct record_data *record)
                                            0,
                                            (record->multiline_input) ? VRNA_OPTION_MULTILINE : 0);
 
-  if (!tmp)
-    vrna_message_error("structure missing for record %d\n", record->number);
+  if (!tmp) {
+    vrna_log_error("structure missing for record %d\n", record->number);
+    exit(EXIT_FAILURE);
+  }
 
   {
-#ifdef MULTISTRAND_EVAL
     char **structures = vrna_strsplit(tmp, "&");
     for (unsigned int a = 0; a < vc->strands; a++) {
       if (!structures[a])
-        vrna_message_error("Sequence and Structure have different number of strand delimiters");
+        vrna_log_error("Sequence and Structure have different number of strand delimiters");
 
       unsigned int l = strlen(structures[a]);
       switch (vc->type) {
         case VRNA_FC_TYPE_SINGLE:
-          if (vc->nucleotides[a].length != l)
-            vrna_message_error(
+          if (vc->nucleotides[a].length != l) {
+            vrna_log_error(
               "Structure and sequence part of strand %u differ in length (%u vs. %u)",
               a,
               l,
               vc->nucleotides[a].length);
+            exit(EXIT_FAILURE);
+          }
 
           break;
 
@@ -685,61 +700,42 @@ process_record(struct record_data *record)
       }
     }
     structure = vrna_strjoin((const char **)structures, NULL);
+
     for (unsigned int a = 0; a < vc->strands; a++)
       free(structures[a]);
+
     free(structures);
-#else
-    int cp = -1;
-    structure = vrna_cut_point_remove(tmp, &cp);
-    if (cp != vc->cutpoint) {
-      vrna_message_warning("cut_point = %d cut = %d", vc->cutpoint, cp);
-      vrna_message_error("Sequence and Structure have different cut points.");
-    }
-
-    n = (int)strlen(structure);
-    if (n != vc->length)
-      vrna_message_error("structure and sequence differ in length!");
-
-#endif
-
     free(tmp);
   }
 
   if (record->tty) {
-#ifdef MULTISTRAND_EVAL
-    if (vc->strands == 1) {
-      vrna_message_info(stdout, "length = %u", vc->length);
-    } else {
-      switch (vc->type) {
-        case VRNA_FC_TYPE_SINGLE:
-          for (unsigned int a = 0; a < vc->strands; a++)
-            vrna_message_info(stdout,
-                              "length%u = %u",
-                              a + 1,
-                              vc->nucleotides[a].length);
-          break;
+    char          *tmp;
+    vrna_string_t lstring;
 
-        case VRNA_FC_TYPE_COMPARATIVE:
-          for (unsigned int a = 0; a < vc->strands; a++)
-            vrna_message_info(stdout,
-                              "length%u = %u",
-                              a + 1,
-                              vc->alignment[a].sequences[0].length);
-          break;
-      }
+    lstring = vrna_string_make("length(s) = {");
+    tmp     = vrna_strdup_printf(" %u",
+                                 (vc->type == VRNA_FC_TYPE_COMPARATIVE) ?
+                                 vc->alignment[0].sequences[0].length :
+                                 vc->nucleotides[0].length);
+
+    lstring = vrna_string_append_cstring(lstring, tmp);
+    free(tmp);
+
+    for (unsigned int i = 1; i < vc->strands; i++) {
+      tmp = vrna_strdup_printf(" %u",
+                               (vc->type == VRNA_FC_TYPE_COMPARATIVE) ?
+                               vc->alignment[i].sequences[0].length :
+                               vc->nucleotides[i].length);
+      lstring = vrna_string_append_cstring(lstring, tmp);
+      free(tmp);
     }
 
-#else
-    if (vc->cutpoint == -1) {
-      vrna_message_info(stdout, "length = %d", n);
-    } else {
-      vrna_message_info(stdout,
-                        "length1 = %d\nlength2 = %d",
-                        vc->cutpoint - 1,
-                        n - vc->cutpoint + 1);
-    }
+    tmp     = vrna_strdup_printf(" }, total = %u, sequences = %u\n", vc->length, vc->strands);
+    lstring = vrna_string_append_cstring(lstring, tmp);
+    free(tmp);
 
-#endif
+    printf("%s", lstring);
+    vrna_string_free(lstring);
   }
 
   /*
@@ -797,8 +793,10 @@ process_alignment_record(struct record_data_msa *record)
 
   o_stream = (struct output_stream *)vrna_alloc(sizeof(struct output_stream));
 
-  if (!record->structure)
-    vrna_message_error("structure missing for record %d\n", record->number);
+  if (!record->structure) {
+    vrna_log_error("structure missing for record %d\n", record->number);
+    exit(EXIT_FAILURE);
+  }
 
   opt       = record->options;
   structure = vrna_db_from_WUSS(record->structure);
@@ -816,7 +814,7 @@ process_alignment_record(struct record_data_msa *record)
                                       VRNA_OPTION_DEFAULT | VRNA_OPTION_EVAL_ONLY);
 
   if (!vc) {
-    vrna_message_warning("Skipping computations for \"%s\"",
+    vrna_log_warning("Skipping computations for \"%s\"",
                          (record->MSA_ID) ? record->MSA_ID : "identifier unavailable");
     return;
   }
