@@ -24,31 +24,37 @@
 #include <unistd.h>
 #include <string.h>
 
-#include "ViennaRNA/fold.h"
-#include "ViennaRNA/part_func.h"
+#include "ViennaRNA/mfe/global.h"
+#include "ViennaRNA/partfunc/global.h"
+#include "ViennaRNA/eval/structures.h"
 #include "ViennaRNA/fold_vars.h"
 #include "ViennaRNA/plotting/probabilities.h"
 #include "ViennaRNA/plotting/structures.h"
 #include "ViennaRNA/utils/basic.h"
 #include "ViennaRNA/utils/strings.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/io/utils.h"
 #include "ViennaRNA/params/io.h"
-#include "ViennaRNA/centroid.h"
-#include "ViennaRNA/MEA.h"
+#include "ViennaRNA/mfe/global.h"
+#include "ViennaRNA/structures/centroid.h"
+#include "ViennaRNA/structures/mea.h"
 #include "ViennaRNA/params/basic.h"
 #include "ViennaRNA/constraints/basic.h"
-#include "ViennaRNA/constraints/SHAPE.h"
+#include "ViennaRNA/probing/SHAPE.h"
 #include "ViennaRNA/constraints/ligand.h"
 #include "ViennaRNA/constraints/soft_special.h"
 #include "ViennaRNA/structured_domains.h"
 #include "ViennaRNA/unstructured_domains.h"
 #include "ViennaRNA/io/file_formats.h"
-#include "ViennaRNA/commands.h"
-#include "ViennaRNA/equilibrium_probs.h"
+#include "ViennaRNA/io/commands.h"
+#include "ViennaRNA/probabilities/basepairs.h"
+#include "ViennaRNA/probabilities/structures.h"
+#include "ViennaRNA/sampling/basic.h"
 #include "ViennaRNA/datastructures/char_stream.h"
 #include "ViennaRNA/datastructures/stream_output.h"
-#include "ViennaRNA/combinatorics.h"
-#include "ViennaRNA/color_output.inc"
+#include "ViennaRNA/combinatorics/basic.h"
+#include "ViennaRNA/intern/color_output.h"
+#include "ViennaRNA/io/sanitize.h"
 
 #include "RNAfold_cmdl.h"
 #include "gengetopt_helpers.h"
@@ -62,6 +68,7 @@ struct options {
   char            *filename_delim;
   int             pf;
   int             noPS;
+  int             plot_layout;
   int             noDP;
   int             noconv;
   int             lucky;
@@ -223,14 +230,16 @@ postscript_layout(vrna_fold_compound_t  *fc,
                   const char            *orig_sequence,
                   const char            *structure,
                   const char            *SEQ_ID,
-                  const char            *ligandMotif,
-                  const char            *filename_delim,
-                  int                   verbose)
+                  struct options        *opt)
 {
   char      *filename_plot  = NULL;
   char      *annotation     = NULL;
   vrna_md_t *md             = &(fc->params->model_details);
 
+  char  *ligandMotif = opt->ligandMotif;
+  char  *filename_delim = opt->filename_delim;
+  int   verbose = opt->verbose;
+
   filename_plot = generate_filename("%s%sss.ps",
                                     "rna.ps",
                                     SEQ_ID,
@@ -250,13 +259,23 @@ postscript_layout(vrna_fold_compound_t  *fc,
     free(m);
   }
 
+  vrna_plot_data_t  aux_data;
+  aux_data.pre = annotation;
+  aux_data.post = NULL;
+  aux_data.md = md;
+  vrna_plot_layout_t  *layout;
+
+  layout = vrna_plot_layout(structure, opt->plot_layout);
+
   THREADSAFE_FILE_OUTPUT(
-    vrna_file_PS_rnaplot_a(orig_sequence,
-                           structure,
-                           filename_plot,
-                           annotation,
-                           NULL,
-                           md));
+    vrna_plot_structure(filename_plot,
+                        orig_sequence,
+                        structure,
+                        VRNA_FILE_FORMAT_PLOT_DEFAULT,
+                        layout,
+                        &aux_data));
+
+  vrna_plot_layout_free(layout);
   free(annotation);
   free(filename_plot);
 }
@@ -290,12 +309,23 @@ ImFeelingLucky(vrna_fold_compound_t *fc,
                                       SEQ_ID,
                                       filename_delim);
 
+    vrna_plot_data_t  aux_data;
+    aux_data.pre = NULL;
+    aux_data.post = NULL;
+    aux_data.md = md;
+    vrna_plot_layout_t  *layout;
+
+    layout = vrna_plot_layout(s, rna_plot_type);
+
     THREADSAFE_FILE_OUTPUT(
-      vrna_file_PS_rnaplot(orig_sequence,
-                           s,
-                           filename_plot,
-                           md));
-    free(filename_plot);
+      vrna_plot_structure(filename_plot,
+                          orig_sequence,
+                          s,
+                          VRNA_FILE_FORMAT_PLOT_DEFAULT,
+                          layout,
+                          &aux_data));
+
+    vrna_plot_layout_free(layout);
   }
 
   free(s);
@@ -333,7 +363,7 @@ collect_unnamed_options(struct RNAfold_args_info  *ggostruct,
   *num_files = 0;
 
   /* collect all unnamed options */
-  if (ggostruct->inputs_num > 0) {
+  if ((ggostruct->inputs_num > 0) && (!sanitize_input(ggostruct->inputs[0]))) {
     input_files = (char **)vrna_realloc(input_files, sizeof(char *) * ggostruct->inputs_num);
     for (i = 0; i < ggostruct->inputs_num; i++)
       input_files[(*num_files)++] = strdup(ggostruct->inputs[i]);
@@ -367,6 +397,7 @@ init_default_options(struct options *opt)
   opt->filename_delim = NULL;
   opt->pf             = 0;
   opt->noPS           = 0;
+  opt->plot_layout    = rna_plot_type;
   opt->noDP           = 0;
   opt->noconv         = 0;
   opt->lucky          = 0;
@@ -421,6 +452,9 @@ main(int  argc,
   if (RNAfold_cmdline_parser(argc, argv, &args_info) != 0)
     exit(1);
 
+  /* prepare logging system and verbose mode */
+  ggo_log_settings(args_info, opt.verbose);
+
   /* get basic set of model details */
   ggo_get_md_eval(args_info, opt.md);
   ggo_get_md_fold(args_info, opt.md);
@@ -432,7 +466,7 @@ main(int  argc,
 
   /* check dangle model */
   if ((opt.md.dangles < 0) || (opt.md.dangles > 3)) {
-    vrna_message_warning("required dangle model not implemented, falling back to default dangles=2");
+    vrna_log_warning("Requested dangle model not implemented, falling back to default dangles=2");
     opt.md.dangles = dangles = 2;
   }
 
@@ -488,10 +522,7 @@ main(int  argc,
   }
 
   if (args_info.layout_type_given)
-    rna_plot_type = args_info.layout_type_arg;
-
-  if (args_info.verbose_given)
-    opt.verbose = 1;
+    opt.plot_layout = rna_plot_type = args_info.layout_type_arg;
 
   if (args_info.outfile_given) {
     opt.tofile = 1;
@@ -533,8 +564,8 @@ main(int  argc,
       if (num_proc_cores(&proc_cores, &proc_cores_conf)) {
         opt.jobs = MIN2(thread_max, proc_cores_conf);
       } else {
-        vrna_message_warning("Could not determine number of available processor cores!\n"
-                             "Defaulting to serial computation");
+        vrna_log_warning("Could not determine number of available processor cores!\n"
+                         "Defaulting to serial computation");
         opt.jobs = 1;
       }
     } else {
@@ -543,7 +574,7 @@ main(int  argc,
 
     opt.jobs = MAX2(1, opt.jobs);
 #else
-    vrna_message_warning(
+    vrna_log_warning(
       "This version of RNAfold has been built without parallel input processing capabilities");
 #endif
 
@@ -563,17 +594,19 @@ main(int  argc,
    # begin initializing
    #############################################
    */
+#if 0
   if (opt.md.circ && opt.md.gquad) {
-    vrna_message_error("G-Quadruplex support is currently not available for circular RNA structures");
+    vrna_log_error("G-Quadruplex support is currently not available for circular RNA structures");
     exit(EXIT_FAILURE);
   }
+#endif
 
   if (opt.md.circ && opt.md.noLP)
-    vrna_message_warning("depending on the origin of the circular sequence, some structures may be missed when using --noLP\n"
-                         "Try rotating your sequence a few times");
+    vrna_log_warning("Depending on the origin of the circular sequence, some structures may be missed when using --noLP\n"
+             "Try rotating your sequence a few times");
 
   if ((opt.verbose) && (opt.jobs > 1))
-    vrna_message_info(stderr, "Preparing %d parallel computation slots", opt.jobs);
+    vrna_log_info("Preparing %d parallel computation slots", opt.jobs);
 
   if (opt.keep_order)
     opt.output_queue = vrna_ostream_init(&flush_cstr_callback, NULL);
@@ -591,15 +624,17 @@ main(int  argc,
       if (!skip) {
         FILE *input_stream = fopen((const char *)input_files[i], "r");
 
-        if (!input_stream)
-          vrna_message_error("Unable to open %d. input file \"%s\" for reading", i + 1,
-                             input_files[i]);
+        if (!input_stream) {
+          vrna_log_error("Unable to open %d. input file \"%s\" for reading",
+                         i + 1,
+                         input_files[i]);
+          exit(EXIT_FAILURE);
+        }
 
         if (opt.verbose) {
-          vrna_message_info(stderr,
-                            "Processing %d. input file \"%s\"",
-                            i + 1,
-                            input_files[i]);
+          vrna_log_info("Processing %d. input file \"%s\"",
+                        i + 1,
+                        input_files[i]);
         }
 
         if (process_input(input_stream, (const char *)input_files[i], &opt) == 0)
@@ -646,6 +681,9 @@ main(int  argc,
 
   free_id_data(opt.id_control);
 
+  if (vrna_log_fp() != stderr)
+    fclose(vrna_log_fp());
+
   return EXIT_SUCCESS;
 }
 
@@ -684,11 +722,15 @@ get_output_stream(unsigned int    init_size,
 
         filename = vrna_filename_sanitize(tmp, opt->filename_delim);
 
-        if ((input_filename) && !strcmp(input_filename, filename))
-          vrna_message_error("Input and output file names are identical");
+        if ((input_filename) && !strcmp(input_filename, filename)) {
+          vrna_log_error("Input and output file names are identical");
+          exit(EXIT_FAILURE);
+        }
 
-        if (!(output = fopen(filename, "a")))
-          vrna_message_error("Failed to open file for writing");
+        if (!(output = fopen(filename, "a"))) {
+          vrna_log_error("Failed to open file for writing");
+          exit(EXIT_FAILURE);
+        }
       } else if (!output) {
         /* we need to open global output file */
         tmp = (opt->output_file) ?
@@ -697,11 +739,15 @@ get_output_stream(unsigned int    init_size,
 
         filename = vrna_filename_sanitize(tmp, opt->filename_delim);
 
-        if ((input_filename) && !strcmp(input_filename, filename))
-          vrna_message_error("Input and output file names are identical");
+        if ((input_filename) && !strcmp(input_filename, filename)) {
+          vrna_log_error("Input and output file names are identical");
+          exit(EXIT_FAILURE);
+        }
 
-        if (!(output = fopen(filename, "a")))
-          vrna_message_error("Failed to open file for writing");
+        if (!(output = fopen(filename, "a"))) {
+          vrna_log_error("Failed to open file for writing");
+          exit(EXIT_FAILURE);
+        }
 
         opt->output_stream = output;
       }
@@ -852,17 +898,17 @@ process_record(struct record_data *record)
   vc = vrna_fold_compound(rec_sequence, &(opt->md), VRNA_OPTION_DEFAULT);
 
   if (!vc) {
-    vrna_message_warning("Skipping computations for \"%s\"",
-                         (record->id) ? record->id : "identifier unavailable");
+    vrna_log_warning("Skipping computations for \"%s\"",
+                     (record->id) ? record->id : "identifier unavailable");
     return;
   }
 
   length = vc->length;
 
   if ((opt->md.circ) && (vrna_rotational_symmetry(rec_sequence) > 1))
-    vrna_message_warning("Input sequence %ld is rotationally symmetric! "
-                         "Symmetry correction might be required to compute actual MFE and equilibrium properties!",
-                         record->number);
+    vrna_log_warning("Input sequence %ld is rotationally symmetric! "
+                     "Symmetry correction might be required to compute actual MFE and equilibrium properties!",
+                     record->number);
 
   /* retrieve string stream, 6*length should be enough memory to start with */
   o_stream = get_output_stream(6 * length,
@@ -871,7 +917,7 @@ process_record(struct record_data *record)
                                record->input_filename);
 
   if (record->tty)
-    vrna_message_info(stdout, "length = %d\n", length);
+    vrna_log_info("length = %d\n", length);
 
   mfe_structure = (char *)vrna_alloc(sizeof(char) * (length + 1));
 
@@ -928,7 +974,7 @@ process_record(struct record_data *record)
   /* check whether the constraint allows for any solution */
   if ((fold_constrained) || (opt->cmds)) {
     if (min_en == (double)(INF / 100.)) {
-      vrna_message_error(
+      vrna_log_error(
         "Supplied structure constraints create empty solution set for sequence:\n%s",
         record->sequence);
       exit(EXIT_FAILURE);
@@ -957,9 +1003,7 @@ process_record(struct record_data *record)
                         record->sequence,
                         mfe_structure,
                         record->SEQ_ID,
-                        opt->ligandMotif,
-                        opt->filename_delim,
-                        opt->verbose);
+                        opt);
     }
   }
 
@@ -978,13 +1022,13 @@ process_record(struct record_data *record)
     vrna_exp_params_rescale(vc, &min_en);
 
     if (length > 2000)
-      vrna_message_info(stderr, "scaling factor %f", vc->exp_params->pf_scale);
+      vrna_log_info("scaling factor %f", vc->exp_params->pf_scale);
 
     energy = (double)vrna_pf(vc, pf_struc);
 
     /* in case we abort because of floating point errors */
     if (length > 1600)
-      vrna_message_info(stderr, "free energy = %8.2f", energy);
+      vrna_log_info("free energy = %8.2f", energy);
 
     if (opt->lucky) {
       ImFeelingLucky(vc,
@@ -1139,12 +1183,14 @@ apply_constraints(vrna_fold_compound_t  *fc,
     cstruc = vrna_extract_record_rest_structure((const char **)rec_rest, 0, coptions);
     unsigned int  cl = (cstruc) ? strlen(cstruc) : 0;
 
-    if (cl == 0)
-      vrna_message_warning("structure constraint is missing");
-    else if (cl < length)
-      vrna_message_warning("structure constraint is shorter than sequence");
-    else if (cl > length)
-      vrna_message_error("structure constraint is too long");
+    if (cl == 0) {
+      vrna_log_warning("structure constraint is missing");
+    } else if (cl < length) {
+      vrna_log_warning("structure constraint is shorter than sequence");
+    } else if (cl > length) {
+      vrna_log_error("structure constraint is too long");
+      exit(EXIT_FAILURE);
+    }
 
     if (cstruc) {
       /** [Adding hard constraints from pseudo dot-bracket] */
@@ -1269,26 +1315,26 @@ add_ligand_motif(vrna_fold_compound_t *vc,
 
     ptr++;
     if (!(sscanf(ptr, "%f", &energy) == 1)) {
-      vrna_message_warning("Energy contribution in ligand motif missing!");
+      vrna_log_warning("Energy contribution in ligand motif missing!");
       error = 1;
     }
 
     if (strlen(seq) != strlen(str)) {
-      vrna_message_warning("Sequence and structure length in ligand motif have unequal lengths!");
+      vrna_log_warning("Sequence and structure length in ligand motif have unequal lengths!");
       error = 1;
     }
 
     if (strlen(seq) == 0) {
-      vrna_message_warning("Sequence length in ligand motif is zero!");
+      vrna_log_warning("Sequence length in ligand motif is zero!");
       error = 1;
     }
 
     if (!error && verbose)
-      vrna_message_info(stderr, "Read ligand motif: %s, %s, %f", seq, str, energy);
+      vrna_log_info("Read ligand motif: %s, %s, %f", seq, str, energy);
   }
 
   if (error || (!vrna_sc_add_hi_motif(vc, seq, str, energy, options)))
-    vrna_message_warning("Malformatted ligand motif! Skipping stabilizing motif.");
+    vrna_log_warning("Malformatted ligand motif! Skipping stabilizing motif.");
 
   free(seq);
   free(str);
@@ -1492,7 +1538,7 @@ add_ligand_motifs_to_list(vrna_ep_t       **list,
         (*list) = vrna_realloc((*list), sizeof(vrna_ep_t) * (size + add + 1));
       }
     } else {
-      /* interior loop motif */
+      /* internal loop motif */
       (*list)[size + cnt].i     = m_ptr->i;
       (*list)[size + cnt].j     = m_ptr->j;
       (*list)[size + cnt].p     = 0.95 * 0.95;
