@@ -18,33 +18,36 @@
 #include "ViennaRNA/plotting/structures.h"
 #include "ViennaRNA/fold_vars.h"
 #include "ViennaRNA/params/basic.h"
-#include "ViennaRNA/commands.h"
+#include "ViennaRNA/io/commands.h"
 #include "ViennaRNA/constraints/basic.h"
 #include "ViennaRNA/constraints/hard.h"
 #include "ViennaRNA/constraints/soft.h"
-#include "ViennaRNA/constraints/SHAPE.h"
+#include "ViennaRNA/probing/SHAPE.h"
 #include "ViennaRNA/io/file_formats.h"
 #include "ViennaRNA/io/utils.h"
 #include "ViennaRNA/cofold.h"
-#include "ViennaRNA/fold.h"
+#include "ViennaRNA/mfe/global.h"
 #include "ViennaRNA/part_func_co.h"
-#include "ViennaRNA/part_func.h"
-#include "ViennaRNA/centroid.h"
-#include "ViennaRNA/MEA.h"
+#include "ViennaRNA/partfunc/global.h"
+#include "ViennaRNA/structures/centroid.h"
+#include "ViennaRNA/structures/mea.h"
+#include "ViennaRNA/eval/structures.h"
 #include "ViennaRNA/utils/basic.h"
 #include "ViennaRNA/utils/strings.h"
-#include "ViennaRNA/utils/structures.h"
+#include "ViennaRNA/structures/pairtable.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/params/io.h"
 #include "ViennaRNA/datastructures/char_stream.h"
 #include "ViennaRNA/datastructures/stream_output.h"
+#include "ViennaRNA/datastructures/string.h"
 #include "ViennaRNA/concentrations.h"
-#include "ViennaRNA/combinatorics.h"
+#include "ViennaRNA/combinatorics/basic.h"
 #include "ViennaRNA/wrap_dlib.h"
 
 #include "RNAmultifold_cmdl.h"
 #include "gengetopt_helpers.h"
 #include "input_id_helpers.h"
-#include "ViennaRNA/color_output.inc"
+#include "ViennaRNA/intern/color_output.h"
 #include "parallel_helpers.h"
 
 
@@ -246,6 +249,9 @@ main(int  argc,
   if (RNAmultifold_cmdline_parser(argc, argv, &args_info) != 0)
     exit(1);
 
+  /* prepare logging system and verbose mode */
+  ggo_log_settings(args_info, opt.verbose);
+
   /* get basic set of model details */
   ggo_get_md_eval(args_info, opt.md);
   ggo_get_md_fold(args_info, opt.md);
@@ -256,7 +262,7 @@ main(int  argc,
 
   /* check dangle model */
   if ((opt.md.dangles < 0) || (opt.md.dangles > 3)) {
-    vrna_message_warning("required dangle model not implemented, falling back to default dangles=2");
+    vrna_log_warning("required dangle model not implemented, falling back to default dangles=2");
     opt.md.dangles = dangles = 2;
   }
 
@@ -302,9 +308,6 @@ main(int  argc,
   if (args_info.commands_given)
     opt.cmds = vrna_file_commands_read(args_info.commands_arg, VRNA_CMD_PARSE_DEFAULTS);
 
-  if (args_info.verbose_given)
-    opt.verbose = 1;
-
   /* filename sanitize delimiter */
   if (args_info.filename_delim_given)
     opt.filename_delim = strdup(args_info.filename_delim_arg);
@@ -329,7 +332,7 @@ main(int  argc,
       if (num_proc_cores(&proc_cores, &proc_cores_conf)) {
         opt.jobs = MIN2(thread_max, proc_cores_conf);
       } else {
-        vrna_message_warning("Could not determine number of available processor cores!\n"
+        vrna_log_warning("Could not determine number of available processor cores!\n"
                              "Defaulting to serial computation");
         opt.jobs = 1;
       }
@@ -339,7 +342,7 @@ main(int  argc,
 
     opt.jobs = MAX2(1, opt.jobs);
 #else
-    vrna_message_warning(
+    vrna_log_warning(
       "This version of RNAfold has been built without parallel input processing capabilities");
 #endif
 
@@ -359,12 +362,13 @@ main(int  argc,
    # begin initializing
    #############################################
    */
-  if (opt.pf && opt.md.gquad)
-    vrna_message_error(
+  if (opt.pf && opt.md.gquad) {
+    vrna_log_error(
       "G-Quadruplex support is currently not available for partition function computations");
-
+    exit(EXIT_FAILURE);
+  }
   if ((opt.verbose) && (opt.jobs > 1))
-    vrna_message_info(stderr, "Preparing %d parallel computation slots", opt.jobs);
+    vrna_log_info("Preparing %d parallel computation slots", opt.jobs);
 
   if (opt.keep_order)
     opt.output_queue = vrna_ostream_init(&flush_cstr_callback, NULL);
@@ -382,13 +386,14 @@ main(int  argc,
       if (!skip) {
         FILE *input_stream = fopen((const char *)input_files[i], "r");
 
-        if (!input_stream)
-          vrna_message_error("Unable to open %d. input file \"%s\" for reading", i + 1,
+        if (!input_stream) {
+          vrna_log_error("Unable to open %d. input file \"%s\" for reading", i + 1,
                              input_files[i]);
+          exit(EXIT_FAILURE);
+        }
 
         if (opt.verbose) {
-          vrna_message_info(stderr,
-                            "Processing %d. input file \"%s\"",
+          vrna_log_info("Processing %d. input file \"%s\"",
                             i + 1,
                             input_files[i]);
         }
@@ -425,6 +430,9 @@ main(int  argc,
 
   free_id_data(opt.id_control);
 
+  if (vrna_log_fp() != stderr)
+    fclose(vrna_log_fp());
+
   return EXIT_SUCCESS;
 }
 
@@ -581,21 +589,36 @@ process_record(struct record_data *record)
   o_stream->err = vrna_cstr(n, stderr);
 
   if (record->tty) {
-    if (vc->cutpoint == -1) {
-      vrna_message_info(stdout, "length = %u", n);
-    } else {
-      vrna_message_info(stdout,
-                        "length1 = %d\nlength2 = %d",
-                        vc->cutpoint - 1,
-                        (int)n - vc->cutpoint + 1);
+    char          *tmp;
+    vrna_string_t lstring;
+
+    lstring = vrna_string_make("length(s) = {");
+    tmp     = vrna_strdup_printf(" %u", 1 + vc->strand_end[0] - vc->strand_start[0]);
+
+    lstring = vrna_string_append_cstring(lstring, tmp);
+    free(tmp);
+
+    for (unsigned int i = 1; i < vc->strands; i++) {
+      char *tmp = vrna_strdup_printf(", %u", 1 + vc->strand_end[i] - vc->strand_start[i]);
+      lstring = vrna_string_append_cstring(lstring, tmp);
+      free(tmp);
     }
+
+    tmp     = vrna_strdup_printf(" }, total = %u, sequences = %u\n", vc->length, vc->strands);
+    lstring = vrna_string_append_cstring(lstring, tmp);
+    free(tmp);
+
+    printf("%s", lstring);
+    vrna_string_free(lstring);
   }
 
-  if (vc->cutpoint == vc->length / 2 + 1) {
-    if (!strncmp(vc->sequence, vc->sequence + vc->cutpoint - 1, vc->cutpoint - 1)) {
+  if (vc->strands > 1) {
+    unsigned int r = vrna_rotational_symmetry(vc->sequence);
+    if (r > 1) {
       vrna_cstr_message_warning(o_stream->err,
-                                "Both input strands are identical, thus inducing rotationally symmetry! "
-                                "Symmetry correction might be required to compute actual MFE!");
+                                "The input sequences show a rotational symmetry of %u! "
+                                "Symmetry correction might be required to compute actual MFE!",
+                                r);
     }
   }
 
@@ -612,21 +635,56 @@ process_record(struct record_data *record)
       int           cp        = -1;
 
       cstruc  = vrna_extract_record_rest_structure((const char **)rec_rest, 0, coptions);
-      cstruc  = vrna_cut_point_remove(cstruc, &cp);
-      if (vc->cutpoint != cp) {
-        vrna_message_error("Sequence and Structure have different cut points.\n"
-                           "sequence: %d, structure: %d",
-                           vc->cutpoint, cp);
+
+      char **structures = vrna_strsplit(cstruc, "&");
+      for (unsigned int a = 0; a < vc->strands; a++) {
+        if (!structures[a]) {
+          vrna_log_error("Sequence and Structure have different number of strand delimiters");
+          exit(EXIT_FAILURE);
+        }
+
+        unsigned int l = strlen(structures[a]);
+        cl += l;
+
+        switch (vc->type) {
+          case VRNA_FC_TYPE_SINGLE:
+            if (vc->nucleotides[a].length != l) {
+              vrna_log_error(
+                "Structure and sequence part of strand %u differ in length (%u vs. %u)",
+                a,
+                l,
+                vc->nucleotides[a].length);
+              exit(EXIT_FAILURE);
+            }
+
+            break;
+
+          case VRNA_FC_TYPE_COMPARATIVE:
+            if (vc->alignment[a].sequences[0].length != l) {
+              vrna_log_error(
+                "Structure and sequence part of strand %u differ in length (%u vs. %u)",
+                a,
+                l,
+                vc->alignment[a].sequences[0].length);
+              exit(EXIT_FAILURE);
+            }
+            break;
+        }
       }
 
-      cl = (cstruc) ? (int)strlen(cstruc) : 0;
+      for (unsigned int a = 0; a < vc->strands; a++)
+        free(structures[a]);
+
+      free(structures);
 
       if (cl == 0)
-        vrna_message_warning("Structure constraint is missing");
+        vrna_log_warning("Structure constraint is missing");
       else if (cl < n)
-        vrna_message_warning("Structure constraint is shorter than sequence");
-      else if (cl > n)
-        vrna_message_error("Structure constraint is too long");
+        vrna_log_warning("Structure constraint is shorter than sequence(s)");
+      else if (cl > n) {
+        vrna_log_error("Structure constraint is too long");
+        exit(EXIT_FAILURE);
+      }
 
       if (cstruc) {
         unsigned int constraint_options = VRNA_CONSTRAINT_DB_DEFAULT;
@@ -660,8 +718,10 @@ process_record(struct record_data *record)
     if (opt->concentration_file) {
       /* read from file */
       FILE *fp = fopen(opt->concentration_file, "r");
-      if (fp == NULL)
-        vrna_message_error("could not open concentration file %s", opt->concentration_file);
+      if (fp == NULL) {
+        vrna_log_error("could not open concentration file %s", opt->concentration_file);
+        exit(EXIT_FAILURE);
+      }
 
       concentrations = read_concentrations(fp, vc->strands);
       fclose(fp);
@@ -687,7 +747,7 @@ process_record(struct record_data *record)
   /* check whether the constraint allows for any solution */
   if ((fold_constrained) || (opt->cmds)) {
     if (min_en == (double)(INF / 100.)) {
-      vrna_message_error(
+      vrna_log_error(
         "Supplied structure constraints create empty solution set for sequence:\n%s",
         record->sequence);
       exit(EXIT_FAILURE);
@@ -1423,7 +1483,7 @@ read_concentrations(FILE    *fp,
     if (s == num_strands)
       i++;
     else
-      vrna_message_warning("Failed to parse all concentrations (%u) from line %d:\n%s\n",
+      vrna_log_warning("Failed to parse all concentrations (%u) from line %d:\n%s\n",
                            num_strands,
                            i,
                            line);
