@@ -20,16 +20,19 @@
 #include <unistd.h>
 #include <string.h>
 #include "ViennaRNA/utils/basic.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/params/convert.h"
 #include "RNAparconv_cmdl.h"
 
+#include "gengetopt_helpers.h"
+
 int
 main(int  argc,
      char *argv[])
 {
   struct  RNAparconv_args_info  args_info;
   char                          *ifileName, *ofileName;
-  int                           i, silent;
+  int                           i, silent, verbose;
   unsigned int                  options;
 
   ifileName = ofileName = NULL;
@@ -44,6 +47,9 @@ main(int  argc,
   if (RNAparconv_cmdline_parser(argc, argv, &args_info) != 0)
     exit(1);
 
+  /* prepare logging system and verbose mode */
+  ggo_log_settings(args_info, verbose);
+
   /* use input file */
   if (args_info.input_given)
     ifileName = strdup(args_info.input_arg);
@@ -69,7 +75,7 @@ main(int  argc,
   if (args_info.without_Dangle3_given)
     options &= ~VRNA_CONVERT_OUTPUT_DANGLE3;
 
-  /* output without interior loop energies + enthalpies */
+  /* output without internal loop energies + enthalpies */
   if (args_info.without_IntE_given) {
     options &= ~(VRNA_CONVERT_OUTPUT_NINIO
                  | VRNA_CONVERT_OUTPUT_INT
@@ -94,7 +100,7 @@ main(int  argc,
   if (args_info.without_MismatchH_given)
     options &= ~VRNA_CONVERT_OUTPUT_MM_HP;
 
-  /* output without interior loop mismatch energies + enthalpies */
+  /* output without internal loop mismatch energies + enthalpies */
   if (args_info.without_MismatchI_given)
     options &= ~(VRNA_CONVERT_OUTPUT_MM_INT
                  | VRNA_CONVERT_OUTPUT_MM_INT_1N
@@ -124,7 +130,8 @@ main(int  argc,
       ofileName = strdup(args_info.inputs[i]);
     } else {
       RNAparconv_cmdline_parser_print_help();
-      vrna_message_error("unrecognized or too many parameter options given!");
+      vrna_log_error("unrecognized or too many parameter options given!");
+      exit(EXIT_FAILURE);
     }
   }
 
@@ -149,5 +156,8 @@ main(int  argc,
 
   convert_parameter_file(ifileName, ofileName, options);
 
+  if (vrna_log_fp() != stderr)
+    fclose(vrna_log_fp());
+
   return EXIT_SUCCESS;
 }
