@@ -15,28 +15,29 @@
 #include <ctype.h>
 #include <unistd.h>
 #include <string.h>
-#include "ViennaRNA/fold.h"
-#include "ViennaRNA/part_func.h"
+#include "ViennaRNA/mfe/local.h"
+#include "ViennaRNA/partfunc/local.h"
 #include "ViennaRNA/fold_vars.h"
 #include "ViennaRNA/utils/basic.h"
 #include "ViennaRNA/utils/strings.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/plotting/probabilities.h"
 #include "ViennaRNA/params/constants.h"
 #include "ViennaRNA/params/io.h"
 #include "ViennaRNA/LPfold.h"
 #include "ViennaRNA/params/basic.h"
-#include "ViennaRNA/constraints/SHAPE.h"
+#include "ViennaRNA/probing/SHAPE.h"
 #include "ViennaRNA/constraints/soft_special.h"
 #include "ViennaRNA/io/file_formats.h"
 #include "ViennaRNA/io/utils.h"
-#include "ViennaRNA/commands.h"
+#include "ViennaRNA/io/commands.h"
 
 #include "RNAplfold_cmdl.h"
 #include "gengetopt_helpers.h"
 #include "input_id_helpers.h"
 #include "modified_bases_helpers.h"
 
-#include "ViennaRNA/color_output.inc"
+#include "ViennaRNA/intern/color_output.h"
 
 #ifndef isnan
 #define isnan(x) \
@@ -186,8 +187,8 @@ main(int  argc,
   if (RNAplfold_cmdline_parser(argc, argv, &args_info) != 0)
     exit(1);
 
-  if (args_info.verbose_given)
-    verbose = 1;
+  /* prepare logging system and verbose mode */
+  ggo_log_settings(args_info, verbose);
 
   /* SHAPE reactivity data */
   ggo_get_SHAPE(args_info, with_shapes, shape_file, shape_method, shape_conversion);
@@ -208,7 +209,7 @@ main(int  argc,
   /* set dangle model */
   if (args_info.dangles_given) {
     if ((args_info.dangles_arg != 0) && (args_info.dangles_arg != 2))
-      vrna_message_warning(
+      vrna_log_warning(
         "required dangle model not implemented, falling back to default dangles=2");
     else
       md.dangles = dangles = args_info.dangles_arg;
@@ -330,14 +331,14 @@ main(int  argc,
     pairdist = winsize;
 
   if (pairdist > winsize) {
-    vrna_message_warning("pairdist (-L %d) should be <= winsize (-W %d);"
+    vrna_log_warning("pairdist (-L %d) should be <= winsize (-W %d);"
                          "Setting pairdist=winsize",
                          pairdist, winsize);
     pairdist = winsize;
   }
 
   if (dangles % 2) {
-    vrna_message_warning("using default dangles = 2");
+    vrna_log_warning("using default dangles = 2");
     md.dangles = dangles = 2;
   }
 
@@ -390,7 +391,7 @@ main(int  argc,
     vrna_seq_toupper(rec_sequence);
 
     if (istty)
-      vrna_message_info(stdout, "length = %d", length);
+      printf("length = %d", length);
 
     /*
      ########################################################
@@ -400,19 +401,19 @@ main(int  argc,
 
     if (length > 1000000) {
       if (!simply_putout && !unpaired) {
-        vrna_message_warning("Switched to simple output mode!!!");
+        vrna_log_warning("Switched to simple output mode!!!");
         simply_putout = 1;
       }
     }
 
     if ((simply_putout) && (plexoutput)) {
-      vrna_message_warning("plexoutput not available in simple output mode!\n"
+      vrna_log_warning("plexoutput not available in simple output mode!\n"
                            "Switching back to full mode instead!");
       simply_putout = 0;
     }
 
     if ((simply_putout) && (binaries)) {
-      vrna_message_warning("binary output not available in simple output mode!\n"
+      vrna_log_warning("binary output not available in simple output mode!\n"
                            "Switching back to full mode instead!");
       simply_putout = 0;
     }
@@ -437,7 +438,7 @@ main(int  argc,
 
     /* adjust winsize, pairdist and ulength if necessary */
     if (length < winsize) {
-      vrna_message_warning("window size %d larger than sequence length %d", winsize, length);
+      vrna_log_warning("window size %d larger than sequence length %d", winsize, length);
       tempwin = winsize;
       winsize = length;
       if (pairdist > winsize) {
@@ -565,7 +566,7 @@ main(int  argc,
       int r = vrna_probs_window(fc, unpaired, plfold_opt, &plfold_callback, (void *)&data);
 
       if (!r) {
-        vrna_message_warning("Something bad happened while processing the input! "
+        vrna_log_warning("Something bad happened while processing the input! "
                              "Aborting now...");
         goto rnaplfold_exit;
       }
@@ -670,6 +671,9 @@ rnaplfold_exit:
 
   free_id_data(id_control);
 
+  if (vrna_log_fp() != stderr)
+    fclose(vrna_log_fp());
+
   return EXIT_SUCCESS;
 }
 
