@@ -14,6 +14,7 @@
 #include <unistd.h>
 #include "ViennaRNA/utils/basic.h"
 #include "ViennaRNA/utils/strings.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/plotting/probabilities.h"
 #include "ViennaRNA/plotting/structures.h"
 #include "ViennaRNA/plotting/alignments.h"
@@ -23,7 +24,7 @@
 #include "ViennaRNA/io/file_formats.h"
 #include "ViennaRNA/io/file_formats_msa.h"
 #include "ViennaRNA/io/utils.h"
-#include "ViennaRNA/utils/alignments.h"
+#include "ViennaRNA/sequences/alignments.h"
 
 #include "RNAplot_cmdl.h"
 #include "gengetopt_helpers.h"
@@ -33,12 +34,15 @@
 #define PRIVATE static
 
 struct options {
+  int           verbose;
   unsigned int  msa_format;
   int           filename_full;
   char          *filename_delim;
   int           msa;
   int           mis;
   int           annotate_covar;
+  double        covar_threshold;
+  double        covar_min_sat;
   int           aln_PS;
   int           aln_PS_cols;
   vrna_md_t     md;
@@ -46,8 +50,8 @@ struct options {
 
   char          *pre;
   char          *post;
-  char          format[5];
-  int           plot_type;
+  char          format[6];
+  unsigned int  plot_type;
 
   int           jobs;
   unsigned int  next_record_number;
@@ -108,12 +112,15 @@ process_alignment_record(struct record_data_msa *record);
 void
 init_default_options(struct options *opt)
 {
+  opt->verbose        = 0;
   opt->msa_format     = VRNA_FILE_FORMAT_MSA_STOCKHOLM;
   opt->filename_full  = 0;
   opt->filename_delim = NULL;
   opt->msa            = 0;
   opt->mis            = 0;
   opt->annotate_covar = 0;
+  opt->covar_threshold  = 2;
+  opt->covar_min_sat    = 0.2;
   opt->aln_PS         = 0;
   opt->aln_PS_cols    = 60;
   vrna_md_set_default(&(opt->md));
@@ -121,8 +128,8 @@ init_default_options(struct options *opt)
   opt->pre        = NULL;
   opt->post       = NULL;
   opt->plot_type  = 1;
-  strncpy(&(opt->format[0]), "ps", 4);
-  opt->format[4] = '\0';
+  strncpy(&(opt->format[0]), "eps", 5);
+  opt->format[5] = '\0';
 
   opt->jobs                       = 1;
   opt->next_record_number         = 0;
@@ -193,6 +200,9 @@ main(int  argc,
   if (RNAplot_cmdline_parser(argc, argv, &args_info) != 0)
     exit(1);
 
+  /* prepare logging system and verbose mode */
+  ggo_log_settings(args_info, opt.verbose);
+
   if (args_info.msa_given) {
     opt.msa = 1;
 
@@ -202,6 +212,12 @@ main(int  argc,
     if (args_info.covar_given)
       opt.annotate_covar = 1;
 
+    if (args_info.covar_threshold_given)
+      opt.covar_threshold = args_info.covar_threshold_arg;
+
+    if (args_info.covar_min_sat_given)
+      opt.covar_min_sat = args_info.covar_min_sat_arg;
+
     if (args_info.aln_given)
       opt.aln_PS = 1;
 
@@ -209,13 +225,14 @@ main(int  argc,
       opt.aln_PS_cols = args_info.aln_EPS_cols_arg;
   }
 
-  if (opt.msa)
+  if (opt.msa) {
     ggo_get_id_control(args_info, opt.id_control, "Alignment", "alignment", "_", 4, 1);
-  else
+  } else {
     ggo_get_id_control(args_info, opt.id_control, "Sequence", "sequence", "_", 4, 1);
+  }
 
   if (args_info.layout_type_given)
-    rna_plot_type = args_info.layout_type_arg;
+    opt.plot_type = (unsigned int)args_info.layout_type_arg;
 
   if (args_info.pre_given)
     opt.pre = strdup(args_info.pre_arg);
@@ -224,8 +241,8 @@ main(int  argc,
     opt.post = strdup(args_info.post_arg);
 
   if (args_info.output_format_given) {
-    strncpy(&(opt.format[0]), args_info.output_format_arg, 4);
-    opt.format[4] = '\0';
+    strncpy(&(opt.format[0]), args_info.output_format_arg, 5);
+    opt.format[5] = '\0';
   }
 
   /* filename sanitize delimiter */
@@ -254,7 +271,7 @@ main(int  argc,
       if (num_proc_cores(&proc_cores, &proc_cores_conf)) {
         opt.jobs = MIN2(thread_max, proc_cores_conf);
       } else {
-        vrna_message_warning("Could not determine number of available processor cores!\n"
+        vrna_log_warning("Could not determine number of available processor cores!\n"
                              "Defaulting to serial computation");
         opt.jobs = 1;
       }
@@ -264,7 +281,7 @@ main(int  argc,
 
     opt.jobs = MAX2(1, opt.jobs);
 #else
-    vrna_message_warning(
+    vrna_log_warning(
       "This version of RNAplot has been built without parallel input processing capabilities");
 #endif
   }
@@ -303,10 +320,11 @@ main(int  argc,
       if (!skip) {
         FILE *input_stream = fopen((const char *)input_files[i], "r");
 
-        if (!input_stream)
-          vrna_message_error("Unable to open %d. input file \"%s\" for reading", i + 1,
+        if (!input_stream) {
+          vrna_log_error("Unable to open %d. input file \"%s\" for reading", i + 1,
                              input_files[i]);
-
+          goto exit_fail;
+        }
         if (processing_func(input_stream, (const char *)input_files[i], &opt) == 0)
           skip = 1;
 
@@ -333,7 +351,13 @@ main(int  argc,
 
   free_id_data(opt.id_control);
 
+  if (vrna_log_fp() != stderr)
+    fclose(vrna_log_fp());
+
   return EXIT_SUCCESS;
+
+  exit_fail:
+    return EXIT_FAILURE;
 }
 
 
@@ -433,24 +457,24 @@ process_alignment_input(FILE            *input_stream,
 
       switch (opt->msa_format) {
         case VRNA_FILE_FORMAT_MSA_CLUSTAL:
-          vrna_message_error(msg, "Clustal");
-          break;
+          vrna_log_error(msg, "Clustal");
+          return 0;
 
         case VRNA_FILE_FORMAT_MSA_STOCKHOLM:
-          vrna_message_error(msg, "Stockholm");
-          break;
+          vrna_log_error(msg, "Stockholm");
+          return 0;
 
         case VRNA_FILE_FORMAT_MSA_FASTA:
-          vrna_message_error(msg, "FASTA");
-          break;
+          vrna_log_error(msg, "FASTA");
+          return 0;
 
         case VRNA_FILE_FORMAT_MSA_MAF:
-          vrna_message_error(msg, "MAF");
-          break;
+          vrna_log_error(msg, "MAF");
+          return 0;
 
         default:
-          vrna_message_error(msg, "Unknown");
-          break;
+          vrna_log_error(msg, "Unknown");
+          return 0;
       }
     }
 
@@ -490,8 +514,8 @@ process_alignment_input(FILE            *input_stream,
           break;
 
         default:
-          vrna_message_error("Which input format are you using?");
-          break;
+          vrna_log_error("Which input format are you using?");
+          return 0;
       }
     }
 
@@ -560,61 +584,40 @@ process_record(struct record_data *record)
     0,
     (record->multiline_input) ? VRNA_OPTION_MULTILINE : 0);
 
-  if (!structure)
-    vrna_message_error("structure missing for record %d\n", record->number);
+  if (!structure) {
+    vrna_log_error("structure missing for record %d\n", record->number);
+    exit(EXIT_FAILURE);
+  }
 
-  if (strlen(rec_sequence) != strlen(structure))
-    vrna_message_error("sequence and structure have unequal length");
+  if (strlen(rec_sequence) != strlen(structure)) {
+    vrna_log_error("sequence and structure have unequal length");
+    exit(EXIT_FAILURE);
+  }
 
   if (record->SEQ_ID)
     ffname = vrna_strdup_printf("%s%sss", record->SEQ_ID, opt->filename_delim);
   else
     ffname = vrna_strdup_printf("rna");
 
+  vrna_plot_data_t            *data       = NULL;
+  vrna_plot_layout_t          *layout     = NULL;
+  vrna_plot_options_puzzler_t *puzzler    = NULL;
+  unsigned int                file_format = VRNA_FILE_FORMAT_PLOT_DEFAULT;
+
   switch (opt->format[0]) {
-    case 'p':
-      tmp_string = vrna_strdup_printf("%s.ps", ffname);
-      free(ffname);
-      ffname = vrna_filename_sanitize(tmp_string, opt->filename_delim);
-      free(tmp_string);
-
-      if (rna_plot_type == VRNA_PLOT_TYPE_PUZZLER) {
-        /* RNA puzzler behavior specification */
-        vrna_plot_options_puzzler_t *puzzler;
-        vrna_plot_layout_t          *layout;
-
-        puzzler           = vrna_plot_options_puzzler();
-        puzzler->filename = ffname;
-        puzzler->drawArcs = 1;
-
-        puzzler->checkAncestorIntersections = opt->checkAncestorIntersections;
-        puzzler->checkSiblingIntersections  = opt->checkSiblingIntersections;
-        puzzler->checkExteriorIntersections = opt->checkExteriorIntersections;
-        puzzler->allowFlipping              = opt->allowFlipping;
-        puzzler->optimize                   = opt->optimize;
-
-        layout = vrna_plot_layout_puzzler(structure,
-                                          puzzler);
-
-        THREADSAFE_FILE_OUTPUT(
-          vrna_file_PS_rnaplot_layout(rec_sequence,
-                                      structure,
-                                      ffname,
-                                      opt->pre,
-                                      opt->post,
-                                      &(opt->md),
-                                      layout));
-
-        vrna_plot_layout_free(layout);
-        vrna_plot_options_puzzler_free(puzzler);
+    case 's':
+      if (opt->format[1] == 'v') { /* svg */
+        tmp_string = vrna_strdup_printf("%s.svg", ffname);
+        free(ffname);
+        ffname = vrna_filename_sanitize(tmp_string, opt->filename_delim);
+        free(tmp_string);
+        file_format = VRNA_FILE_FORMAT_SVG;
       } else {
-        THREADSAFE_FILE_OUTPUT(
-          vrna_file_PS_rnaplot_a(rec_sequence,
-                                 structure,
-                                 ffname,
-                                 opt->pre,
-                                 opt->post,
-                                 &(opt->md)));
+        tmp_string = vrna_strdup_printf("%s.ssv", ffname);
+        free(ffname);
+        ffname = vrna_filename_sanitize(tmp_string, opt->filename_delim);
+        free(tmp_string);
+        file_format = VRNA_FILE_FORMAT_SSV;
       }
 
       break;
@@ -624,10 +627,8 @@ process_record(struct record_data *record)
       free(ffname);
       ffname = vrna_filename_sanitize(tmp_string, opt->filename_delim);
       free(tmp_string);
+      file_format = VRNA_FILE_FORMAT_GML;
 
-      THREADSAFE_FILE_OUTPUT(
-        gmlRNA(rec_sequence, structure, ffname, 'x')
-        );
       break;
 
     case 'x':
@@ -635,28 +636,59 @@ process_record(struct record_data *record)
       free(ffname);
       ffname = vrna_filename_sanitize(tmp_string, opt->filename_delim);
       free(tmp_string);
+      file_format = VRNA_FILE_FORMAT_XRNA;
 
-      THREADSAFE_FILE_OUTPUT(
-        xrna_plot(rec_sequence, structure, ffname)
-        );
-      break;
-
-    case 's':
-      tmp_string = vrna_strdup_printf("%s.svg", ffname);
-      free(ffname);
-      ffname = vrna_filename_sanitize(tmp_string, opt->filename_delim);
-      free(tmp_string);
-
-      THREADSAFE_FILE_OUTPUT(
-        svg_rna_plot(rec_sequence, structure, ffname)
-        );
       break;
 
     default:
-      RNAplot_cmdline_parser_print_help();
-      exit(EXIT_FAILURE);
+      tmp_string = vrna_strdup_printf("%s.eps", ffname);
+      free(ffname);
+      ffname = vrna_filename_sanitize(tmp_string, opt->filename_delim);
+      free(tmp_string);
+      file_format = VRNA_FILE_FORMAT_EPS;
+
+      break;
+
   }
 
+  if ((opt->pre) || (opt->post)) {
+    data = (vrna_plot_data_t *)vrna_alloc(sizeof(vrna_plot_data_t));
+    data->pre     = opt->pre;
+    data->post    = opt->post;
+    data->md      = &(opt->md);
+    data->options = 0U;
+  }
+
+  if (opt->plot_type == VRNA_PLOT_TYPE_PUZZLER) {
+    /* RNA puzzler behavior specification */
+
+    puzzler           = vrna_plot_options_puzzler();
+    puzzler->filename = ffname;
+    puzzler->drawArcs = 1;
+
+    puzzler->checkAncestorIntersections = opt->checkAncestorIntersections;
+    puzzler->checkSiblingIntersections  = opt->checkSiblingIntersections;
+    puzzler->checkExteriorIntersections = opt->checkExteriorIntersections;
+    puzzler->allowFlipping              = opt->allowFlipping;
+    puzzler->optimize                   = opt->optimize;
+
+    layout = vrna_plot_layout_puzzler(structure,
+                                      puzzler);
+  } else {
+    layout = vrna_plot_layout(structure, opt->plot_type);
+  }
+
+  THREADSAFE_FILE_OUTPUT(
+    vrna_plot_structure(ffname,
+                        rec_sequence,
+                        structure,
+                        file_format,
+                        layout,
+                        data));
+
+  vrna_plot_layout_free(layout);
+  vrna_plot_options_puzzler_free(puzzler);
+
   /* clean up */
   free(record->id);
   free(record->SEQ_ID);
@@ -681,12 +713,14 @@ process_record(struct record_data *record)
 static void
 process_alignment_record(struct record_data_msa *record)
 {
-  char            *consensus_sequence, *structure, *ffname, *tmp_string, **A,
+  char            *consensus_sequence, *structure, *ffname, *tmp_string,
                   *pre, *post;
   struct options  *opt;
 
-  if (!record->structure)
-    vrna_message_error("structure missing for record %d\n", record->number);
+  if (!record->structure) {
+    vrna_log_error("structure missing for record %d\n", record->number);
+    exit(EXIT_FAILURE);
+  }
 
   opt       = record->options;
   structure = vrna_db_from_WUSS(record->structure);
@@ -709,7 +743,15 @@ process_alignment_record(struct record_data_msa *record)
     ffname = vrna_strdup_printf("alirna");
 
   if (opt->annotate_covar) {
-    A = vrna_annotate_covar_db((const char **)record->alignment, structure, &(opt->md));
+    vrna_string_t *A;
+
+    short int *pt = vrna_ptable(structure);
+
+    A = vrna_annotate_covar_pt((const char **)record->alignment,
+                               pt,
+                               &(opt->md),
+                               opt->covar_threshold,
+                               opt->covar_min_sat);
 
     if (pre)
       pre = vrna_strdup_printf("%s\n%s", pre, A[0]);
@@ -721,8 +763,10 @@ process_alignment_record(struct record_data_msa *record)
     else
       post = vrna_strdup_printf("%s", A[1]);
 
-    free(A[0]);
-    free(A[1]);
+
+    free(pt);
+    vrna_string_free(A[0]);
+    vrna_string_free(A[1]);
     free(A);
   }
 
@@ -790,11 +834,18 @@ process_alignment_record(struct record_data_msa *record)
       ffname = vrna_strdup_printf("aln.ps");
 
     THREADSAFE_FILE_OUTPUT(
-      vrna_file_PS_aln(ffname,
-                       (const char **)record->alignment,
-                       (const char **)record->names,
-                       structure,
-                       opt->aln_PS_cols)
+      vrna_file_PS_aln_opt(ffname,
+                           (const char **)record->alignment,
+                           (const char **)record->names,
+                           structure,
+                           (vrna_aln_opt_t){
+                            .start = 0,
+                            .end = 0,
+                            .offset = 0,
+                            .columns = opt->aln_PS_cols,
+                            .color_threshold = opt->covar_threshold,
+                            .color_min_sat = opt->covar_min_sat
+                           })
       );
   }
 
