@@ -20,16 +20,20 @@
 #include "ViennaRNA/fold.h"
 #include "ViennaRNA/params/default.h"
 #include "ViennaRNA/snoop.h"
-#include "ViennaRNA/part_func.h"
+#include "ViennaRNA/partfunc/global.h"
 #include "ViennaRNA/fold_vars.h"
 #include "ViennaRNA/plotting/alignments.h"
 #include "ViennaRNA/plotting/probabilities.h"
 #include "ViennaRNA/plotting/structures.h"
 #include "ViennaRNA/utils/basic.h"
 #include "ViennaRNA/utils/strings.h"
-#include "ViennaRNA/utils/alignments.h"
+#include "ViennaRNA/sequences/alignments.h"
+#include "ViennaRNA/utils/log.h"
 #include "ViennaRNA/io/utils.h"
 #include "RNAsnoop_cmdl.h"
+
+#include "gengetopt_helpers.h"
+
 static void
 aliprint_struc(snoopT     *dup,
                const char **s1,
@@ -138,13 +142,14 @@ main(int  argc,
                                     distance /*distance between two subopts*/,
                                     min_d1 /*minimal distance between 5' sno and first duplex*/,
                                     min_d2 /*minimal distance between 3' sno and second duplex*/,
-                                    max_asymm /*maximal asymmetry in the stem interior loop*/,
+                                    max_asymm /*maximal asymmetry in the stem internal loop*/,
                                     alignment_length /*maximal target RNA alignment length*/,
                                     alignment /*flag to use ali version*/,
                                     delta,
                                     redraw /*if used (option I) allow to redraw command line output into ps files */;
 
   int noconv = 0;
+  int verbose;
 
   string_s            = NULL;
   string_t            = NULL;
@@ -162,6 +167,9 @@ main(int  argc,
   if (RNAsnoop_cmdline_parser(argc, argv, &args_info) != 0)
     exit(1);
 
+  /* prepare logging system and verbose mode */
+  ggo_log_settings(args_info, verbose);
+
   /* Redraw results from RNAsnoop */
   if (args_info.constraint_given)
     fold_constrained = 1;
@@ -729,7 +737,8 @@ main(int  argc,
         free(temp1[i]);
         free(temp2[i]);
       }
-      vrna_message_error("unequal number of seqs in alignments");
+      vrna_log_error("unequal number of seqs in alignments");
+      goto exit_fail;
     }
 
     for (i = 0; temp1[i]; i++) {
@@ -809,7 +818,14 @@ main(int  argc,
 
   fclose(sno);
   fclose(mrna);
-  return 0;
+
+  if (vrna_log_fp() != stderr)
+    fclose(vrna_log_fp());
+
+  return EXIT_SUCCESS;
+
+  exit_fail:
+    return EXIT_FAILURE;
 }
 
 
@@ -1097,7 +1113,7 @@ read_rnaup(char       *fname,
     perror("Empty File");
 
   if (strchr(tmp, '>')) {
-    vrna_message_error("file %s is not in RNAup format", fname);
+    vrna_log_error("file %s is not in RNAup format", fname);
     exit(EXIT_FAILURE);
   }
 
@@ -1184,7 +1200,7 @@ read_plfold_i(char      *fname,
     perror("Empty File");
 
   if (strchr(tmp, '>')) {
-    vrna_message_error("file %s is not in RNAplfold format", fname);
+    vrna_log_error("file %s is not in RNAplfold format", fname);
     exit(EXIT_FAILURE);
   }
 
@@ -1405,7 +1421,8 @@ redraw_output(char  *fname,
         free(temp1[i]);
         free(temp2[i]);
       }
-      vrna_message_error("unequal number of seqs in alignments");
+      vrna_log_error("unequal number of seqs in alignments");
+      exit(EXIT_FAILURE);
     }
 
     for (i = 0; temp1[i]; i++) {
