@@ -25,7 +25,8 @@
 #include "ViennaRNA/plotting/probabilities.h"
 #include "ViennaRNA/utils/basic.h"
 #include "ViennaRNA/utils/strings.h"
-#include "ViennaRNA/part_func.h"
+#include "ViennaRNA/utils/log.h"
+#include "ViennaRNA/partfunc/global.h"
 #include "ViennaRNA/part_func_up.h"
 #include "ViennaRNA/duplex.h"
 #include "ViennaRNA/params/constants.h"
@@ -41,7 +42,7 @@
 #define MAX(x, y) (((x) > (y)) ? (x) : (y))
 #define EQUAL(A, B) (fabs((A)-(B)) < 1000 * DBL_EPSILON)
 
-PRIVATE void
+PRIVATE int
 tokenize(char *line,
          char **seq1,
          char **seq2);
@@ -65,7 +66,7 @@ print_interaction(interact    *Int,
                   int         incr5);
 
 
-PRIVATE void
+PRIVATE int
 print_unstru(pu_contrib *p_c,
              int        w);
 
@@ -104,7 +105,7 @@ main(int  argc,
                           *orig_s2,
                           *orig_target;
   int         i, j, length1, length2, length_target, sym, istty,
-              noconv, max_u, **unpaired_values, ulength_num;
+              noconv, max_u, **unpaired_values, ulength_num, verbose;
   double      min_en, sfact;
 
   /* variables for output */
@@ -154,6 +155,9 @@ main(int  argc,
   if (RNAup_cmdline_parser(argc, argv, &args_info) != 0)
     exit(1);
 
+  /* prepare logging system and verbose mode */
+  ggo_log_settings(args_info, verbose);
+
   /* do not create header */
   if (args_info.no_header_given)
     header = 0;
@@ -184,7 +188,7 @@ main(int  argc,
   /* set dangle model */
   if (args_info.dangles_given) {
     if ((args_info.dangles_arg != 0) && (args_info.dangles_arg != 2))
-      vrna_message_warning(
+      vrna_log_warning(
         "required dangle model not implemented, falling back to default dangles=2");
     else
       dangles = args_info.dangles_arg;
@@ -457,7 +461,8 @@ main(int  argc,
 
           break;
         default:
-          vrna_message_error("This should never happen (again)");
+          vrna_log_error("This should never happen (again)");
+          goto exit_fail;
           break;
       }
     }
@@ -470,11 +475,13 @@ main(int  argc,
     }
 
     /* break on any error, EOF or quit request */
-    if (input_type & (VRNA_INPUT_QUIT | VRNA_INPUT_ERROR))
+    if (input_type & (VRNA_INPUT_QUIT | VRNA_INPUT_ERROR)) {
       break;                              /* else assume a proper sequence of letters of a certain alphabet (RNA, DNA, etc.) */
-    else
-      tokenize(input_string, &s1, &s2);   /* this also frees the input_string */
-
+    } else {
+      int ret = tokenize(input_string, &s1, &s2);   /* this also frees the input_string */
+      if (ret == 0)
+        goto exit_fail;
+    }
     length1 = (int)strlen(s1);
     length2 = (s2) ? (int)strlen(s2) : 0;
 
@@ -482,7 +489,7 @@ main(int  argc,
 
     /* check if we have to change the mode we are operating in */
     if ((cut_point != -1) && (up_mode & RNA_UP_MODE_1)) {
-      vrna_message_warning(
+      vrna_log_warning(
         "Two concatenated sequences given, switching to pairwise interaction mode!");
       up_mode = RNA_UP_MODE_2;
     }
@@ -500,10 +507,11 @@ main(int  argc,
           if (s_target == NULL)
             read_again = 1;
         } else if (s_target != NULL) {
-          vrna_message_error(
+          vrna_log_error(
             "After the first sequence (pair): Input a single sequence (no &)!\n"
             "Each input seq. is compared to the very first seq. given.\n"
             );
+          goto exit_fail;
         }
 
         break;
@@ -521,15 +529,18 @@ main(int  argc,
         free(input_string);
       }
       /* break on any error, EOF or quit request */
-      if (input_type & (VRNA_INPUT_QUIT | VRNA_INPUT_ERROR))
+      if (input_type & (VRNA_INPUT_QUIT | VRNA_INPUT_ERROR)) {
         break;                              /* else assume a proper sequence of letters of a certain alphabet (RNA, DNA, etc.) */
-      else
-        tokenize(input_string, &s2, &s3);   /* this also frees the input_string */
-
-      if (cut_point != -1)
-        vrna_message_error(
+      } else {
+        int ret = tokenize(input_string, &s2, &s3);   /* this also frees the input_string */
+        if (ret == 0)
+          goto exit_fail;
+      }
+      if (cut_point != -1) {
+        vrna_log_error(
           "Don't confuse me by mixing concatenated (&) with single sequences! Go, have some sleep and then check your input again...");
-
+        goto exit_fail;
+      }
       length2 = (int)strlen(s2);
     }
 
@@ -559,16 +570,20 @@ main(int  argc,
       } else if ((input_type & VRNA_INPUT_MISC) && (strlen(input_string) > 0)) {
         old_cut   = cut_point;
         cut_point = -1;
-        tokenize(input_string, &cstruc1, &cstruc2);
+        int ret = tokenize(input_string, &cstruc1, &cstruc2);
+        if (ret == 0)
+          goto exit_fail;
       } else {
-        vrna_message_error("constraints missing");
+        vrna_log_error("constraints missing");
+        goto exit_fail;
       }
 
       /* now that we've got the constraining structure(s) check if the input was valid */
-      if (old_cut != cut_point)
-        vrna_message_error(
+      if (old_cut != cut_point) {
+        vrna_log_error(
           "RNAup -C: mixed single/dual sequence or constraint strings or different cut points");
-
+        goto exit_fail;
+      }
       read_again = 0;
 
       if (cut_point == -1) {
@@ -580,28 +595,35 @@ main(int  argc,
 
       if (read_again) {
         input_type = get_input_line(&input_string, VRNA_INPUT_NOSKIP_COMMENTS);
-        if (input_type & VRNA_INPUT_QUIT)
+        if (input_type & VRNA_INPUT_QUIT) {
           break;
-        else if ((input_type & VRNA_INPUT_MISC) && (strlen(input_string) > 0))
-          tokenize(input_string, &cstruc2, &cstruc_tmp);
-        else
-          vrna_message_error("constraints missing");
-
-        if (cut_point != -1)
-          vrna_message_error(
+        } else if ((input_type & VRNA_INPUT_MISC) && (strlen(input_string) > 0)) {
+          int ret = tokenize(input_string, &cstruc2, &cstruc_tmp);
+          if (ret == 0)
+            goto exit_fail;
+        } else {
+          vrna_log_error("constraints missing");
+          goto exit_fail;
+        }
+        if (cut_point != -1) {
+          vrna_log_error(
             "Don't confuse me by mixing concatenated (&) with single sequences! Go, have some sleep and then check your input again...");
+          goto exit_fail;
+        }
       }
 
       /* check length(s) of input sequence(s) and constraint(s) */
       if (strlen(cstruc1) != length1) {
         fprintf(stderr, "%s\n%s\n", s1, cstruc1);
-        vrna_message_error("RNAup -C: constraint string and structure have unequal length");
+        vrna_log_error("RNAup -C: constraint string and structure have unequal length");
+        goto exit_fail;
       }
 
       if (s2 != NULL) {
         if (strlen(cstruc2) != length2) {
           fprintf(stderr, "%s\n%s\n", s2, cstruc2);
-          vrna_message_error("RNAup -C: constraint string and structure have unequal length");
+          vrna_log_error("RNAup -C: constraint string and structure have unequal length");
+          goto exit_fail;
         }
       }
     } /* thats all for constraint folding */
@@ -634,8 +656,10 @@ main(int  argc,
     }
 
     /* check ulength values against sequences given */
-    if (max_u > length1)
-      vrna_message_error("maximum unpaired region exceeds sequence length");
+    if (max_u > length1) {
+      vrna_log_error("maximum unpaired region exceeds sequence length");
+      goto exit_fail;
+    }
 
     if (up_mode & RNA_UP_MODE_3) {
       /* if we haven't seen the target yet, store it now */
@@ -720,7 +744,7 @@ main(int  argc,
     min_en    = fold(s1, structure);
     pf_scale  = exp(-(sfact * min_en) / RT / length1);
     if (length1 > 2000)
-      vrna_message_info(stderr, "scaling factor %f", pf_scale);
+      vrna_log_info("scaling factor %f", pf_scale);
 
     if (cstruc1 != NULL)
       strncpy(structure, cstruc1, length1 + 1);
@@ -748,9 +772,13 @@ main(int  argc,
       case RNA_UP_MODE_1:
         for (i = 1; i <= unpaired_values[0][0]; i++) {
           j = unpaired_values[i][0];
-          do
-            print_unstru(unstr_out, j);
-          while (++j <= unpaired_values[i][1]);
+          int ret;
+          do {
+            ret = print_unstru(unstr_out, j);
+          } while ((++j <= unpaired_values[i][1]) &&
+                   (ret != 0));
+          if (ret == 0)
+            goto exit_fail;
         }
         if (output && header)
           head =
@@ -788,7 +816,7 @@ main(int  argc,
           min_en    = fold(s_target, structure);
           pf_scale  = exp(-(sfact * min_en) / RT / length_target);
           if (length_target > 2000)
-            vrna_message_info(stderr, "scaling factor %f", pf_scale);
+            vrna_log_info("scaling factor %f", pf_scale);
 
           if (cstruc_target != NULL)
             strncpy(structure, cstruc_target, length_target + 1);
@@ -908,7 +936,13 @@ main(int  argc,
   } while (1);
   free(cmdl_parameters);
 
+  if (vrna_log_fp() != stderr)
+    fclose(vrna_log_fp());
+
   return EXIT_SUCCESS;
+
+  exit_fail:
+    return EXIT_FAILURE;
 }
 
 
@@ -1008,7 +1042,7 @@ adjustUnpairedValues(int ***unpaired_values)
 /* call:  tokenize(line,&seq1,&seq2); the sequence string is split at the "&"
  * and the first seq is written in seq1, the second into seq2  */
 /* using sscanf instead of strcpy get's rid of trainling junk on the input line */
-void
+int
 tokenize(char *line,
          char **seq1,
          char **seq2)
@@ -1022,8 +1056,10 @@ tokenize(char *line,
     (*seq1) = (char *)vrna_alloc((cut + 1) * sizeof(char));
     (*seq2) = (char *)vrna_alloc(((strlen(line) - cut) + 2) * sizeof(char));
 
-    if (strchr(pos + 1, '&'))
-      vrna_message_error("more than one cut-point in input");
+    if (strchr(pos + 1, '&')) {
+      vrna_log_error("more than one cut-point in input");
+      return 0;
+    }
 
     *pos = '\0';
     (void)sscanf(line, "%s", *seq1);
@@ -1039,12 +1075,13 @@ tokenize(char *line,
       cut_point = cut;
     } else if (cut_point != cut) {
       fprintf(stderr, "cut_point = %d cut = %d\n", cut_point, cut);
-      vrna_message_error("Sequence and Structure have different cut points.");
+      vrna_log_error("Sequence and Structure have different cut points.");
+      return 0;
     }
   }
 
   free(line);
-  return;
+  return 1;
 }
 
 
@@ -1291,7 +1328,7 @@ print_interaction(interact    *Int,
   }
 
   if (nix_up)
-    vrna_message_warning(
+    vrna_log_warning(
       "RNAduplex structure doesn't match any structure of RNAup structure ensemble");
 
   free(i_long);
@@ -1302,7 +1339,7 @@ print_interaction(interact    *Int,
 
 
 /* print coordinates and free energy for the region of highest accessibility */
-PRIVATE void
+PRIVATE int
 print_unstru(pu_contrib *p_c,
              int        w)
 {
@@ -1331,7 +1368,9 @@ print_unstru(pu_contrib *p_c,
       }
     }
     printf("%4d,%4d \t (%.3f) \t for u=%3d\n", min_i, min_j, min_gu, w);
+    return 1;
   } else {
-    vrna_message_error("error with prob unpaired");
+    vrna_log_error("error with prob unpaired");
+    return 0;
   }
 }
