@@ -1,10 +1,13 @@
 #include <math.h>
 #include <stdio.h>
 #include <unistd.h>
+#include <stdlib.h>
+#include <errno.h>
 
 #include <ViennaRNA/io/file_formats.h>
 #include <ViennaRNA/constraints/basic.h>
-#include <ViennaRNA/constraints/SHAPE.h>
+#include <ViennaRNA/probing/basic.h>
+#include <ViennaRNA/probing/SHAPE.h>
 
 static int
 deltaCompare(double a,
@@ -18,17 +21,30 @@ deltaCompare(double a,
 }
 
 
-static void
-writeTempFile(char        *tempfile,
+static int
+writeTempFile(char        tempfile[42],
               const char  *data)
 {
   FILE *f;
+  int   fd;
+  errno = 0;
 
-  ck_assert(tmpnam(tempfile) != NULL);
-  f = fopen(tempfile, "w");
-  ck_assert(f != NULL);
-  fputs(data, f);
-  fclose(f);
+  /* set tempfile buffer to 0 */
+  memset(tempfile, 0, 42);
+  strncpy(tempfile, "/tmp/vrna-test-XXXXXX", 22);
+
+
+  /* Create the temporary file, this function will replace the 'X's */
+  if ((fd = mkstemp(tempfile)) < 1) {
+    fprintf(stderr,
+            "Creation of temp file failed with error [%s]\n",
+            strerror(errno));
+  } else {
+    write(fd, data, sizeof(char) * strlen(data));
+    close(fd);
+  }
+
+  return (fd > 0) ? 1 : 0;
 }
 
 
@@ -176,151 +192,168 @@ writeTempFile(char        *tempfile,
 
 #test test_vrna_file_SHAPE_read
 {
-  char          tempfile[L_tmpnam + 1];
+  char          tempfile[42];
   const size_t  len = 5;
   char          sequence[len];
   double        values[len];
   int           ret;
 
+
   //1 entry
-  writeTempFile(tempfile, "1 A 0.5\n");
-  ret = vrna_file_SHAPE_read(tempfile, 1, 0, sequence, values);
-  ck_assert_int_eq(ret, 1);
-  ck_assert_str_eq(sequence, "A");
-  ck_assert(deltaCompare(values[1], 0.5));
-  unlink(tempfile);
+  if (writeTempFile(tempfile, "1 A 0.5\n")) {
+    ret = vrna_file_SHAPE_read(tempfile, 1, 0, sequence, values);
+    ck_assert_int_eq(ret, 1);
+    ck_assert_str_eq(sequence, "A");
+    ck_assert(deltaCompare(values[1], 0.5));
+    unlink(tempfile);
+  }
 
   //1 entry 2 columns
-  writeTempFile(tempfile, "1 0.5\n");
-  ret = vrna_file_SHAPE_read(tempfile, 1, 0, sequence, values);
-  ck_assert_int_eq(ret, 1);
-  ck_assert_str_eq(sequence, "N");
-  ck_assert(deltaCompare(values[1], 0.5));
-  unlink(tempfile);
+  if (writeTempFile(tempfile, "1 0.5\n")) {
+    ret = vrna_file_SHAPE_read(tempfile, 1, 0, sequence, values);
+    ck_assert_int_eq(ret, 1);
+    ck_assert_str_eq(sequence, "N");
+    ck_assert(deltaCompare(values[1], 0.5));
+    unlink(tempfile);
+  }
 
   //multiple entries
-  writeTempFile(tempfile, "1 A 0.1\n2 T 0.2\n3 G 0.3\n4 C 0.4\n");
-  ret = vrna_file_SHAPE_read(tempfile, 4, 0, sequence, values);
-  ck_assert_int_eq(ret, 1);
-  ck_assert_str_eq(sequence, "ATGC");
-  ck_assert(deltaCompare(values[1], 0.1));
-  ck_assert(deltaCompare(values[2], 0.2));
-  ck_assert(deltaCompare(values[3], 0.3));
-  ck_assert(deltaCompare(values[4], 0.4));
-  unlink(tempfile);
+  if (writeTempFile(tempfile, "1 A 0.1\n2 T 0.2\n3 G 0.3\n4 C 0.4\n")) {
+    ret = vrna_file_SHAPE_read(tempfile, 4, 0, sequence, values);
+    ck_assert_int_eq(ret, 1);
+    ck_assert_str_eq(sequence, "ATGC");
+    ck_assert(deltaCompare(values[1], 0.1));
+    ck_assert(deltaCompare(values[2], 0.2));
+    ck_assert(deltaCompare(values[3], 0.3));
+    ck_assert(deltaCompare(values[4], 0.4));
+    unlink(tempfile);
+  }
 
   //value formats
-  writeTempFile(tempfile, "1 A 1\n2 T 2.\n3 G .3\n4 C 1e-1\n");
-  ret = vrna_file_SHAPE_read(tempfile, 4, 0, sequence, values);
-  ck_assert_int_eq(ret, 1);
-  ck_assert_str_eq(sequence, "ATGC");
-  ck_assert(deltaCompare(values[1], 1));
-  ck_assert(deltaCompare(values[2], 2));
-  ck_assert(deltaCompare(values[3], 0.3));
-  ck_assert(deltaCompare(values[4], 0.1));
-  unlink(tempfile);
+  if (writeTempFile(tempfile, "1 A 1\n2 T 2.\n3 G .3\n4 C 1e-1\n")) {
+    ret = vrna_file_SHAPE_read(tempfile, 4, 0, sequence, values);
+    ck_assert_int_eq(ret, 1);
+    ck_assert_str_eq(sequence, "ATGC");
+    ck_assert(deltaCompare(values[1], 1));
+    ck_assert(deltaCompare(values[2], 2));
+    ck_assert(deltaCompare(values[3], 0.3));
+    ck_assert(deltaCompare(values[4], 0.1));
+    unlink(tempfile);
+  }
 
   //whitespaces
-  writeTempFile(tempfile, "1 \t 0.5\n2    A\t1\n");
-  ret = vrna_file_SHAPE_read(tempfile, 2, 0, sequence, values);
-  ck_assert_int_eq(ret, 1);
-  ck_assert_str_eq(sequence, "NA");
-  ck_assert(deltaCompare(values[1], 0.5));
-  ck_assert(deltaCompare(values[2], 1));
-  unlink(tempfile);
+  if (writeTempFile(tempfile, "1 \t 0.5\n2    A\t1\n")) {
+    ret = vrna_file_SHAPE_read(tempfile, 2, 0, sequence, values);
+    ck_assert_int_eq(ret, 1);
+    ck_assert_str_eq(sequence, "NA");
+    ck_assert(deltaCompare(values[1], 0.5));
+    ck_assert(deltaCompare(values[2], 1));
+    unlink(tempfile);
+  }
 
   //missing value
-  writeTempFile(tempfile, "1 A\n");
-  ret = vrna_file_SHAPE_read(tempfile, 1, 123, sequence, values);
-  ck_assert_int_eq(ret, 1);
-  ck_assert_str_eq(sequence, "A");
-  ck_assert(deltaCompare(values[1], 123));
-  unlink(tempfile);
+  if (writeTempFile(tempfile, "1 A\n")) {
+    ret = vrna_file_SHAPE_read(tempfile, 1, 123, sequence, values);
+    ck_assert_int_eq(ret, 1);
+    ck_assert_str_eq(sequence, "A");
+    ck_assert(deltaCompare(values[1], 123));
+    unlink(tempfile);
+  }
 
   //missing nucleotide & value
-  writeTempFile(tempfile, "1\n");
-  ret = vrna_file_SHAPE_read(tempfile, 1, 123, sequence, values);
-  ck_assert_int_eq(ret, 1);
-  ck_assert_str_eq(sequence, "N");
-  ck_assert(deltaCompare(values[1], 123));
-  unlink(tempfile);
+  if (writeTempFile(tempfile, "1\n")) {
+    ret = vrna_file_SHAPE_read(tempfile, 1, 123, sequence, values);
+    ck_assert_int_eq(ret, 1);
+    ck_assert_str_eq(sequence, "N");
+    ck_assert(deltaCompare(values[1], 123));
+    unlink(tempfile);
+  }
 
   //upper limit
-  writeTempFile(tempfile, "1 A 0.1\n2 T 0.2\n3 G 0.3\n4 C 0.4\n");
-  ret = vrna_file_SHAPE_read(tempfile, 3, 0, sequence, values);
-  ck_assert_int_eq(ret, 0);
-  unlink(tempfile);
+  if (writeTempFile(tempfile, "1 A 0.1\n2 T 0.2\n3 G 0.3\n4 C 0.4\n")) {
+    ret = vrna_file_SHAPE_read(tempfile, 3, 0, sequence, values);
+    ck_assert_int_eq(ret, 0);
+    unlink(tempfile);
+  }
 
   //lower limit
-  writeTempFile(tempfile, "0 A 0.1\n1 T 0.2\n2 G 0.3\n3 C 0.4\n");
-  ret = vrna_file_SHAPE_read(tempfile, 4, 0, sequence, values);
-  ck_assert_int_eq(ret, 0);
-  unlink(tempfile);
+  if (writeTempFile(tempfile, "0 A 0.1\n1 T 0.2\n2 G 0.3\n3 C 0.4\n")) {
+    ret = vrna_file_SHAPE_read(tempfile, 4, 0, sequence, values);
+    ck_assert_int_eq(ret, 0);
+    unlink(tempfile);
+  }
 
   //unordered
-  writeTempFile(tempfile, "3 G 0.3\n2 T 0.2\n4 C 0.4\n1 A 0.1\n");
-  ret = vrna_file_SHAPE_read(tempfile, 4, 0, sequence, values);
-  ck_assert_int_eq(ret, 1);
-  ck_assert_str_eq(sequence, "ATGC");
-  ck_assert(deltaCompare(values[1], 0.1));
-  ck_assert(deltaCompare(values[2], 0.2));
-  ck_assert(deltaCompare(values[3], 0.3));
-  ck_assert(deltaCompare(values[4], 0.4));
-  unlink(tempfile);
+  if (writeTempFile(tempfile, "3 G 0.3\n2 T 0.2\n4 C 0.4\n1 A 0.1\n")) {
+    ret = vrna_file_SHAPE_read(tempfile, 4, 0, sequence, values);
+    ck_assert_int_eq(ret, 1);
+    ck_assert_str_eq(sequence, "ATGC");
+    ck_assert(deltaCompare(values[1], 0.1));
+    ck_assert(deltaCompare(values[2], 0.2));
+    ck_assert(deltaCompare(values[3], 0.3));
+    ck_assert(deltaCompare(values[4], 0.4));
+    unlink(tempfile);
+  }
 
   //missing indices middle
-  writeTempFile(tempfile, "1 A 0.1\n4 C 0.4\n");
-  ret = vrna_file_SHAPE_read(tempfile, 4, 123, sequence, values);
-  ck_assert_int_eq(ret, 1);
-  ck_assert_str_eq(sequence, "ANNC");
-  ck_assert(deltaCompare(values[1], 0.1));
-  ck_assert(deltaCompare(values[2], 123));
-  ck_assert(deltaCompare(values[3], 123));
-  ck_assert(deltaCompare(values[4], 0.4));
-  unlink(tempfile);
+  if (writeTempFile(tempfile, "1 A 0.1\n4 C 0.4\n")) {
+    ret = vrna_file_SHAPE_read(tempfile, 4, 123, sequence, values);
+    ck_assert_int_eq(ret, 1);
+    ck_assert_str_eq(sequence, "ANNC");
+    ck_assert(deltaCompare(values[1], 0.1));
+    ck_assert(deltaCompare(values[2], 123));
+    ck_assert(deltaCompare(values[3], 123));
+    ck_assert(deltaCompare(values[4], 0.4));
+    unlink(tempfile);
+  }
 
   //missing indices start end
-  writeTempFile(tempfile, "2 T 0.2\n3 G 0.3\n");
-  ret = vrna_file_SHAPE_read(tempfile, 4, 123, sequence, values);
-  ck_assert_int_eq(ret, 1);
-  ck_assert_str_eq(sequence, "NTGN");
-  ck_assert(deltaCompare(values[1], 123));
-  ck_assert(deltaCompare(values[2], 0.2));
-  ck_assert(deltaCompare(values[3], 0.3));
-  ck_assert(deltaCompare(values[4], 123));
-  unlink(tempfile);
+  if (writeTempFile(tempfile, "2 T 0.2\n3 G 0.3\n")) {
+    ret = vrna_file_SHAPE_read(tempfile, 4, 123, sequence, values);
+    ck_assert_int_eq(ret, 1);
+    ck_assert_str_eq(sequence, "NTGN");
+    ck_assert(deltaCompare(values[1], 123));
+    ck_assert(deltaCompare(values[2], 0.2));
+    ck_assert(deltaCompare(values[3], 0.3));
+    ck_assert(deltaCompare(values[4], 123));
+    unlink(tempfile);
+  }
 
   //invalid file
   ret = vrna_file_SHAPE_read(NULL, 0, 0, sequence, values);
   ck_assert_int_eq(ret, 0);
 
   //missing linebreak
-  writeTempFile(tempfile, "1 A 0.5");
-  ret = vrna_file_SHAPE_read(tempfile, 1, 0, sequence, values);
-  ck_assert_int_eq(ret, 1);
-  ck_assert_str_eq(sequence, "A");
-  ck_assert(deltaCompare(values[1], 0.5));
-  unlink(tempfile);
+  if (writeTempFile(tempfile, "1 A 0.5")) {
+    ret = vrna_file_SHAPE_read(tempfile, 1, 0, sequence, values);
+    ck_assert_int_eq(ret, 1);
+    ck_assert_str_eq(sequence, "A");
+    ck_assert(deltaCompare(values[1], 0.5));
+    unlink(tempfile);
+  }
 
   //garbage + entry
-  writeTempFile(tempfile, "\nblablabla\n#evil_comment 123\n1 A 0.5\n\ngarbage\n");
-  ret = vrna_file_SHAPE_read(tempfile, 1, 0, sequence, values);
-  ck_assert_int_eq(ret, 1);
-  ck_assert_str_eq(sequence, "A");
-  ck_assert(deltaCompare(values[1], 0.5));
-  unlink(tempfile);
+  if (writeTempFile(tempfile, "\nblablabla\n#evil_comment 123\n1 A 0.5\n\ngarbage\n")) {
+    ret = vrna_file_SHAPE_read(tempfile, 1, 0, sequence, values);
+    ck_assert_int_eq(ret, 1);
+    ck_assert_str_eq(sequence, "A");
+    ck_assert(deltaCompare(values[1], 0.5));
+    unlink(tempfile);
+  }
 
   //garbage only
-  writeTempFile(tempfile, "\nblablabla\n\ngarbage\n");
-  ret = vrna_file_SHAPE_read(tempfile, 1, 0, sequence, values);
-  ck_assert_int_eq(ret, 0);
-  unlink(tempfile);
+  if (writeTempFile(tempfile, "\nblablabla\n\ngarbage\n")) {
+    ret = vrna_file_SHAPE_read(tempfile, 1, 0, sequence, values);
+    ck_assert_int_eq(ret, 0);
+    unlink(tempfile);
+  }
 
   //empty file
-  writeTempFile(tempfile, "");
-  ret = vrna_file_SHAPE_read(tempfile, 1, 0, sequence, values);
-  ck_assert_int_eq(ret, 0);
-  unlink(tempfile);
+  if (writeTempFile(tempfile, "")) {
+    ret = vrna_file_SHAPE_read(tempfile, 1, 0, sequence, values);
+    ck_assert_int_eq(ret, 0);
+    unlink(tempfile);
+  }
 }
 
 #test test_vrna_sc_SHAPE_parse_method
